Package: affy
Version: 1.90.0
Title: Methods for Affymetrix Oligonucleotide Arrays
Author: Rafael A. Irizarry <rafa@ds.dfci.harvard.edu>, Laurent Gautier
        <lgautier@gmail.com>, Benjamin Milo Bolstad
        <bmb@bmbolstad.com>, and Crispin Miller
        <cmiller@picr.man.ac.uk> with contributions from Magnus
        Astrand <Magnus.Astrand@astrazeneca.com>, Leslie M. Cope
        <cope@mts.jhu.edu>, Robert Gentleman, Jeff Gentry, Conrad
        Halling <challing@agilixcorp.com>, Wolfgang Huber, James
        MacDonald <jmacdon@u.washington.edu>, Benjamin I. P. Rubinstein,
        Christopher Workman <workman@cbs.dtu.dk>, John Zhang
Maintainer: Robert D. Shear <rshear@ds.dfci.harvard.edu>
URL: https://bioconductor.org/packages/affy
BugReports: https://github.com/rafalab/affy/issues
Depends: R (>= 2.8.0), BiocGenerics (>= 0.1.12), Biobase (>= 2.5.5)
Imports: affyio (>= 1.13.3), BiocManager, graphics, grDevices, methods,
        preprocessCore, stats, utils
Suggests: tkWidgets (>= 1.19.0), affydata, widgetTools, hgu95av2cdf
LinkingTo: preprocessCore
Description: The package contains functions for exploratory
        oligonucleotide array analysis. The dependence on tkWidgets
        only concerns few convenience functions. 'affy' is fully
        functional without it.
License: LGPL (>= 2.0)
Collate: ProgressBarText.R ppset.ttest.R ppsetApply.R expressoWidget.R
        getCDFenv.R AffyRNAdeg.R avdiff.R barplot.ProbeSet.R
        bg.Affy.chipwide.R bg.R expresso.R fit.li.wong.R
        generateExprVal.method.avgdiff.R
        generateExprVal.method.liwong.R generateExprVal.method.mas.R
        generateExprVal.method.medianpolish.R
        generateExprVal.method.playerout.R hlog.R justrma.R
        loess.normalize.R maffy.R mas5.R merge.AffyBatch.R
        normalize.constant.R normalize.contrasts.R
        normalize.invariantset.R normalize.loess.R normalize.qspline.R
        normalize.quantiles.R pairs.AffyBatch.R plot.density.R
        plotLocation.R plot.ProbeSet.R pmcorrect.mas.R AffyBatch.R
        mva.pairs.R ProbeSet.R read.affybatch.R rma.R summary.R
        tukey.biweight.R whatcdf.R xy2indices.R zzz.R
biocViews: Microarray, OneChannel, Preprocessing
LazyLoad: yes
git_url: https://git.bioconductor.org/packages/affy
git_branch: RELEASE_3_23
git_last_commit: 7ed56e7
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 20:32:35 UTC; biocbuild

useDynLib("affy")

importFrom(BiocGenerics, updateObject, colnames, boxplot, image)

import(affyio)

import(zlibbioc)

importClassesFrom(Biobase, AnnotatedDataFrame, AssayData, eSet, MIAME,
                  Versioned, VersionedBiobase, Versions)

importClassesFrom(methods, ANY, character, environment, integer,
                  matrix, missing, numeric)

importMethodsFrom(Biobase, annotatedDataFrameFrom, annotation,
                  assayData, classVersion, "classVersion<-",
                  description, "description<-", experimentData, exprs,
                  "exprs<-", featureNames, "featureNames<-",
                  isCurrent, isVersioned,
		  notes,
                  "notes<-", pData, "pData<-", phenoData,
                  "phenoData<-", preproc, "preproc<-", protocolData,
                  "protocolData<-", rowMedians, sampleNames, "sampleNames<-",
                  se.exprs, "se.exprs<-")

importMethodsFrom(methods, initialize, show)

importFrom(Biobase, addVigs2WinMenu, assayDataElementReplace,
           assayDataNew, multiassign, read.AnnotatedDataFrame,
           read.MIAME, testBioCConnection)

importMethodsFrom(BiocGenerics, normalize)

importFrom(BiocManager, repositories)

importFrom(graphics, abline, axis, barplot, frame, hist,
           lines, matplot, mtext, pairs, par, plot, points,
           smoothScatter, text)

importFrom(grDevices, dev.interactive, gray)

importFrom(methods, as, callNextMethod, getMethod, is, new,
           slot)

importFrom(preprocessCore, normalize.quantiles,
           normalize.quantiles.robust, rma.background.correct)

importFrom(stats, approx, density, IQR, lm, loess, loess.control,
           median, medpolish, model.response, model.weights, optim,
           pnorm, predict, quantile, runif, sd, smooth.spline,
           splinefun, t.test, terms, var, wilcox.test)

importFrom(utils, available.packages, contrib.url, data, flush.console,
           install.packages, object.size, packageVersion)

##export everything that does not start with a .
exportPattern("^[^\\.]")

export(.setAffyOptions)

exportMethods(show, boxplot, image, exprs, "exprs<-", featureNames, geneNames, sampleNames,
              se.exprs, updateObject)

exportClasses("AffyBatch")

## S3 methods

S3method(plot, ProbeSet)
S3method("$", AffyBatch)
S3method(barplot, ProbeSet)
S3method(merge, AffyBatch)
S3method(pairs, AffyBatch)

[<img src="https://www.bioconductor.org/images/logo/jpg/bioconductor_logo_rgb.jpg" width="200" align="right"/>](https://bioconductor.org/)

**affy** is an R/Bioconductor package conaining methods for Affymetrix oligonucleotide arrays.

See https://bioconductor.org/packages/affy for more information including how to install the release version of the package (please refrain from installing directly from GitHub).

## routine tests for the normalization methods
library(affy)
library(affydata)

data(Dilution)

n.meth <- normalize.methods(Dilution)

## remove qspline
##n.meth <- n.meth[ ! (n.meth %in% c("qspline"))]

for (m in n.meth) {
  cat("-->method=", m, "...")
  Dilution.n <- normalize(Dilution, method=m)
  cat("done.\n")
}

###these are summary functions they take matrices of probes x chips
###and return expression and se (when applicable)

##DEBUG: appending the se to the expression values in a same vector
##       is too much hackish (I think)... we need to think about something
##       better

avdiff <- function(x,constant=3){
  e <- apply(x,2,function(y){
    o <- order(y)
    yy <- y[-c(o[1],o[length(y)])] #take out biggest and smallest
    if(length(yy)<2)  # SK, some genes have only one probe
      mean(y)
    else
      mean(y[abs(y-mean(yy))<constant*sd(yy)])
  })
  list(exprs=e,se.exprs=apply(x,2,sd)/sqrt(nrow(x)))
}

li.wong <- function(data.matrix,remove.outliers=TRUE,
                    normal.array.quantile=0.5,
                    normal.resid.quantile=0.9,
                    large.threshold=3,
                    large.variation=0.8,
                    outlier.fraction=0.14,
                    delta = 1e-06,maxit=50,outer.maxit=50,verbose=FALSE){

  e <-  fit.li.wong(t(data.matrix),remove.outliers,normal.array.quantile,normal.resid.quantile,large.threshold,large.variation,outlier.fraction,delta,maxit,outer.maxit,verbose)
  list(exprs=e$theta,se.exprs=e$sigma.theta)
}


medianpolish <- function(x, ...){
  tmp <- medpolish(log2(x), trace.iter=FALSE, ...)
  ##rough estimate
  sigma <- 1.483*median(abs(as.vector(tmp$residuals)))/sqrt(nrow(x))
  list(exprs=tmp$overall + tmp$col,se.exprs=rep(sigma, ncol(x)))
}

tukeybiweight <- function(x, c=5, epsilon=0.0001){
  tmp <- function(x, c=5, epsilon=0.0001)
    {
      m <- median(x)
      s <- median(abs(x - m))
      u <- (x - m) / (c * s + epsilon)
      w <- rep(0, length(x))
      i <- abs(u) <= 1
      w[i] <- ((1 - u^2)^2)[i]
      t.bi <- sum(w * x) / sum(w)
      return(t.bi)
    }
  list(exprs=apply(log2(x),2,tmp),se.exprs=rep(NA,ncol(x)))
}

barplot.ProbeSet <- function(height,
                             xlab="Probe pair",ylab="Intensity",
                             main=NA,
                             col.pm="red", col.mm="blue",
                             beside=TRUE, names.arg="pp",
                             ask = TRUE, scale = TRUE,
                             ...)
{

  opar <- par()$ask
  par(ask=ask)
  on.exit(par(ask=opar))

  if (names.arg == "pp") {
    names.arg <- seq(1, nrow(pm(height)))
  }

  col <- c(col.pm, col.mm)

  if (scale) {
    ylim <- range(c(pm(height), mm(height)), na.rm=TRUE)
  } else {
    ylim <- NULL
  }

  if (is.na(main)) {
    main <- paste(height@id, "( sample", 1:ncol(pm(height)), ")")
  } else {
    main <- rep(main, length=ncol(pm(height)))
  }
  
  for (i in 1:ncol(pm(height))) {
    
    hh <- rbind(pm(height)[, i], mm(height)[, i])
      
    barplot(hh, xlab=xlab, ylab=ylab,
            main=main[i],
            col=col,
            beside=beside,
            names.arg=names.arg,
            ylim = ylim,
            ...)
  }
}

tukey.biweight <- function(x, c=5, epsilon=0.0001)
  {
    m <- median(x)
    s <- median(abs(x - m))
    u <- (x - m) / (c * s + epsilon)
    w <- rep(0, length(x))
    i <- abs(u) <= 1
    w[i] <- ((1 - u^2)^2)[i]
    t.bi <- sum(w * x) / sum(w)
    return(t.bi)
  }

tukeybiweight <-  function(x, c=5, epsilon=0.0001)
  list(exprs=apply(x,2,tukey.biweight,c=c,epsilon=epsilon),se.exprs=rep(NA,ncol(x)))

merge.AffyBatch <- function(x, y, annotation=paste(annotation(x), annotation(y)),
                            description=NULL,
                            notes=character(0), ...) {

  adim <- dim(intensity(x))[1]

  if ((nrow(x) != nrow(y)) || (ncol(x) != ncol(y)))
    stop("cannot merge chips of different sizes !")

  if (cdfName(x) != cdfName(y))
    warning("cdfName mismatch (using the cdfName of x)!")

  if (is.null(description)){
    description <- new("MIAME")
    description@title <- "Created from merging two AffyBatches. No description was supplied. The description of the two original AffyBatches was not kept."
  }                       

  lx <- length(x)
  ly <- length(y)

  phenodata <- phenoData(x)
  pData(phenodata) <- rbind(pData(x),pData(y))
  protocoldata <- protocolData(x)
  pData(protocoldata) <- rbind(pData(protocolData(x)),pData(protocolData(y)))
  notes(description) <- 
    if (length(notes)==0) 
      list(paste("Merge from two AffyBatches with notes: 1)", notes(experimentData(x)), ", and 2)",notes(experimentData(y))))
    else notes
  return(new("AffyBatch",
             exprs=cbind(intensity(x),intensity(y)),
             phenoData=phenodata,
             experimentData=description, ##need to write a merge for MIAME
             cdfName=cdfName(x),
             nrow=nrow(x),
             ncol=ncol(x),
             annotation=x@annotation,
             protocolData=protocoldata
         ))
}

loess.normalize <- function(mat,subset=sample(1:(dim(mat)[2]),5000),
                      epsilon=10^-2,maxit=1,log.it=TRUE,verbose=TRUE,span=2/3,
                      family.loess="symmetric")
{
    .Deprecated("normalize.loess", "affy")
  J <- dim(mat)[2]
  II <- dim(mat)[1]
  newData <- mat
  if(log.it){
    mat <- log2(mat)
    newData <- log2(newData)
  }
  change <- epsilon +1
  fs <- matrix(0,II,J)##contains what we substract
  iter <- 0
  w <- c(0,rep(1,length(subset)),0) ##this way we give 0 weight to the
  ##extremes added so that we can interpolate
  while(iter < maxit){
    iter <- iter+1
    means <- matrix(0,II,J) ##contains temp of what we substract
    for(j in 1:(J-1)){
      for(k in (j+1):J){
        y <- newData[,j]-newData[,k]
        x <-(newData[,j]+newData[,k])/2
        index <- c(order(x)[1],subset,order(-x)[1])
        ##put endpoints in so we can interpolate
        xx <- x[index]
        yy <- y[index]
        aux <-loess(yy~xx,span=span,degree=1,weights=w,family=family.loess)
        aux <- predict(aux,data.frame(xx=x))/J
        means[,j] <- means[,j] + aux
        means[,k] <- means[,k] - aux
        if(verbose) cat("Done with",j,"vs",k," in iteration ",iter,"\n")
      }
    }
    fs <- fs+means
    newData <- mat-fs
    change <- max(colMeans((means[subset,])^2))
    if(verbose) cat(iter,change,"\n")
    oldfs <- fs
  }
  if(change>epsilon & maxit>1) warning(paste("No convergence after",maxit,"iterations.\n"))
  if(log.it) return(2^newData)
  else return(newData)
}

.initNormalize <- function(all.affy, env) {
  if (debug.affy123) cat("-->detecting normalization methods from naming convention\n")

  ## this could move into the respective methods of AffyBatch later

  assign("normalize.AffyBatch.methods",
         sub("^normalize\\.AffyBatch\\.", "",
             grep("^normalize.AffyBatch", all.affy, value = TRUE)),
         envir=env)
}

.initExpression <- function(all.affy, env) {
  if (debug.affy123) cat("-->detecting expression value methods from naming convention\n")

  ## the first one is deprecated (well... "should be"...)
  vals <- sub("^generateExprVal\\.method\\.", "",
              grep("^generateExprVal.method", all.affy, value = TRUE))
  assign("generateExprSet.methods", vals,
         envir=env)
  assign("express.summary.stat.methods", vals,
         envir=env)
}

.initBackgroundCorrect <- function(all.affy, env) {
  if (debug.affy123) cat("-->detecting background correction methods from naming convention\n")
  start <- nchar("bg.correct.")
  assign("bgcorrect.methods",
         sub("^bg\\.correct\\.", "",
             grep("^bg.correct", all.affy, value = TRUE)),
         envir=env)
       }

.initPmCorrect <- function(all.affy, env) {
  if (debug.affy123) cat("-->detecting pm correction methods from naming convention\n")
  
  assign("pmcorrect.methods",
         sub("^pmcorrect\\.", "",
             grep("^pmcorrect", all.affy, value = TRUE)),
         envir=env)
}

.setAffyOptions <- function(affy.opt=NA) {

  if (! any(is.na(affy.opt))) {
    if (class(affy.opt) != "BioCPkg")
      stop("obviously invalid package options !")

    BioC <- getOption("BioC")
    BioC$affy <- affy.opt
    options("BioC"=BioC)
    return()
  }

  ## add affy specific options
  ## (not unlike what is done in 'Biobase')
  if (is.null(getOption("BioC"))) {
    BioC <- list()
    class(BioC) <- "BioCOptions"
    options("BioC"=BioC)
  }

  probesloc.first <- list(what="environment", where=.GlobalEnv)
  probesloc.second <- list(what="libPath", where=NULL)
  probesloc.third <- list(what="data", where="affy")
  probesloc.fourth <- list(what="bioC", where=.libPaths()[1])


  ## default for the methods
  bgcorrect.method <- "mas"
  normalize.method <- "quantiles"
  pmcorrect.method <- "pmonly"
  summary.method <- "liwong"

  affy.opt <- list(compress.cdf=FALSE, compress.cel=FALSE,
                   use.widgets=FALSE,
                   probesloc = list(probesloc.first, probesloc.second,
                   probesloc.third, probesloc.fourth),
                   bgcorrect.method = bgcorrect.method,
                   normalize.method = normalize.method,
                   pmcorrect.method = pmcorrect.method,
                   summary.method = summary.method,
                   xy.offset = 0 ## this one is for temporary compatibility
                   ) 

  class(affy.opt) <- "BioCPkg"

  BioC <- getOption("BioC")
  BioC$affy <- affy.opt
  options("BioC"=BioC)
  ## ---
}

.affyInternalEnv <- NULL

.onLoad <- function(libname, pkgname) {
   
#  where <- match(paste("package:", pkgname, sep=""), search())
  all.affy <- ls(environment(sys.function()))

   ##a place to store some variables that need to be accessed
   .affyInternalEnv <<- new.env(parent=emptyenv())

  .initNormalize(all.affy, .affyInternalEnv)
  .initExpression(all.affy, .affyInternalEnv)
  .initBackgroundCorrect(all.affy, .affyInternalEnv)
  .initPmCorrect(all.affy, .affyInternalEnv)

  .setAffyOptions()

  if(.Platform$OS.type == "windows" && interactive()
        && .Platform$GUI ==  "Rgui"){
        addVigs2WinMenu("affy")
    }
}

#############################################################
##
## read.affybatch.R
##
## Adapted by B. M. Bolstad from read.affybatch in the affy
## package version 1.2.  The goal is a faster, less memory hungry
## ReadAffy. To do this we will shunt more work off to
## the c code.
##
## History
## Jun 13-15 Intial version
## Jun 16    Verbose flag passed to C routine
## Jun 17    New method for checking header of first cel
##           file.
## Jul 7     Added the function read.probematrix which
##           reads in PM, MM or both into matrices
## Sep 28    changed name from read.affybatch2 to read.affybatch
##           and cleaned up some old commented stuff
## Apr 13, 2004 - fixed problem in read.probematrix
## Nov 15, 2005 - add functionality to read the
##                stddev values into the se.exprs slot (non-default behaviour)
##
## Jan 24, 2006 - JWM: added cdfname to allow for the use of non-standard mappings
## Mar 6, 2006 - change .Call to reference affyio. that is new location for parsing code
## Dec 12, 2006 - added checkCelFiles() to ensure all filenames are celfiles so unintended
##                arguments don't get passed in via ...
## Apr 19, 2013 - JWM: added warning and error messages for Gene ST and Exon ST arrays
## Sept 26, 2013 - naked .Call() to affyio replaced
##
#############################################################


read.affybatch <- function(..., filenames=character(0),
                           phenoData=new("AnnotatedDataFrame"),
                           description=NULL,
                           notes="",
                           compress = getOption("BioC")$affy$compress.cel,
                           rm.mask = FALSE, rm.outliers=FALSE, rm.extra=FALSE,
                           verbose = FALSE,sd=FALSE, cdfname = NULL) {

  auxnames <- unlist(list(...))
  filenames <- c(filenames, auxnames)
  checkValidFilenames(filenames)

  n <- length(filenames)
  pdata <- pData(phenoData)
  ## try to read sample names form phenoData. if not there use CEL
  ## filenames
  if(dim(pdata)[1] != n) {
    ## if empty pdata filename are samplenames
    warning("Incompatible phenoData object. Created a new one.\n")

    samplenames <- sub("^/?([^/]*/)*", "", filenames)
    pdata <- data.frame(sample=1:n, row.names=samplenames)
    phenoData <- new("AnnotatedDataFrame",
                     data=pdata,
                     varMetadata=data.frame(
                       labelDescription="arbitrary numbering",
                       row.names="sample"))
  }
  else samplenames <- rownames(pdata)

  if (is.null(description))
    {
      description <- new("MIAME")
      preproc(description)$filenames <- filenames
      preproc(description)$affyversion <- library(help=affy)$info[[2]][[2]][2]
    }
  if (length(notes)==0) notes(description) <- notes
  ## read the first file to see what we have
  if (verbose) cat(1, "reading",filenames[[1]],"...")

  headdetails <- read.celfile.header(as.character(filenames[[1]]))
  ##now we use the length
  dim.intensity <- headdetails[[2]]   ##dim(intensity(cel))
  ##and the cdfname as ref
  ref.cdfName <- headdetails[[1]]   #cel@cdfName

  if(length(grep("gene1[01]st", cleancdfname(ref.cdfName))) == 1)
      warning(paste0("\n\nThe affy package can process data from the Gene ST 1.x series of arrays,\n",
                    "but you should consider using either the oligo or xps packages, which are specifically\n",
                     "designed for these arrays.\n\n"), call. = FALSE)
  if(length(grep("gene2[01]st|ex[1-2][0-1]st|hta20|mta10", cleancdfname(ref.cdfName))) == 1)
      stop(paste0("\n\nThe affy package is not designed for this array type.\n",
                   "Please use either the oligo or xps package.\n\n"), call. = FALSE)
              

  scandates <-
    sapply(seq_len(length(filenames)), function(i) {
             sdate <- read.celfile.header(filenames[i], info = "full")[["ScanDate"]]
             if (is.null(sdate) ||length(sdate) == 0 ) NA_character_ else sdate
           })
  protocol <-
    new("AnnotatedDataFrame",
        data=data.frame("ScanDate"=scandates, row.names=sampleNames(phenoData),
                        stringsAsFactors=FALSE),
        dimLabels=c("sampleNames", "sampleColumns"))

  ## allow for non-standard cdfs
  if(is.null(cdfname))
    cdfname <- ref.cdfName

  if (verbose)
    cat(paste("instantiating an AffyBatch (intensity a ", prod(dim.intensity), "x", length(filenames), " matrix)...", sep=""))

  if (verbose)
    cat("done.\n")

  ## Change sampleNames to be consistent with row.names of phenoData
  ## object

  exprs <-  affyio::read_abatch(filenames, rm.mask,
               rm.outliers, rm.extra, ref.cdfName,
               dim.intensity[c(1,2)],verbose)
  colnames(exprs) <- samplenames
  
  #### this is where the code changes from the original read.affybatch.
  #### what we will do here is read in from the 1st to the nth CEL file
  if (!sd){
    return(new("AffyBatch",
               exprs  = exprs,
               ##se.exprs = array(NaN, dim=dim.sd),
               cdfName    = cdfname,   ##cel@cdfName,
               phenoData  = phenoData,
               nrow       = dim.intensity[2],##["Rows"],
               ncol       = dim.intensity[1],##["Cols"],
               annotation = cleancdfname(cdfname, addcdf=FALSE),
               protocolData  = protocol,
               description= description,
               notes      = notes))
  } else {
    return(new("AffyBatch",
               exprs  = exprs,
               se.exprs =  affyio::read_abatch_stddev(filenames, rm.mask,
               rm.outliers, rm.extra, ref.cdfName,
               dim.intensity[c(1,2)],verbose),
               cdfName    = cdfname,   ##cel@cdfName,
               phenoData  = phenoData,
               nrow       = dim.intensity[2],##["Rows"],
               ncol       = dim.intensity[1],##["Cols"],
               annotation = cleancdfname(cdfname, addcdf=FALSE),
               protocolData  = protocol,
               description= description,
               notes      = notes))
  }
}





######################################################################################

read.probematrix <- function(..., filenames = character(0), phenoData = new("AnnotatedDataFrame"),
                             description = NULL, notes = "", compress = getOption("BioC")$affy$compress.cel,
                             rm.mask = FALSE, rm.outliers = FALSE, rm.extra = FALSE, verbose = FALSE,which="pm",
                             cdfname = NULL){

  auxnames <- unlist(list(...))
  filenames <- c(filenames, auxnames)

  which <- match.arg(which,c("pm","mm","both"))

  if (verbose)
    cat(1, "reading", filenames[[1]], "to get header information\n")
  headdetails <- read.celfile.header(as.character(filenames[[1]]))
  ref.cdfName <- headdetails[[1]]
  cleaned.cdfName <- cleancdfname(ref.cdfName, addcdf = FALSE)
  ## Allow for usage of alternative cdfs
  if(is.null(cdfname))
    Data <- new("AffyBatch", cdfName = ref.cdfName, annotation = cleaned.cdfName)
  else
    Data <- new("AffyBatch", cdfName = cdfname, annotation = cleaned.cdfName)
  
  cdfInfo <- as.list(getCdfInfo(Data))
  cdfInfo <- cdfInfo[order(names(cdfInfo))]

  read.celfile.probeintensity.matrices(filenames = filenames,
                                       cdfInfo = cdfInfo,
                                       rm.mask = rm.mask,
                                       rm.outliers = rm.outliers,
                                       rm.extra = rm.extra,
                                       verbose = verbose,
                                       which = which)
}


list.celfiles <-   function(...){
  files <- list.files(...)
  return(files[grep("\\.[cC][eE][lL]\\.gz$|\\.[cC][eE][lL]$", files)])
}

AllButCelsForReadAffy <- function(..., filenames=character(0),
                                  widget=getOption("BioC")$affy$use.widgets,
                                  celfile.path=NULL,
                                  sampleNames=NULL,
                                  phenoData=NULL,
                                  description=NULL){

  ##first figure out filenames
  auxnames <- unlist(as.list(substitute(list(...)))[-1])

  if (widget){
    requireNamespace("tkWidgets")
    widgetfiles <- tkWidgets::fileBrowser(
      textToShow="Choose CEL files",
      testFun=tkWidgets::hasSuffix("[cC][eE][lL]|[cC][eE][lL].gz")
    )
  }
  else{
    widgetfiles <- character(0)
  }

  if(!is.null(celfile.path)){
    auxnames <- file.path(celfile.path, auxnames)
    filenames <- file.path(celfile.path, filenames)
  }

  filenames <- c(filenames, auxnames, widgetfiles)

  if(length(filenames)==0){
    if(is.null(celfile.path)) celfile.path <- getwd()
    filenames <- list.celfiles(celfile.path,full.names=TRUE)
  }
  if(length(filenames)==0) stop("No cel filennames specified and no cel files in specified directory:",celfile.path,"\n")

  if(is.null(sampleNames)){
    sampleNames <- sub("^/?([^/]*/)*", "", filenames)
  }
  else{
    if(length(sampleNames)!=length(filenames)){
      warning("sampleNames not same length as filenames. Using filenames as sampleNames instead\n")
      sampleNames <- sub("^/?([^/]*/)*", "", filenames)
    }
  }

  chkSn <- function(filenames, samplenames){
      fntest <- sub("^/?([^/]*/)*", "", filenames)
      if(all(fntest %in% samplenames)){
          filenames <<- filenames[match(samplenames, fntest)]
      } else {
          warning(paste0("Mismatched phenoData and celfile names!\n\n",
                         "Please note that the row.names of your phenoData ",
                         "object should be identical to what you get from ",
                         "list.celfiles()!\nOtherwise you are responsible for ",
                         "ensuring that the ordering of your phenoData object ",
                         "conforms to the ordering of the celfiles as they are ",
                         "read into the AffyBatch!\nIf not, errors may ",
                         "result from using the phenoData for subsetting or ",
                         "creating linear models, etc.\n\n"),
                  call. = FALSE)
      }
  }
  
  if(is.character(phenoData)) { 
    ## if character, read file 
    if(length(phenoData)!=1) stop(sprintf("'phenoData' must be of length 1, but is %d.", length(phenoData)))
    phenoData <- read.AnnotatedDataFrame(filename=phenoData)
    sampleNames <- sampleNames(phenoData)
    chkSn(filenames, sampleNames)
  } else if(is.data.frame(phenoData)) {
    ## if data.frame, coerce
    phenoData <-  as(phenoData, "AnnotatedDataFrame")
    sampleNames <- sampleNames(phenoData)
    chkSn(filenames, sampleNames)
  } else if(is.null(phenoData)) {
    phenoData <- new("AnnotatedDataFrame",
                     data = data.frame(sample=seq_along(sampleNames),
                                       row.names=sampleNames),
                     varMetadata = data.frame(labelDescription="arbitrary numbering",
                                    row.names=names(pData)))
  } else if (!is(phenoData, "AnnotatedDataFrame")) {
    stop(sprintf("'phenoData' must be of class 'AnnotatedDataFrame', but is %s.", class(phenoData)))
  }
  
  ##get MIAME information
  if(is.character(description)){
    description <- read.MIAME(filename=description,widget=FALSE)
  }
  else{
      if (! is(description, "MIAME")) {
          description <- new("MIAME")
      }
  }

  ##MIAME stuff
  description@preprocessing$filenames <- filenames
  description@preprocessing$affyversion <- library(help=affy)$info[[2]][[2]][2]

  return(list(filenames   = filenames,
              phenoData   = phenoData,
              sampleNames = sampleNames,
              description = description))
}

###this is user friendly wrapper for read.affybatch
ReadAffy <- function(..., filenames=character(0),
                     widget=getOption("BioC")$affy$use.widgets,
                     compress=getOption("BioC")$affy$compress.cel,
                     celfile.path=NULL,
                     sampleNames=NULL,
                     phenoData=NULL,
                     description=NULL,
                     notes="",
                     rm.mask=FALSE, rm.outliers=FALSE, rm.extra=FALSE,
                     verbose=FALSE,sd=FALSE, cdfname = NULL) {

  l <- AllButCelsForReadAffy(..., filenames=filenames,
                             widget=widget,
                             celfile.path=celfile.path,
                             sampleNames=sampleNames,
                             phenoData=phenoData,
                             description=description)

  ##and now we are ready to read cel files
  ret <- read.affybatch(filenames=l$filenames,
                        phenoData=l$phenoData,
                        description=l$description,
                        notes=notes,
                        compress=compress,
                        rm.mask=rm.mask,
                        rm.outliers=rm.outliers,
                        rm.extra=rm.extra,
                        verbose=verbose,sd=sd,cdfname=cdfname)

  sampleNames(ret) <- l$sampleNames
  return(ret)
}

checkValidFilenames <- function(filenames) {
    ## Returns TRUE if filenames is a character vector containing
    ## paths to files that exist (directories don't count).
    ## A suitable error message is printed via stop() if invalid
    ## file names are encountered.
    if (!is.character(filenames))
      stop(strwrap(paste("file names must be specified using a character",
                         "vector, not a", sQuote(typeof(filenames)))),
           call.=FALSE)

    if (length(filenames) == 0)
      stop("no file names provided")

    if (any(sapply(filenames, nchar) < 1))
      stop("empty file names are not allowed")

    finfo <- file.info(filenames)
    whBad <- sapply(finfo[["isdir"]], function(x) !identical(FALSE, x))
    if (any(whBad)) {
        msg <- paste("the following are not valid files:\n",
                     paste("  ", filenames[whBad], collapse="\n"))
        stop(msg, call.=FALSE)
    }
    TRUE
}

xy2indices <- function(x, y, nc=NULL, cel=NULL, abatch=NULL, cdf=NULL, xy.offset = NULL) {

  if ( is.null(xy.offset) ) {
    xy.offset <- getOption("BioC")$affy$xy.offset
  }
  
  if (any(x < xy.offset) || any(y < xy.offset))
    stop("Xs and Ys must start at 0 or 1 (please refer to the help file) !")
  
  ct <- sum(c(is.null(nc), is.null(cel), is.null(abatch), is.null(cdf)))
  if (ct != 3)
    stop("One and only one of 'nc', 'cel', 'abatch', 'cdf' should be specified.")
  if (! is.null(cel))
    stop("Cel class no longer supported") #nr <- nrow(intensity(cel))
  if (! is.null((abatch)))
    nc <- ncol(abatch)
  if(!is.null(cdf)){
    require(cdf, character.only = TRUE, quietly = TRUE) ||
            stop(paste(cdf, "package must be installed first"))
    nam <- paste(sub("cdf$", "", cdf), "dim", sep = "")
    nc <- get("NCOL", get(nam))
  }
  
  return( (x - xy.offset) + 1 + nc * (y - xy.offset) )
}


indices2xy <- function(i, nc=NULL, cel=NULL, abatch=NULL, cdf=NULL, xy.offset = NULL) {

  if ( is.null(xy.offset) ) {
    xy.offset <- getOption("BioC")$affy$xy.offset
  }
  
  if (any(i <= 0))
    stop("Indices must start at 0 or 1 (please refer to the help file) !")

  ct <- sum(c(is.null(nc), is.null(cel), is.null(abatch), is.null(cdf)))
  
  if (ct != 3)
    stop("One and only one of 'nc', 'cel', 'abatch', 'cdf' should be specified.")
  if (! is.null(cel))
    stop("Cel class no longer supported")#    nr <- nrow(intensity(cel))
  if (! is.null((abatch)))
    nc <- ncol(abatch)
  if(!is.null(cdf)){
    require(cdf, character.only = TRUE, quietly = TRUE) ||
            stop(paste(cdf, "package must be installed first"))
    nam <- paste(sub("cdf$", "", cdf), "dim", sep = "")
    nc <- get("NCOL", get(nam))
  }
  
  
  ##x <- i %% nr
  ##x[x == 0] <- nr
  ##y <- (i - 1) %/% nr + 1  
  x <- (i  - 1) %% nc + xy.offset
  y <- (i - 1) %/% nc + xy.offset
  xy <- cbind(x, y)
  colnames(xy) <- c("x", "y")
  return(xy)
}

## Sept 11, 2003 - justRMA calls just.rma2
### A user friendly wrapper for just.rma
justRMA <- function(..., filenames=character(0),
                     widget=getOption("BioC")$affy$use.widgets,
                    compress=getOption("BioC")$affy$compress.cel,
                    celfile.path=getwd(),
                    sampleNames=NULL,
                    phenoData=NULL,
                    description=NULL,
                    notes="",
                    rm.mask=FALSE, rm.outliers=FALSE, rm.extra=FALSE,
                    hdf5=FALSE, hdf5FilePath=NULL,verbose=FALSE,
                    normalize=TRUE, background=TRUE,
                    bgversion=2, destructive=FALSE,
                    cdfname = NULL){

  l <- AllButCelsForReadAffy(..., filenames=filenames,
                             widget=widget,
                             celfile.path=celfile.path,
                             sampleNames=sampleNames,
                             phenoData=phenoData,
                             description=description)


  ##and now we are ready to read cel files
 ret<- just.rma(filenames=l$filenames,
                  phenoData=l$phenoData,
                description=l$description,
                notes=notes,
                compress=compress,
                rm.mask=rm.mask,
                rm.outliers=rm.outliers,
                rm.extra=rm.extra,
                verbose=verbose,
                normalize=normalize,
                background=background,
                bgversion=bgversion,
                destructive=destructive,
                cdfname = cdfname)
  sampleNames(ret) <- l$sampleNames
  return(ret)

}




###########################################################################################
#
# this function uses a different parsing routine
# It was added Jul 7, 2003 by B. M. Bolstad
#
###########################################################################################

just.rma <- function(..., filenames=character(0),
                     phenoData=new("AnnotatedDataFrame"),
                     description=NULL,
                     notes="",
                     compress=getOption("BioC")$affy$compress.cel,
                     rm.mask=FALSE, rm.outliers=FALSE, rm.extra=FALSE,
                     verbose=FALSE, background=TRUE, normalize=TRUE,
                     bgversion=2, destructive=FALSE, cdfname = NULL) {

  auxnames <- unlist(list(...))
  filenames <- c(filenames, auxnames)

  checkValidFilenames(filenames)
  
  n <- length(filenames)

  pdata <- pData(phenoData)
  ##try to read sample names form phenoData. if not there use CEL filenames
  if(dim(pdata)[1]!=n){#if empty pdata filename are samplenames
    warning("Incompatible phenoData object. Created a new one.\n")

    samplenames <- gsub("^/?([^/]*/)*", "", unlist(filenames))
    pdata <- data.frame(sample=1:n,row.names=samplenames)
    phenoData <- new("AnnotatedDataFrame",
                     data=pdata,
                     varMetadata=data.frame(
                       labelDescription="arbitrary numbering",
                       row.names="sample"))
  }
  else samplenames <- rownames(pdata)

  if (is.null(description))
    {
      description <- new("MIAME")
      description@preprocessing$filenames <- filenames
      description@preprocessing$affyversion <-
          as.character(packageVersion("affy"))
    }
  ## read the first file to see what we have
  ##if (verbose) cat(1, "reading",filenames[[1]],"...")

  ## get information from cdf environment

  headdetails <- read.celfile.header(filenames[[1]])
  if(is.null(cdfname))
    cdfname <- headdetails[[1]]
  scandates <-
    sapply(seq_len(length(filenames)), function(i) {
             sdate <- read.celfile.header(filenames[i], info = "full")[["ScanDate"]]
             if (is.null(sdate) || length(sdate) == 0) NA_character_ else sdate
           })
  protocol <-
    new("AnnotatedDataFrame",
        data=data.frame("ScanDate"=scandates, row.names = sampleNames(phenoData),
        stringsAsFactors=FALSE),
        dimLabels=c("sampleNames", "sampleColumns"))
  tmp <- new("AffyBatch",
             cdfName=cdfname,
             annotation=cleancdfname(cdfname, addcdf=FALSE))
  pmIndex <- pmindex(tmp)
  probenames <- rep(names(pmIndex), unlist(lapply(pmIndex,length)))
  pNList <- split(0:(length(probenames) -1), probenames)	
  

  ## read pm data into matrix

  probeintensities <- read.probematrix(filenames=filenames, cdfname = cdfname)

  ##pass matrix of pm values to rma

  ngenes <- length(geneNames(tmp))

  exprs <- .Call("rma_c_complete",probeintensities$pm, pNList, ngenes, normalize, background, bgversion, verbose, PACKAGE="affy")
  
  colnames(exprs) <- samplenames
  se.exprs <- array(NA, dim(exprs),
                    dimnames=list(rownames(exprs), colnames(exprs)))

  annotation <- annotation(tmp)
  notes(description) <- notes
  new("ExpressionSet",
      phenoData = phenoData,
      protocolData = protocol,
      annotation = annotation,
      experimentData = description,
      exprs = exprs, se.exprs = se.exprs)
}

# matdensity <- function(x,
#                        ylab="density", xlab="x", type="l", plot=TRUE,
#                        ...) {
  
#   x.density <- apply(mat, 2, density)

#   all.x <- do.call("cbind", lapply(x.density, function(x) x$x))
#   all.y <- do.call("cbind", lapply(x.density, function(x) x$y))
  
#   if (plot)
#     matplot(all.x, all.y, ylab=ylab, xlab=xlab, ...)
 
#   invisible(list(all.x=all.x, all.y=all.y))                     
                       
# }

plotDensity <- function(mat,
                        ylab="density", xlab="x", type="l", col=1:6,
                        na.rm = TRUE,
                        ...) {
  
  x.density <- apply(mat, 2, density, na.rm = na.rm)

  all.x <- do.call(cbind, lapply(x.density, function(x) x$x))
  all.y <- do.call(cbind, lapply(x.density, function(x) x$y))
  
  matplot(all.x, all.y, ylab=ylab, xlab=xlab, type=type, col=col, ...)

  invisible(list(all.x=all.x, all.y=all.y))
}
 

plotDensity.AffyBatch <- function(x, col=1:6, log=TRUE,
                                  which=c("pm","mm","both"),
                                  ylab="density",
                                  xlab=NULL,
                                  ...){
  
  Index <- unlist(indexProbes(x, which=which))
  
  x <- intensity(x)[Index, ,drop=FALSE]
  
  if(log){
    x <- log2(x)
    if(is.null(xlab)) xlab <- "log intensity"
  }
  else  if(is.null(xlab)) xlab <- "intensity"
  
  invisible(plotDensity(x, ylab=ylab, xlab=xlab, col=col, ...))

}

plotLocation <- function(x, col="green", pch=22, ...) {
  if (is.list(x)) {
    x <- cbind(unlist(lapply(x, function(x) x[,1])),
               unlist(lapply(x, function(x) x[,2])))
  }
  ## need to use nrow - x[,2] for correct y position.
  ## use image width to get nrow, which isn't ideal
  ## but follows assumption for this function that an
  ## image already exists
  nrow <- ceiling(par("usr")[4])
  if(nrow == 1) stop(paste("\nYou must first generate an image of an array",
     "for this function to work!\n\n"), call. = FALSE)
  points(x[,1], nrow - x[,2]
         , pch=pch, col=col, ...)
}

normalize.AffyBatch.constant <- function(abatch, refindex=1, FUN=mean, na.rm=TRUE) {
  
  n <- length( abatch )
  
  if (! (refindex %in% 1:n)) stop("invalid reference index for normalization")
  refconstant <- FUN(intensity(abatch)[,refindex], na.rm=na.rm)
  
  #set.na.spotsd(abatch)

  normhisto <- vector("list", length=n)
  for (i in (1:n)[-refindex]) {
    m <- normalize.constant(intensity(abatch)[,i], refconstant, FUN=FUN, na.rm=na.rm)
    myhistory <- list(name="normalized by constant",
                      constant=attr(m,"constant"))
    attr(m,"constant") <- NULL
    intensity(abatch)[, i] <- m
    normhisto[[i]] <- myhistory
  }
  attr(abatch, "normalization") <- normhisto
  return(abatch)
}       


normalize.constant <- function(x, refconstant, FUN=mean, na.rm=TRUE) {
  thisconstant <- FUN(x, na.rm=na.rm)
  r <- x / thisconstant * refconstant
  attr(r,"constant") <-  refconstant / thisconstant 
  return(r)
}

normalize.AffyBatch.contrasts <- function(abatch,span=2/3,choose.subset=TRUE,subset.size=5000,verbose=TRUE,family="symmetric",type=c("together","pmonly","mmonly","separate")) {

  type <- match.arg(type)
  
  if (type == "pmonly"){
    Index <- unlist(pmindex(abatch))
  } else if (type == "mmonly"){
    Index <- unlist(mmindex(abatch))
  } else if (type == "together"){
    Index <- unlist(indexProbes(abatch,"both"))
  } else if (type == "separate"){
    abatch <- normalize.AffyBatch.contrasts(abatch,span=span,choose.subset=choose.subset,subset.size=subset.size,verbose=verbose,family=family,type="pmonly")
    Index <- unlist(mmindex(abatch))
  }
  
  ##we need default argumetns becuase they are used in this transitional file
  alldata <- intensity(abatch)[Index,]
  
  if(choose.subset)
    subset1 <- maffy.subset(alldata,verbose=verbose,subset.size=subset.size)$subset
  else
    subset1 <- sample(1:dim(alldata)[1],subset.size)
  aux <-   maffy.normalize(alldata,subset=subset1,verbose=verbose,span=span,family=family)
  
  intensity(abatch)[Index,] <- aux

  ##attr(abatch, "normalization") <- normhisto
  return(abatch)
}

## A quick demo to overview the package
##         -- Laurent

if(dev.cur() <= 1) get(getOption("device"))()

opar <- par(ask= (interactive() &&
                  (.Device %in% c("X11", "GTK", "windows", "Macintosh"))),
            bg="cornsilk", mfrow=c(1,1))



## load the data
library(affydata)
data(cdf.example)
data(Dilution)

## display the image of the data in the CEL file

cel <- Dilution[[1]]

image(cel,transfo=log)

image(cel,transfo=log)

## find the locations for probes corresponding to a given ID

l.pm <- locate.name("AFFX-BioC-5_at", cdf.example, type="pm")
plotLocation(l.pm, cdf.example, col="red")
l.mm <- locate.name("AFFX-BioC-5_at", cdf.example, type="mm")
plotLocation(l.mm, cdf.example, col="blue")

arrows(20,60, min(l.pm[,1]), min(l.pm[,2]), col="white")
legend(20,60,c("perfect match","mismatch"),c("red","blue"),bg="white")

rm(l.pm, l.mm)



## The '5', 'M' or '3' in the names means the set of probes relates to
## the 5-prime, middle or 3-prime sectors  respectively.
## The at or st ending means the sequence relates to the complementary
## sequence of the gene or not respectively.

namesspot <- c("AFFX-BioB-5_at","AFFX-BioB-M_at", "AFFX-BioB-3_at")

p <- probeset(Dilution, genenames=namesspot)

par(mfrow=c(3,3))

for (pps in p)
  barplot(pps)


## normalize
plot.new()
par(mfrow=c(2,2))

nat <- pmormm(cdf.example)

cel2 <- Dilution[[2]]
plot(intensity(cel), intensity(cel2), xlab="CEL file 1", ylab="CEL file 2",main="raw values",sub="all probes plotted",type="n")
points(intensity(cel)[nat], intensity(cel2)[nat], col="red")
points(intensity(cel)[!nat], intensity(cel2)[!nat], col="blue")
points(intensity(cel)[is.na(nat)], intensity(cel2)[is.na(nat)], pch="+")
legend(25000, 15000, c("PM","MM","Unknown","identity line"), c("red","blue","black","grey"), bg="white")
abline(0, 1, type="l", col="gray")
rm(nat)

abatch.n <- normalize(Dilution, method="constant", refindex=2)
plot(intensity(abatch.n[[1]]), intensity(abatch.n[[2]]), xlab="CEL file 1", ylab="CEL file 2",main="normalized by constant",sub="all probes plotted")
abline(0, 1, type="l", col="gray")


abatch.n <- normalize(Dilution, method="invariantset")
i.set <- history(abatch.n[[1]])$invariantset

plot(intensity(cel), intensity(cel2), xlab="CEL file 1", ylab="CEL file 2",main="raw values",sub="all probes plotted")

abline(0, 1, type="l", col="gray")
legend(25000,15000,c("invariant set","identity line","spline through the invariant set"),c("orange","grey","red"),bg="white")

plot(intensity(abatch.n[[1]]), intensity(abatch.n[[2]]), xlab="CEL file 1", ylab="CEL file 2",main="normalized by invariant set",sub="all probes plotted")
points(intensity(abatch.n[[1]])[i.set], intensity(abatch.n[[2]])[i.set], col="orange",pch=16)
abline(0, 1, type="l", col="gray")
legend(20000,10000,c("invariant set","identity line"),c("orange","grey"),bg="white")

##
## Normalization and its effects can be observed in a couple of commands
##

#par(mfrow=c(2,2))
#data(Dilution)
#boxplot(Dilution)
#boxplot(normalize(Dilution))


p <- probeset(Dilution, "1001_at")[[1]]

par(mfcol=c(5,2))
mymethods <- express.summary.stat.methods
nmet <- length(mymethods)
nc <- ncol(pm(p))

layout(matrix(c(1:nc, rep(nc+1, nc)), nc, 2), width = c(1, 1))
##p@pm <- log(p@pm)
##p@mm <- log(p@mm)

barplot(p)

results <- matrix(0, nc, nmet)
rownames(results) <- paste("sample", 1:nc)
colnames(results) <- mymethods

for (i in 1:nmet) {
  ev <- express.summary.stat(p, summary=mymethods[i], pmcorrect="pmonly")
  if (mymethods[[i]] != "medianpolish")
    results[, i] <- log(ev$exprs)
  else
    results[, i] <- ev$exprs
}
##matplot(results, matrix(1:3, nr=nc, nc=nmet), type="l", lty=1:3, col=rainbow(nmet),
##        xlab="expression value", lab="sample")
dotchart(results, labels=paste("sample", 1:nc))
##legend()

par(opar)
rm(opar)

###
###
### Code for M and MvA plots
###
### Mar 6, 2004 - added the generic Mbox. It performs
###               the equivalent of Mbox in affyPLM
###               added a generic MAplot. Similar
###               functionality is implemented in
###               affyPLM
###               a function ma.plot now does the actual plotting
###               for mva.pairs
###
### Aug 23, 2004 - change the placement location of statistics in
###                ma.plot
### Nov 30, 2005 - fix double logging when pairs=TRUE in MAplot
### Feb 15, 2006 - fixed passing of cex variable into mva.pairs
### Feb 24, 2006 - add smoothScatter option to ma.plot
### Apr 11, 2006 - fix problem with smoothScatter option.
### Jun 22, 2006 - Fix problem with where statistics appear when xlim is set. Add plotmethod="add". move pch to MAplot. Fix it so cex is passed down to plot(). Make adding the loess line optional. Make lwd,lty,col settable for the loess line
### Jul 21, 2006 - allow MAplot to have character string sampleName arguments for which,subset,ref. But then removed subset
### Jul 23, 2006 - added groups to MAplot for AffyBatch
### Aug  4, 2006 - fix small bug in how plots are titled.
### Oct 11, 2006 - change some apply(x,1,median) to rowMedians(x)
###

ma.plot <- function(A,M,subset=sample(1:length(M),min(c(10000, length(M)))),show.statistics=TRUE,span=2/3,family.loess="gaussian",cex=2,plot.method=c("normal","smoothScatter","add"),add.loess=TRUE,lwd=1,lty=1,loess.col="red",...){

  plot.method <- match.arg(plot.method)
  
  fn.call <- list(...)

  sigma <- IQR(M)
  mean <- median(M)
  if (!is.element("ylim",names(fn.call))){
    yloc <- max(M)
  } else {
    yloc <- max(fn.call$ylim)
  }
  if (!is.element("xlim",names(fn.call))){
    xloc <- max(A)
  } else {
    xloc <- max(fn.call$xlim)
  }
  
  if(plot.method == "smoothScatter"){
    plotmethod <- "smoothScatter"
  } else if (plot.method == "add"){
    plotmethod <- "add"
  } else {
    plotmethod <- "normal"
  }
  
  

  
  aux <- loess(M[subset]~A[subset],degree=1,span=span,family=family.loess)$fitted
  if (plotmethod == "smoothScatter"){
    smoothScatter(A,M,...)
  } else if (plotmethod == "add"){
    points(A,M,cex=cex,...)
  } else {
    plot(A,M,cex=cex,...)
  }

  if (add.loess){
    o <- order(A[subset])
    A <- A[subset][o]
    M <- aux[o]
    o <-which(!duplicated(A))
    lines(approx(A[o],M[o]),col=loess.col,lwd=lwd,lty=lty)
  }
  abline(0,0,col="blue")
  

    
  # write IQR and Median on to plot
  if (show.statistics){
    txt <- format(sigma,digits=3)
    txt2 <- format(mean,digits=3)
    text(xloc ,yloc,paste(paste("Median:",txt2),paste("IQR:",txt),sep="\n"),cex=cex,adj=c(1,1))
  }

  
}



mva.pairs <- function(x,labels=colnames(x),log.it=TRUE,span=2/3,family.loess="gaussian",
                      digits=3,line.col=2,main="MVA plot", cex = 2, ...){
  if(log.it) x <-log2(x)
  J <- dim(x)[2]
  frame()
  old.par <- par(no.readonly = TRUE)
  on.exit(par(old.par))
  par(mfrow=c(J,J),mgp=c(0,.2,0),mar=c(1,1,1,1),oma=c(1,1.4,2,1))
  for(j in 1:(J-1)){
    par(mfg=c(j,j))
    plot(1,1,type="n",xaxt="n",yaxt="n",xlab="",ylab="")
    text(1,1,labels[j],cex=cex)
    for(k in (j+1):J){
      par(mfg=c(j,k))
      yy <- x[,j]-x[,k]
      xx <-(x[,j]+x[,k])/2
      sigma <- IQR(yy)
      mean <- median(yy)
      ma.plot(xx,yy,tck=0,show.statistics=FALSE,pch=".",xlab="",ylab="",tck=0,span=span,...)
      par(mfg=c(k,j))
      #sigma <- IQR(yy)
      txt <- format(sigma,digits=digits)
      txt2 <- format(mean,digits=digits)
      plot(c(0,1),c(0,1),type="n",ylab="",xlab="",xaxt="n",yaxt="n")
      text(0.5,0.5,paste(paste("Median:",txt2),paste("IQR:",txt),sep="\n"),cex=cex)
    }
  }
  par(mfg=c(J,J));plot(1,1,type="n",xaxt="n",yaxt="n",xlab="",ylab="");
  text(1,1,labels[J],cex=cex)
  mtext("A",1,outer=TRUE,cex=1.5)
  mtext("M",2,outer=TRUE,cex=1.5,las=1)
  mtext(main,3,outer=TRUE,cex=1.5)
  invisible()
}









setGeneric("Mbox",function(object,...)
           standardGeneric("Mbox"))


setMethod("Mbox",signature("AffyBatch"),
          function(object,log=TRUE,type=c("both","pm","mm"),...){
             type <- match.arg(type)
             if (type == "both"){
              pms <- unlist(indexProbes(object, "both"))
            } else if (type == "pm"){
              pms <- unlist(pmindex(object))
            } else if (type == "mm"){
              mms <- unlist(mmindex(object))
            }
            if(log){
              x <- log2(intensity(object)[pms, ])
            } else {
              x <- intensity(object)[pms, ]
            }
            medianchip <- rowMedians(x)
            M <- sweep(x,1,medianchip,FUN='-')
            boxplot(data.frame(M),...)
          })

setGeneric("MAplot",function(object,...)
           standardGeneric("MAplot"))


setMethod("MAplot",signature("AffyBatch"),
          function(object,log=TRUE,type=c("both","pm","mm"),groups=NULL,ref=NULL,which=NULL,pairs=FALSE,pch=".",ref.fn=c("median","mean"),ref.title="vs pseudo-median reference chip",...){

            type <- match.arg(type)
            if (type == "both"){
              pms <- unlist(indexProbes(object, "both"))
            } else if (type == "pm"){
              pms <- unlist(pmindex(object))
            } else if (type == "mm"){
              pms <- unlist(mmindex(object))
            }
            if(log){
              x <- log2(intensity(object)[pms, ])
            } else {
              x <- intensity(object)[pms, ]
            }
            
            if (is.null(groups)){
              if (is.character(ref)){
                ref.indices <- match(ref,sampleNames(object))
                if (all(is.na(ref.indices))){
                  stop("No known sampleNames in ref")
                }
                
                if (any(is.na(ref.indices))){
                  warning(paste("Omitting the following from ref:",ref[is.na(ref.indices)], "because they can not be found."))
                }
                ref <- ref.indices[!is.na(ref.indices)]
              }
            
              if (is.character(subset)){
                subset.indices <- match(subset,sampleNames(object))
                if (all(is.na(subset.indices))){
                  stop("No known sampleNames in subset")
                }
                
                if (any(is.na(subset.indices))){
                  warning(paste("Omitting the following from subset:",subset[is.na(subset.indices)], "because they can not be found."))
                }
                subset <- subset.indices[!is.na(subset.indices)]
              }
              
              if (is.character(which)){
                which.indices <- match(which,sampleNames(object))
                if (all(is.na(which.indices))){
                  stop("No known sampleNames in which")
                }
                
                if (any(is.na(which.indices))){
                  warning(paste("Omitting the following from which:",which[is.na(which.indices)], "because they can not be found."))
                }
                which <- which.indices[!is.na(which.indices)]
              }
              
              
              if (is.null(which)){
                which <- 1:dim(exprs(object))[2]
              }
              
              ref.fn <- match.arg(ref.fn)

              if(!pairs){
                if (is.null(ref)){
                  medianchip <- rowMedians(x)            ###apply(x, 1, median)
                }  else if (length(ref) > 1){
                  if (ref.fn == "median"){
                    medianchip <- rowMedians(x[,ref])
                  } else {
                    medianchip <- rowMeans(x[,ref])
                  }
                } else {
                  medianchip <- x[,ref]
                }
                
                M <- sweep(x,1,medianchip,FUN='-')
                A <- 1/2*sweep(x,1,medianchip,FUN='+')
                if (is.null(ref)){
                  for (i in which){
                    title <- paste(sampleNames(object)[i],"vs pseudo-median reference chip")
                    ma.plot(A[,i],M[,i],main=title,xlab="A",ylab="M",pch=pch,...)
                  }
                } else {
                  for (i in which){
                    if (length(ref) == 1){
                      if (i != ref){ ##changed which to i
                        title <- paste(sampleNames(object)[i],"vs",sampleNames(object)[ref])
                        ma.plot(A[,i],M[,i],main=title,xlab="A",ylab="M",pch=pch,...)
                      }
                    } else {
                      title <- paste(sampleNames(object)[i],"vs",ref.title)
                      ma.plot(A[,i],M[,i],main=title,xlab="A",ylab="M",pch=pch,...)
                    }
                  }
                }
                
              } else {
                if(!is.null(ref)) stop("Can't use pairs with non-null 'ref'")
                if(is.null(which)) which <- 1:ncol(x)
                mva.pairs(x[,which],log.it=FALSE,...)
              }
            } else {
              ## group labels have been given
              ## check that group variable is of same length as number of samples 
                
              if (dim(x)[2] != length(groups)){
                stop("'groups' is of wrong length.")
              }
              
              
              ### group labels variable can be integer, character or factor variable.
              ### need to check that if any names supplied
              ### for ref or which can be found in group.labels
                
              if (!is.null(which)){
                if (is.numeric(groups)){
                  if (!is.numeric(which)){
                    stop("'which' labels must also be found in 'groups'") 
                  } else {
                    if (!all(is.element(which,groups))){
                      stop("'which' labels must also be found in 'groups'") 
                    }
                  }
                } else if (is.factor(groups)){
                  if (!is.character(which)){
                    stop("'which' should be character vector") 
                  } else {
                    if (!all(is.element(which,as.character(groups)))){
                      stop("'which' labels must also be found in 'groups'") 
                    }
                  }
                } else if (is.character(groups)){
                  if (!is.character(which)){
                    stop("'which' should be character vector") 
                  } else {
                    if (!all(is.element(which,groups))){
                      stop("'which' labels must also be found in 'groups'") 
                    }
                  }
                }
              }
              
              if (!is.null(ref)){
                if (is.numeric(groups)){
                  if (!is.numeric(ref)){
                    stop("'ref' labels must also be found in 'groups'") 
                  } else {
                    if (!all(is.element(ref,groups))){
                      stop("'ref' labels must also be found in 'groups'") 
                    }
                  }
                } else if (is.factor(groups)){
                  if (!is.character(ref)){
                    stop("'ref' should be character vector") 
                  } else {
                    if (!all(is.element(ref,as.character(groups)))){
                      stop("'ref' labels must also be found in 'groups'") 
                    }
                  }
                } else if (is.character(groups)){
                  if (!is.character(ref)){
                    stop("'ref' should be character vector") 
                  } else {
                    if (!all(is.element(ref,groups))){
                      stop("'ref' labels must also be found in 'groups'") 
                    }
                  }
                }
              }
              
              ref.fn <- match.arg(ref.fn)
              
              groups.list <- split(1:dim(x)[2], as.factor(groups))
              
              
              grouped.data <- matrix(0,nrow(x),length(groups.list))
              colnames(grouped.data) <- names(groups.list)
              which.col <- 1
              for (group in groups.list){
                grouped.data[,which.col] <- rowMeans(x[,group,drop=FALSE])
                which.col <- which.col + 1
              }
              
              
              if (!pairs){
                if (is.null(which)){
                  which <- names(groups.list)
                }
                
                if (is.null(ref)){
                  if (ref.fn == "median"){
                    medianchip <- rowMedians(grouped.data)    ####apply(grouped.data, 1, median)
                  } else {
                    medianchip <- rowMeans(grouped.data)
                  }
                  
                } else if (length(ref) == 1){
                  ref.name <- ref
                  ref <- match(ref,names(groups.list))
                  medianchip <- grouped.data[,ref]
                } else {
                  ref <- match(ref,names(groups.list))
                  if (ref.fn == "median"){
                    medianchip <- rowMedians(grouped.data[,ref])
                  } else {
                    medianchip <- rowMeans(grouped.data[,ref])
                  }
                  
                }
                
                M <- sweep(grouped.data,1,medianchip,FUN='-')
                A <- 1/2*sweep(grouped.data,1,medianchip,FUN='+')
                if (is.null(ref)){
                  for (i in which){
                    title <- paste(i,ref.title)
                    ma.plot(A[,i],M[,i],main=title,xlab="A",ylab="M",pch=pch,...)
                  }
                } else {
                  for (i in which){
                    if (length(ref) == 1){
                      if (i != ref.name){
                        title <- paste(i,"vs",ref.name)
                        ma.plot(A[,i],M[,i],main=title,xlab="A",ylab="M",pch=pch,...)
                      }
                    } else {
                      title <- paste(i,ref.title)
                      ma.plot(A[,i],M[,i],main=title,xlab="A",ylab="M",pch=pch,...)
                    }
                  }
                }
              } else {
                if (!is.null(ref)) stop("Can't use pairs with non-null 'ref'")
                if (is.null(which)){
                  which <- names(groups.list)
                }
                
                mva.pairs(grouped.data[,which],log.it=FALSE,...)
              }
            }
          })

##this function changes the Affymetrix cdf file name to the Bioconductor
##annotation name for that cdf file
## note: we had a hard time finding exact rules to match what is in the
## CEL file with what is in the CDF file
## ex: CEL says 'ecoli' while CDF says 'ecoligenome'
## or: CEL says '' while CDF says hu6800.1sq
cleancdfname <- function(cdfname, addcdf=TRUE) {
  if( !is.character(cdfname) )
                stop(paste("invalid CDF name:", cdfname))
  if ( nchar(cdfname)[1] == 0 )
               stop("supplied cdf name has zero length")
  mapCdfName <- NULL # To appease R CMD check
  mapCdfName <- local({
      data("mapCdfName", package="affy", envir=environment())
      get("mapCdfName")
  })
  i <- match(cdfname, mapCdfName$inCDF)
  if (is.na(i)) {
    tmp <- tolower(cdfname) #make lower case
    tmp <- gsub("_", "", tmp) #take out underscore
    tmp <- gsub("-", "", tmp) #take out underscore
    tmp <- gsub("\ ", "", tmp) ##take out spaces
    if (addcdf) {
        ## make sure we haven't already added "cdf"
        endsWith <- substr(tmp, nchar(tmp)-2, nchar(tmp))
        if (endsWith != "cdf")
          tmp <- paste(tmp, "cdf", sep="")
    }
  } else {
    tmp <- mapCdfName$inBioC[i]
  }
  return(tmp)
}

##this function gets the cdf from a celfile
whatcdf <- function(filename, compress=getOption("BioC")$affy$compress.cel)
  return(read.celfile.header(as.character(filename))[[1]])

## -------------------------------------------
## routine tests for expression values methods
## -------------------------------------------

library(affy)
library(affydata)

data(Dilution)

 essm = express.summary.stat.methods()
 i <- match("playerout", essm)
 meths <- essm[-i]

for (m in meths) {
  for (mbc in pmcorrect.methods()) {
     cat("expression value with method=", m, "bg correct=", mbc, "...")
     computeExprSet(Dilution, pmcorrect.method=mbc, summary.method=m)
     cat("done.\n")
   }
}

## playerout alone 'cause very slow
m <- "playerout"
for (mbc in pmcorrect.methods()) {
  cat("expression value with method=", m, "bg correct=", mbc, "...")
  computeExprSet(Dilution, pmcorrect.method=mbc, summary.method=m,
                 ids=geneNames(Dilution)[1:3]) 
  cat("done.\n")
}

getCdfInfo <- function(object,  how=getOption("BioC")$affy$probesloc, verbose=FALSE) {
    ## cdfname is the cdf environment
    ## methods is a vector detailing how to get the file - one of
    ## 'library', 'bioC'

    if (length(how) == 0)
        stop("No available method to obtain CDF file")

    cdfname <- cdfName(object)

    badOut <- list()
    for (i in 1:length(how)) {
        cur <- how[[i]]
        out <- switch(cur$what,
                      "environment" = cdfFromEnvironment(cdfname,
                      cur$where, verbose),
                      "libPath" = cdfFromLibPath(cdfname, cur$where,
                      verbose=verbose),
                      "bioC" = cdfFromBioC(cdfname, cur$where,
                      verbose)
                      )

        if (is.environment(out))
            return(out)
        else
            badOut <- c(badOut, out)
    }

    stop(paste("Could not obtain CDF environment, problems encountered:",
               paste(unlist(badOut),collapse="\n"),sep="\n"))
}

cdfFromEnvironment <- function(cdfname, where, verbose=TRUE) {
    if (verbose)
        print(paste("Attempting to locate",cdfname,"in specified environment"))

    if (exists(cdfname, inherits=FALSE, where=where))
        return(as.environment(get(cdfname,inherits=FALSE,envir=where)))
    else {
        if (verbose)
            print(paste("Specified environment does not contain",cdfname))
        return(list(paste("Specified environment does not contain",cdfname)))
    }
}

cdfFromBioC <- function(cdfname, lib=.libPaths()[1], verbose=TRUE) {
    cdfname <- cleancdfname(cdfname)
    if (verbose)
      print(paste("Attempting to obtain",cdfname,"from Bioconductor website"))

    ## First try libPaths
    libs <- .libPaths()
    if (!all(lib %in% libs))
      libs <- unique(c(lib, libs))
    result <- cdfFromLibPath(cdfname, lib=libs, verbose=verbose)
    if (is.environment(result))
      return(result)

    if (length(lib) > 1) {
        warning("Ignoring all but first element of argument lib")
        lib <- lib[1]
    }

    if (verbose)
      print(paste("The environment ",cdfname," was not found in",
                  " these directories: ",
                  paste(libs, collapse=", "),
                  ".  Now searching the internet repository.",
                  sep=""))
    if (verbose)
      print(paste("Checking to see if your internet connection works ..."))
    if (testBioCConnection()) {
        ## Check for file permissions
        if (file.access(lib, mode=0) < 0) {
            if (verbose) {
                print(paste("Directory",lib,"does not seem to exist.\n",
                            "Please check your 'lib' parameter and try again"))
                return(list("Bioconductor - lib does not exist"))
            }
        }

        if (file.access(lib,mode=2) < 0) {
            if (verbose) {
                print(paste("You do not have write access to",lib,
                            "\nPlease check your permissions or provide",
                            "a different 'lib' parameter"))
                return(list("Bioconductor - lib is not writeable"))
            }
        }

        biocContribUrl <- sapply(repositories(), contrib.url)
        biocPkgs <- available.packages(biocContribUrl)
        if (! cdfname %in% biocPkgs[, "Package"]) {
            if (verbose)
              print(paste("Environment",cdfname,
                          "was not found in the Bioconductor",
                          "repository."))
            return(list(paste("Bioconductor -",cdfname,"not available")))
        } else {
            install.packages(cdfname, lib=lib,
                             repos=repositories(),
                             dependencies=c("Depends","Imports"))
            ## no way to know if we succeeded or not, with install.packages
            ##if (verbose)
            ##  print(paste("Installation of environment",
            ##              cdfname, "was succesful."))
        }
    } else {
        if (verbose)
          print(paste("The current operation could not access",
                      "the Bioconductor repository.  Please",
                      "check your internet connection, and",
                      "report further problems to",
                      "bioconductor@stat.math.ethz.ch"))
        return(list("Bioconductor - could not connect"))
    }
    return(cdfFromLibPath(cdfname, lib=lib, verbose=verbose))
}

cdfFromLibPath <- function(cdfname, lib = NULL, verbose=TRUE) {
    cdfname <- cleancdfname(cdfname)
    ## First check to see if package is installed
    if (verbose)
      print(paste("Checking to see if package",cdfname,
                  "is already installed"))

    if (length(find.package(cdfname, lib.loc=lib, quiet=TRUE)) == 0)
      return(list(paste("Library - package",cdfname,"not installed")))

    ## See if package is already loaded
    if (cdfname %in% .packages()) {
        if (verbose)
          print(paste("The package", cdfname, "is already loaded"))
    }
    else {
        if (verbose)
          print(paste("Attempting to load package", cdfname))
        ## Attempt to load the library requested
        do.call(library, list(cdfname, lib.loc=lib, character.only=TRUE))

        ## Check to see if it got loaded
        if (! cdfname %in% .packages()) {
            ## package didn't get loaded
            if (verbose)
              print(paste("The package", cdfname, "could not be loaded"))
            return(list(paste("Library - package",cdfname,"is not loadable")))
        }
    }

    return(get(cdfname, envir=as.environment(paste("package:", cdfname, sep=""))))
}

normalize.AffyBatch.qspline <- function(abatch, type=c("together","pmonly","mmonly","separate"),...) {

  type <- match.arg(type)

  if (type == "together"){
    Index <- unlist(indexProbes(abatch,"both"))
    intensity(abatch)[Index,] <- normalize.qspline(intensity(abatch)[Index,], ...)
  } else if (type == "pmonly"){
    Index <- unlist(indexProbes(abatch,"pm"))
    intensity(abatch)[Index,] <- normalize.qspline(intensity(abatch)[Index,], ...)
  } else if (type == "mmonly"){
    Index <- unlist(indexProbes(abatch,"mm"))
    intensity(abatch)[Index,] <- normalize.qspline(intensity(abatch)[Index,], ...)
  } else if (type == "separate"){
    Index <- unlist(indexProbes(abatch,"pm"))
    intensity(abatch)[Index,] <- normalize.qspline(intensity(abatch)[Index,], ...)
    Index <- unlist(indexProbes(abatch,"mm"))
    intensity(abatch)[Index,] <- normalize.qspline(intensity(abatch)[Index,], ...)
  }
  #set.na.spotsd(listcel)
  normhisto <- vector("list", length=ncol(intensity(abatch)))
  ##need to use MIAME for this
  for (i in 1:length(abatch)) {
    normhisto[[i]] <- list(name="normalized by qspline")
  }

  attr(abatch, "normalization") <- normhisto

  return(abatch)
}

normalize.qspline <- function(x,
                              target        = NULL,
                              samples       = NULL,
                              fit.iters     = 5,
                              min.offset    = 5,
                              spline.method = "natural", # c("fmm", "natural", "periodic")
                              smooth        = TRUE,
                              spar          = 0,     # smoothing parameter
                              p.min         = 0,
                              p.max         = 1.0,
                              incl.ends     = TRUE,
                              converge      = FALSE,
                              verbose       = TRUE,
                              na.rm         = FALSE
                              ){

  if (is.null(target))
    target <- exp(apply(log(x), 1, mean))

  x.n <- dim(x)[1]
  m   <- dim(x)[2]

  if (is.null(samples))
    samples <- max(round(x.n/1000), 100)
  else
    if (samples < 1)
      samples <- round(samples * x.n)

  p <- (1:samples) / samples
  p <- p[ which(p <= p.max) & which(p >= p.min) ]
  samples <- length(p)

  k <- fit.iters

  if (na.rm==TRUE)
    y.n <- sum(!is.na(target))
  else
    y.n <- length(target)

  py.inds  <- as.integer(p * y.n)
  y.offset <- round(py.inds[1]/fit.iters)

  if (y.offset <= min.offset) {
    y.offset <- min.offset;
    k <- round(py.inds[1]/min.offset)
  }

  if (k <= 1) {
    warning("'k' found is non-sense. using default 'fit.iter'")
    k <- fit.iters
  }

  y.offset <- c(0, array(y.offset, (k-1)))
  y.order <- order(target)

  fx <- matrix(0, x.n,m)
  if(verbose==TRUE)
    print(paste("samples=",samples, "k=", k, "first=", py.inds[1]))

  for (i in 1:m) {
                                        # to handel NA values for each array
    if (na.rm==TRUE)
      x.valid <- which(!is.na(x[,i]))
    else
      x.valid <- 1:x.n

    x.n <- length(x.valid)
    px.inds  <- as.integer(p * x.n)
      x.offset <- round(px.inds[1]/fit.iters)

    if (x.offset<=min.offset) {
      x.offset <- min.offset;
      k <- min(round(px.inds[1]/min.offset), k)
    }

    x.offset <- c(0, array(x.offset, (k-1)))
    x.order  <- order(x[,i]) # NA's at the end (?)

    y.inds   <- py.inds ## must be reset each iteration
    x.inds   <- px.inds

    for (j in 1:k) {
         y.inds <- y.inds - y.offset[j]
         x.inds <- x.inds - x.offset[j]
         ty.inds <- y.inds
         tx.inds <- x.inds
         if (verbose==TRUE)
           print(paste("sampling(array=", i, "iter=", j, "off=",
                       x.inds[1], -x.offset[j], y.inds[1], -y.offset[j], ")"))

         if (converge==TRUE) {
           ty.inds <- as.integer(c(1, y.inds))
           tx.inds <- as.integer(c(1, x.inds))

           if (j > 1) {
             ty.inds <- c(ty.inds, y.n)
             tx.inds <- c(tx.inds, x.n)
           }
         }
         qy <- target[y.order[ty.inds]]
         qx <-  x[x.order[tx.inds],i]

         if (smooth==TRUE) {
           sspl <- smooth.spline(qx, qy, spar=spar)
           qx <- sspl$x
           qy <- sspl$y
         }

         fcn <- splinefun(qx, qy, method=spline.method)
         fx[x.valid,i] <- fx[x.valid,i] + fcn(x[x.valid,i])/k
       }

    if (na.rm==TRUE) {
      invalid <- which(is.na(x[,i]))
      fx[invalid,i] <- NA
    }
  }
  return(fx)
}

normalize.AffyBatch.loess <- function(abatch,type=c("together","pmonly","mmonly","separate"),...) {

  type <- match.arg(type)

  if (type == "separate"){
    Index <- unlist(indexProbes(abatch,"pm"))
    intensity(abatch)[Index,] <- normalize.loess(intensity(abatch)[Index,], ...)
    Index <- unlist(indexProbes(abatch,"mm"))
    intensity(abatch)[Index,] <- normalize.loess(intensity(abatch)[Index,], ...)
  } else if (type=="together"){
    Index <- unlist(indexProbes(abatch,"both"))
    intensity(abatch)[Index,] <- normalize.loess(intensity(abatch)[Index,], ...)
  } else if (type=="pmonly"){
    Index <- unlist(indexProbes(abatch,"pm"))
    intensity(abatch)[Index,] <- normalize.loess(intensity(abatch)[Index,], ...)
  } else if (type=="mmonly"){
    Index <- unlist(indexProbes(abatch,"mm"))
    intensity(abatch)[Index,] <- normalize.loess(intensity(abatch)[Index,], ...)
  }
  ##set.na.spotsd(listcel) # set 'sd' to nothing (meaningless after normalization)
  ##cat(cols,rows)


  ##need to use MIAME
  ##for (i in 1:abatch@nexp) {
  ##  history(abatch)[[i]] <- list(name="normalized by loess")
  ##}

  return(abatch)
}



normalize.loess <- function(mat, subset=sample(1:(dim(mat)[1]), min(c(5000, nrow(mat)))),
                            epsilon=10^-2, maxit=1, log.it=TRUE, verbose=TRUE, span=2/3,
                            family.loess="symmetric"){

  J <- dim(mat)[2]
  II <- dim(mat)[1]
  if(log.it){
    mat <- log2(mat)
  }

  change <- epsilon +1
  iter <- 0
  w <- c(0, rep(1,length(subset)), 0) ##this way we give 0 weight to the
                                      ##extremes added so that we can interpolate

  while(iter < maxit){
    iter <- iter + 1
    means <- matrix(0,II,J) ##contains temp of what we substract

    for (j in 1:(J-1)){
      for (k in (j+1):J){
        y <- mat[,j] - mat[,k]
        x <- (mat[,j] + mat[,k]) / 2
        index <- c(order(x)[1], subset, order(-x)[1])
        ##put endpoints in so we can interpolate
        xx <- x[index]
        yy <- y[index]
        aux <-loess(yy~xx, span=span, degree=1, weights=w, family=family.loess)
        aux <- predict(aux, data.frame(xx=x)) / J
        means[, j] <- means[, j] + aux
        means[, k] <- means[, k] - aux
        if (verbose)
          cat("Done with",j,"vs",k,"in iteration",iter,"\n")
      }
    }
    mat <- mat - means
    change <- max(colMeans((means[subset,])^2))

    if(verbose)
      cat(iter, change,"\n")

  }

  if ((change > epsilon) & (maxit > 1))
    warning(paste("No convergence after", maxit, "iterations.\n"))

  if(log.it) {
    return(2^mat)
  } else
    return(mat)
}

expresso <- function(afbatch,
                     ## --
                     bg.correct=TRUE,
                     bgcorrect.method = NULL,
                     bgcorrect.param = list(),
                     ## --
                     normalize = TRUE,
                     normalize.method = NULL,
                     normalize.param=list(),
                     ## --
                     pmcorrect.method = NULL,
                     pmcorrect.param = list(),
                     ## --
                     summary.method = NULL,
                     summary.param = list(),
                     summary.subset = NULL,
                     ## ---
                     verbose = TRUE,
                     widget = FALSE
                     ) {
    # JZ added this function
    setCorrections <- function(){
        bioc.opt <- getOption("BioC")
        if(bg.correct){
            if(is.null(bgcorrect.method)){
                BGMethods <- bgcorrect.methods()
            }else{
                BGMethods <- bgcorrect.method
            }
        }else{
            BGMethods <- "None"
        }
        if(normalize){
            if(is.null(normalize.method)){
                normMethods <- normalize.methods(afbatch)
            }else{
                normMethods <- normalize.method  
            }
        }else{
            normMethods <- "None"
        }
        # Default for this one may not be correct
        if(is.null(pmcorrect.method)){
            PMMethods <- pmcorrect.methods()
        }else{
            PMMethods <- pmcorrect.method
        }
        # Default for this one may not be correct
        if(is.null(summary.method)){
            expMethods <- generateExprSet.methods()

        }else{
            expMethods <- summary.method
        }

        corrections <- expressoWidget(BGMethods, normMethods, PMMethods,
                           expMethods, bioc.opt$affy$bgcorrect.method,
                                      bioc.opt$affy$normalize.method,
                                      bioc.opt$affy$pmcorrect.method,
                                      bioc.opt$affy$summary.method)
        if(!is.null(corrections)){
            if(corrections[["BG"]] != "None"){
                bgcorrect.method <<- corrections[["BG"]]
            }
            if(corrections[["NORM"]] != "None"){
                normalize.method <<- corrections[["NORM"]]
            }
            if(corrections[["PM"]] != "None"){
                pmcorrect.method <<- corrections[["PM"]]
            }
            if(corrections[["EXP"]] != "None"){
                summary.method <<- corrections[["EXP"]]
            }
        }else{
            stop("Aborted by user")
        }
    }

  if (widget) {
    requireNamespace("tkWidgets")
  }

  nchips <- length(afbatch)

  ###background stuff must be added before normalization!

  if(widget){
      setCorrections()
  }

  ## -- summary of what will be done
  if (verbose) {
    if (bg.correct){
      cat("background correction:", bgcorrect.method, "\n")
    }
    if (normalize) {
      cat("normalization:", normalize.method, "\n")
    }
    cat("PM/MM correction :", pmcorrect.method, "\n")
    cat("expression values:", summary.method, "\n")
  }


  ## -- background correct (if needed)
  if (bg.correct) {

    if (verbose)
      cat("background correcting...")

    afbatch <- do.call("bg.correct", c(alist(afbatch, method=bgcorrect.method), bgcorrect.param))

    if (verbose)
      cat("done.\n")
  }

  ## -- normalize (if wished)
  if (normalize) {

    if (verbose)
      cat("normalizing...")

    afbatch <- do.call(BiocGenerics::normalize, c(alist(afbatch, normalize.method), normalize.param))

    if (verbose)
      cat("done.\n")
  }

  eset <- computeExprSet(afbatch,
                         summary.method=summary.method, pmcorrect.method= pmcorrect.method,
                         ids=summary.subset,
                         summary.param=summary.param, pmcorrect.param=pmcorrect.param)

  return(eset)
}

##*******************************************************************************************
#**********  maffy.normalize     *****
maffy.normalize <- function(data,subset,verbose=FALSE,span=0.25,family="symmetric",log.it=TRUE){

k <- dim(data)[2]   ### Number of chips

####   Create the transformation matrix
t1 <- 1/sqrt(k)
t2 <- (k-2-t1)/(k-1)
t3 <- -(1+t1)/(k-1)

transmat <- matrix(t3,k,k)
for(i in 1:k){
     transmat[1,i]<-t1
     transmat[i,1]<-t1
}
for(i in 2:k) transmat[i,i]<-t2

#### Find normalizing curve

if(verbose) cat("Fitting normalizing curve\n")
n<- length(subset)
data.subset <- data[subset,]

data.subset <- log(data.subset)%*%t(transmat)
index <- order(data.subset[,1])
data.subset <- data.subset[index,]

if( k>2) curve <- multiloess(data.subset[,2:k]~data.subset[,1],span=span,family=family,surface="direct")
else     curve <-      loess(data.subset[,2:k]~data.subset[,1],span=span,family=family,surface="direct")

### Transform the normalizing curve before and after normalization
scaled   <- cbind(data.subset[,1],matrix(0,n,k-1)) %*%(transmat)
unscaled <- cbind(data.subset[,1],curve$fitted)      %*%(transmat)

w <-c(0,rep(1,n,n),0)

data.scaled <- NULL

### Normalize each array
for(i in 1:k){
    if(verbose) cat("Normalizing chip ",i,"\n")
    if(log.it){
        mini  <- log(min(data[,i]))
        maxi  <- log(max(data[,i]))
    }
    else{
        mini  <- min(data[,i])
        maxi  <- max(data[,i])
    }

    curve <- loess(c(mini,scaled[,i],maxi)~c(mini,unscaled[,i],maxi),weights=w,span=span)

    if(log.it)
         temp <-  exp(predict(curve,log(data[,i])))
    else
         temp <-      predict(curve,data[,i])

    data.scaled <- cbind(data.scaled,temp)
}

data.scaled

}


##*******************************************************************************************
#**********  Select A subset with small rank-range over arrays  *****

maffy.subset <- function(data,subset.size=5000,maxit=100,subset.delta=max(round(subset.size/100),25),verbose=FALSE){


k     <- dim(data)[2]   ### Number of chips
n     <- dim(data)[1]   ## Size of starting subset, i.e. all rows

if(verbose)
      cat("Data size",n,"x",k,"Desired subset size",subset.size,"+-",subset.delta,"\n")

means <- data%*%(rep(1,k,k)/k)

index0 <- order(means)

data.sorted <- data[index0,]

## Init
set <- rep(TRUE,n,n)      ## Set-indicator
index.set <- 1:n       ## Indexes for subset
nprev <- n+1
iter  <- 1
part.of.n <- 1

## loop
while(nprev>n & n>(subset.size+subset.delta) & iter <maxit){
    if(verbose)
      cat("Comuting ranks of old subset....")
    ranks <-apply(data.sorted[index.set,],2,rank)              ## Compute ranks, chip by chip.
    ranks.range <- apply(ranks,1,function(r) max(r)-min(r) )   ## Range of ranks over chips

    q <-min((n*part.of.n+subset.size)/((1+part.of.n)*n),1)     ## Select quantiles
    low <- quantile(ranks.range[1:(n*0.2)+n*0.0],probs=q,names=FALSE)/n
    high <-quantile(ranks.range[n+1-(1:(n*0.2))],probs=q,names=FALSE)/n

    newset <-  ranks.range < (low*n+(0:(n-1))*(high-low))        ## Set-indicator of new set

    if(sum(newset)<subset.size-subset.delta){                  ## To small?
       part.of.n <- 1+part.of.n
       if(verbose)
         cat("\nSize of newset to small (",sum(newset),"). Increasing part.of.n.\n")
    }
    else{                                                      ## New set OK
       set <- newset
       index.set <- subset(index.set,set)
       index.set <- index.set[!is.na(index.set)]
       nprev <- n
       n <- length(index.set)
       if(verbose)
          cat("Size of new subset: ",n,"\n")
   }

   iter <- iter+1
}
##end loop

if(!iter <maxit) warning("Maximum number of iterations reached, result my not be correct\n")

list(subset=index0[index.set])

}




##*******************************************************************************************
multiloess <-
function(formula, data=NULL, weights, subset, na.action, model = FALSE,
	 span = 0.75, enp.target, degree = 2,
	 normalize = TRUE,
	 family = c("gaussian", "symmetric"),
	 method = c("loess", "model.frame"),
	 control = loess.control(...), ...)
{
    parametric <- FALSE
    drop.square <- FALSE

    mt <- terms(formula, data = data)
    mf <- match.call(expand.dots=FALSE)
    mf$model <- mf$span <- mf$enp.target <- mf$degree <-
	mf$parametric <- mf$drop.square <- mf$normalize <- mf$family <-
	    mf$control <- mf$... <- NULL
    mf[[1]] <- as.name("model.frame")
    mf <- eval(mf, parent.frame())
    if (match.arg(method) == "model.frame") return(mf)
    y <- model.response(mf, "numeric")

    if(is.vector(y))  stop("The respons variable is not a matrix, use loess")

    w <- model.weights(mf)
    if(is.null(w)) w <- rep(1, NROW(y))
    nmx <- as.character(attr(mt, "variables"))[-(1:2)]
    x <- mf[, nmx, drop=FALSE]
    if(any(sapply(x, is.factor))) stop("predictors must all be numeric")
    x <- as.matrix(x)
    D <- ncol(x)
    nmx <- colnames(x)
    names(nmx) <- nmx
    drop.square <- match(nmx, nmx[drop.square], 0) > 0
    parametric <- match(nmx, nmx[parametric], 0) > 0
    if(!match(degree, 0:2, 0)) stop("degree must be 0, 1 or 2")
    iterations <- if(family=="gaussian") 1 else control$iterations
    if(!missing(enp.target))
	if(!missing(span))
	    warning("both span and enp.target specified: span will be used")
	else {				# White book p.321
	    tau <- switch(degree+1, 1, D+1, (D+1)*(D+2)/2) - sum(drop.square)
	    span <- 1.2 * tau/enp.target
	}
    fit <- simplemultiLoess(y, x, w, span, degree,
		       normalize, control$statistics, control$surface,
		       control$cell, iterations, control$trace.hat)
    fit$call <- match.call()
    fit$terms <- mt
    fit$xnames <- nmx
    fit$x <- x
    fit$y <- y
    fit$weights <- w
    if(model) fit$model <- mf
    fit
}



##*******************************************************************************************
simplemultiLoess <- function(y, x, weights, span = 0.75, degree = 2,
	   normalize = TRUE,
	   statistics = "approximate", surface = "interpolate",
	   cell = 0.2, iterations = 1, trace.hat = "exact")
{

    ## Extra init
    parametric <- FALSE
    drop.square <- FALSE

    M <- NCOL(y)
    A <- rep(1,M,M)

    D <- NCOL(x)
    N <- NROW(x)

    fitted.all <- matrix(1,N,M)
    fitted.residuals <- matrix(1,N,M)
    pseudo.resid.all <-  matrix(1,N,M)

    if(!N || !D)	stop("invalid `x'")
    if(!length(y))	stop("invalid `y'")
    x <- as.matrix(x)
    max.kd <-  max(N, 200)
    robust <- rep(1, N)
    divisor<- rep(1, D)
    if(normalize && D > 1) {
	trim <- ceiling(0.1 * N)
	divisor <-
	    sqrt(apply(apply(x, 2, sort)[seq(trim+1, N-trim), , drop = FALSE],
		       2, var))
	x <- x/rep(divisor, rep(N, D))
    }
    sum.drop.sqr <- sum(drop.square)
    sum.parametric <- sum(parametric)
    nonparametric <- sum(!parametric)
    order.parametric <- order(parametric)
    x <- x[, order.parametric]
    order.drop.sqr <- (2 - drop.square)[order.parametric]
    if(degree==1 && sum.drop.sqr)
	stop("Specified the square of a factor predictor to be dropped when degree = 1")
    if(D == 1 && sum.drop.sqr)
	stop("Specified the square of a predictor to be dropped with only one numeric predictor")
    if(sum.parametric == D) stop("Specified parametric for all predictors")

    if(iterations)
    for(j in 1:iterations) {
	robust <- weights * robust
	if(j > 1) statistics <- "none"
	if(surface == "interpolate" && statistics == "approximate")
	    statistics <- if(trace.hat == "approximate") "2.approx"
	    else if(trace.hat == "exact") "1.approx"
	surf.stat <- paste(surface, statistics, sep="/")
        for(k in 1:M) {
        	z <- .C(stats:::C_loess_raw,
			as.double(y[,k]),
			as.double(x),
			as.double(weights),
			as.double(robust),
			as.integer(D),
			as.integer(N),
			as.double(span),
			as.integer(degree),
			as.integer(nonparametric),
			as.integer(order.drop.sqr),
			as.integer(sum.drop.sqr),
			as.double(span*cell),
			as.character(surf.stat),
			fitted.values = double(N),
			parameter = integer(7),
			a = integer(max.kd),
			xi = double(max.kd),
			vert = double(2*D),
			vval = double((D+1)*max.kd),
			diagonal = double(N),
			trL = double(1),
			delta1 = double(1),
			delta2 = double(1),
			as.integer(surf.stat == "interpolate/exact"))
			fitted.all[,k] <- z$fitted.values
	}

	if(j==1) {
	    trace.hat.out <- z$trL
	    one.delta <- z$delta1
	    two.delta <- z$delta2
	}

	residuals.all <- (y-fitted.all)
	fitted.residuals <- sqrt((residuals.all^2)%*%A)

	if(j < iterations)
	    robust <- .Fortran(stats:::C_lowesw,
			       as.double(fitted.residuals),
			       as.integer(N),
			       robust = double(N),
			       integer(N))$robust
    }
    if(surface == "interpolate")
    {
	pars <- z$parameter
	names(pars) <- c("d", "n", "vc", "nc", "nv", "liv", "lv")
	enough <- (D + 1) * pars["nv"]
	fit.kd <- list(parameter=pars, a=z$a[1:pars[4]], xi=z$xi[1:pars[4]],
		       vert=z$vert, vval=z$vval[1:enough])
    }
    if(iterations > 1) {
        for(k in 1:M) {
		pseudovalues <- .Fortran(stats:::C_lowesp,
					 as.integer(N),
					 as.double(y[,k]),
					 as.double(fitted.all[,k]),
					 as.double(weights),
					 as.double(robust),
					 integer(N),
					 pseudovalues = double(N))$pseudovalues
		zz <- .C(stats:::C_loess_raw,
			as.double(pseudovalues),
			as.double(x),
			as.double(weights),
			as.double(weights),
			as.integer(D),
			as.integer(N),
			as.double(span),
			as.integer(degree),
			as.integer(nonparametric),
			as.integer(order.drop.sqr),
			as.integer(sum.drop.sqr),
			as.integer(span*cell),
			as.character(surf.stat),
			temp = double(N),
			parameter = integer(7),
			a = integer(max.kd),
			xi = double(max.kd),
			vert = double(2*D),
			vval = double((D+1)*max.kd),
			diagonal = double(N),
			trL = double(1),
			delta1 = double(1),
			delta2 = double(1),
			as.integer(0))
		pseudo.resid.all[,k] <- pseudovalues-zz$temp
	}

	pseudo.resid <- sqrt((pseudo.resid.all^2)%*%A)

    }
    sum.squares <- if(iterations <= 1) sum(weights * fitted.residuals^2)
    else sum(weights * pseudo.resid^2)
    enp <- one.delta + 2*trace.hat.out - N
    s <- sqrt(sum.squares/one.delta)
    pars <- list(robust=robust, span=span, degree=degree, normalize=normalize,
		 parametric=parametric, drop.square=drop.square,
		 surface=surface, cell=cell, family=
		 if(iterations <= 1) "gaussian" else "symmetric",
		 iterations=iterations)
    fit <- list(n=N, fitted=fitted.all, residuals=residuals.all,
		enp=enp, s=s, one.delta=one.delta, two.delta=two.delta,
		trace.hat=trace.hat.out, divisor=divisor)
    fit$pars <- pars
    if(surface == "interpolate") fit$kd <- fit.kd
    class(fit) <- "loess"
    fit

}


##*******************************************************************************************

######################################################
#
# rma - RMA interface to c code
#
# the RMA method implemented in c code
#
# this code serves as interface to the c code.
# currently
# implemented (version 0.25) background correction
#
# Background correction code has been added.
#
# note this function does not leave the supplied
# AffyBatch unchanged if you select DESTRUCTIVE=TRUE. this is
# for memory purposes but can be quite
# dangerous if you are not careful. Use destructive=FALSE if this is
# deemed likely to be a problem.
#
# UPDATE: note that the affybatch is now not affected if you use
# destructive=TRUE and you might actually save a little memory.
# the destructive refers only to Plobs, which would be destroyed.
#
# History
#
# Feb 22, 2004 - activated subset. In is now possible to
#                do the entire RMA procedure using a subset of probesets
#
# Oct 26, 2007 = makesure verbosity flag is correctly passed down to C-level routines
#
# Oct 28, 2007 MM are no longer passed to the C code
#
# Jul 2, 2008 - change how probeNames (which is really probe/row indexing) is passed down to
#               c code
#
#
########################################################

rma <- function(object,subset=NULL, verbose=TRUE, destructive = TRUE,normalize=TRUE,background=TRUE,bgversion=2,...){

  rows <- length(probeNames(object,subset))
  cols <- length(object)

  if (is.null(subset)){
    ngenes <- length(geneNames(object))
  } else {
    ngenes <- length(subset)
  }

  pNList <- probeNames(object,subset)
  pNList <- split(0:(length(pNList) -1), pNList)

  if (destructive){
  	exprs <-
  	.Call("rma_c_complete",pm(object,subset), pNList, ngenes, normalize, background, bgversion, verbose, PACKAGE="affy")
  } else {
	exprs <-
  	.Call("rma_c_complete_copy", pm(object,subset), pNList, ngenes, normalize, background, bgversion, verbose, PACKAGE="affy")
  }
  colnames(exprs) <- sampleNames(object)

  new("ExpressionSet",
      phenoData = phenoData(object),
      annotation = annotation(object),
      protocolData = protocolData(object),
      experimentData = experimentData(object),
      exprs = exprs)
}

# A function that takes user inputs for correction methods for
# expresso (affy). Default values can be missing, in which case the
# first element will be chosen as the default.

expressoWidget <- function(BGMethods, normMethods, PMMethods, expMethods,
                           BGDefault, normDefault, PMDefault, expDefault){
    methodList <- list()
    END <- FALSE

    if(any(missing(BGMethods), missing(normMethods),
           missing(PMMethods), missing(expMethods))){
        stop("At least one of the method arguments is missing")
    }
    if(any(c(length(BGMethods), length(normMethods),
             length(PMMethods), length(expMethods)) == 0)){
        stop("At least one of the method argument is of length 1")
    }

    if(missing(BGDefault)){
        BGM <- tcltk::tclVar(BGMethods[1])
    }else{
        BGM <- tcltk::tclVar(BGDefault)
    }
    if(missing(normDefault)){
        NMM <- tcltk::tclVar(normMethods[1])
    }else{
        NMM <- tcltk::tclVar(normDefault)
    }
    if(missing(PMDefault)){
        PMM <- tcltk::tclVar(PMMethods[1])
    }else{
        PMM <- tcltk::tclVar(PMDefault)
    }
    if(missing(expDefault)){
        EXM <- tcltk::tclVar(expMethods[1])
    }else{
        EXM <- tcltk::tclVar(expDefault)
    }


    quit <- function(){
        tcltk::tkdestroy(base)
    }
    end <- function(){
        END <<- TRUE
        methodList[["BG"]] <<- tcltk::tclvalue(BGM)
        methodList[["NORM"]] <<- tcltk::tclvalue(NMM)
        methodList[["PM"]] <<- tcltk::tclvalue(PMM)
        methodList[["EXP"]] <<- tcltk::tclvalue(EXM)
        quit()
    }

    base <- tcltk::tktoplevel()
    ## post -- hook
    on.exit(tcltk::tkdestroy(base))

    tcltk::tktitle(base) <- "Expresso methods selection"
    ## Description text
    tcltk::tkpack(tcltk::tklabel(base, text = "Welcome to Expresso methods selection"),
           expand = FALSE, fill = "x", padx = 5, pady = 5)
    tcltk::tkpack(tcltk::tklabel(base, text = paste("You need to choose correction",
                         "methods or go with the defaults")),
           expand = FALSE, fill = "x", padx = 5)

    ## Selections for correction methods
    methodFrame <- tcltk::tkframe(base)
    ## Background selection
    BGLabel <- tcltk::tklabel(methodFrame, text = "Background correction")
    BGDropdown <- tcltk::tkframe(methodFrame)
    widgetTools::dropdownList(BGDropdown, BGMethods, BGM, 20,
                                      tcltk::tclvalue(BGM), TRUE)
    tcltk::tkgrid(BGLabel, BGDropdown)
    tcltk::tkgrid.configure(BGLabel, sticky = "e")
    tcltk::tkgrid.configure(BGDropdown, sticky = "w")

    ## Normlization
    NMLabel <- tcltk::tklabel(methodFrame, text = "Normalization")
    NMDropdown <- tcltk::tkframe(methodFrame)
    widgetTools::dropdownList(NMDropdown,normMethods, NMM, 20,
                                      tcltk::tclvalue(NMM), TRUE)
    tcltk::tkgrid(NMLabel, NMDropdown)
    tcltk::tkgrid.configure(NMLabel, sticky = "e")
    tcltk::tkgrid.configure(NMDropdown, sticky = "w")

    ## PM correction
    PMLabel <- tcltk::tklabel(methodFrame, text = "PM correction")
    PMDropdown <- tcltk::tkframe(methodFrame)
    widgetTools::dropdownList(PMDropdown, PMMethods, PMM, 20,
                                      tcltk::tclvalue(PMM), TRUE)
    tcltk::tkgrid(PMLabel, PMDropdown)
    tcltk::tkgrid.configure(PMLabel, sticky = "e")
    tcltk::tkgrid.configure(PMDropdown, sticky = "w")

    ## PM correction
    EXLabel <- tcltk::tklabel(methodFrame, text = "Expression")
    EXDropdown <- tcltk::tkframe(methodFrame)
    widgetTools::dropdownList(EXDropdown, expMethods, EXM, 20,
                                      tcltk::tclvalue(EXM), TRUE)
    tcltk::tkgrid(EXLabel, EXDropdown)
    tcltk::tkgrid.configure(EXLabel, sticky = "e")
    tcltk::tkgrid.configure(EXDropdown, sticky = "w")

    tcltk::tkpack(methodFrame, expand = TRUE, fill = "both", padx = 5,
           pady = 10)

    butFrame <- tcltk::tkframe(base)
    quitBut <- tcltk::tkbutton(butFrame, text = "Quit", width = 7, command = quit)
    endBut <- tcltk::tkbutton(butFrame, text = "Select", width = 7, command = end)
    tcltk::tkgrid(quitBut, endBut, padx = 5)
    tcltk::tkpack(butFrame, expand = FALSE, fill = "x", pady = 5)

    tcltk::tkwait.window(base)

    if(END){
        return(methodList)
    }else{
        return(NULL)
    }
}

avdiff <- function(x,verbose=FALSE){
  if(missing(x)) stop("Argument x missing, with no default\n")
  cat("Computing average difference for",dim(x$pm)[2],"columns")
  avdiff <- apply(x$pm-x$mm,2,function(y){
    cat(".")
    tapply(y,x$name,function(z){
      o <- order(z)
      zz <- z[-c(o[1],o[length(z)])] #take out biggest and smallest
      mean(z[abs(z-mean(zz))<3*sd(zz)])
    })
  })
  colnames(avdiff) <- x$chip.names
  return(avdiff)
}

fit.li.wong <- function(data.matrix, remove.outliers=TRUE,
                        normal.array.quantile=0.5,
                        normal.resid.quantile=0.9,
                        large.threshold=3,
                        large.variation=0.8,
                        outlier.fraction=0.14,
                        delta = 1e-06,maxit=50,outer.maxit=50, verbose=FALSE, ...){
  if(missing(data.matrix)) stop("Argument data.matrix missing, with no default")
  II <- dim(data.matrix)[1] ##II instrad of I cause I is a fuction in R
  J <- dim(data.matrix)[2]
  if(J==1){
    warning("Li and Wong's algorithm is not suitable when only one probe pair")
    return(list(theta = as.vector(data.matrix), phi = 1, sigma.eps = NA, sigma.theta = NA, sigma.phi=NA, theta.outliers=NA, phi.outliers=NA, single.outliers=NA,convergence1=NA,convergence2=NA,iter = NA, delta = NA))
  }
  cI <- II ##current I
  cJ <- J ##current J
  theta.outliers.old <- rep(FALSE, II) ##ith entry will be true if theta_i is an outlier
  phi.outliers.old <- rep(FALSE, J) ##jth entry will be true if phi_j is an outlier
  single.outliers.old <- matrix(FALSE, II, J) ##ij entry will be true if y_is an outlier
  theta.outliers <- theta.outliers.old ##need this to now if change ocurred in outliers
  phi.outliers <- phi.outliers.old ##need this to know if chages occured in outlies
  single.outliers <- single.outliers.old
  flag1 <- NA ##these will be false if convergence not reacher,
  flag2 <- NA ## this will be false if outliers respectively cuase iter to stop
  if(remove.outliers){
    flag1 <- TRUE; flag2<-TRUE
    original.data.matrix <- data.matrix ##so we can get it back after outlier removal
    change.theta <- 1 #start with 1
    change.phi <- 1
    change.single <- 1
    outer.iter <- 0
    while(flag1 & flag2 & change.theta+change.phi+change.single >0 & outer.iter < outer.maxit) {
      outer.iter <- outer.iter + 1

      if((outer.iter%%3==0 & change.theta>0)  |
         (outer.iter%%3==1 & change.phi>0)){ #something has to change
        ##starting values
        phi <- colMeans(data.matrix)
        c <- sqrt(cJ/sum(phi[!phi.outliers]^2))
        phi <- c * phi
        theta <- (data.matrix[, !phi.outliers, drop=FALSE] %*% phi[!phi.outliers, drop=FALSE])/cJ
        iter <- 0
        change <- 1 #start with one
        theta.old <- rep(0, II)
        while(change > delta & iter < maxit) {
          iter <- iter + 1
          phi <- t(data.matrix[!theta.outliers, ,drop=FALSE]) %*% theta[!theta.outliers, drop=FALSE] ##ignore the outliers
          c <- sqrt(cJ/sum(phi[!phi.outliers, drop=FALSE]^2))
          phi <- c * phi
          theta <- (data.matrix[,!phi.outliers, drop=FALSE] %*% phi[!phi.outliers, drop=FALSE])/cJ
          change <- max(abs(theta[!theta.outliers] - theta.old[!theta.outliers]))
          if(verbose) cat(paste("Outlier iteration:",outer.iter,"estimation iteration:",iter,"chage=",change,"\n"))
          theta.old <- theta
        }
        if(iter>=maxit){ ##convergence not reached. might as well get out
          warning(paste("No convergence in inner loop after",iter,"in outerler tieration",outer.iter,"\n"))
          flag1 <- FALSE
        }
        if(mean(phi[!phi.outliers]<0)>.5){ ##for identifiability.. theta*phi = (-theta)*(-phi), i require that most phis are positive
          theta <- -theta
          phi <- -phi
        }
        theta <- as.vector(theta)
        phi <- as.vector(phi)

        data.matrixhat <- outer(theta, phi)
        resid <- data.matrix-data.matrixhat
      }
      ##DEALING WITH OUTLIERS
      ##we alternate removal of outliers
      ##if even iteration take out thetas that are outliers (as defined by Li and Wong).
      if(outer.iter%%3==1){ ## we start with single outliers
        single.outliers <- resid > large.threshold*quantile(abs(resid),normal.resid.quantile)
        single.outliers[rowSums(single.outliers) > outlier.fraction*cJ,]<-rep(FALSE,J)
        ##probably chip oulier, defer calling outlier
        single.outliers[,colSums(single.outliers) > outlier.fraction*cI]<-rep(FALSE,II)
        ##probably probe outlier, defer calling outlier
        data.matrix[single.outliers] <- data.matrixhat[single.outliers]
        data.matrix[!single.outliers] <- original.data.matrix[!single.outliers]
        change.single <- sum(abs(single.outliers.old-single.outliers)) #sum will be total of changes
        single.outliers.old <- single.outliers
      }
      else{
        sigma.theta <- sqrt(rowSums(resid[, !phi.outliers, drop=FALSE]^2)/(cJ - 1))
        sigma.phi <- sqrt(colSums(resid[!theta.outliers, , drop=FALSE]^2)/(cI - 1))
        ###THETA OUTLIERS
        if(outer.iter%%3==2){
          theta.outliers <- sigma.theta > large.threshold*quantile(sigma.theta,normal.array.quantile) | theta^2/sum(theta^2) > large.variation
          cI <- sum(!theta.outliers)
          if(cI<3) {
            warning("No convergence achieved, too many outliers")
            flag2 <- FALSE
          }
          ##single outliers in outlier chips are not longer single outliers
          single.outliers[theta.outliers,] <- rep(FALSE,J)
          data.matrix[single.outliers] <- data.matrixhat[single.outliers]
          data.matrix[!single.outliers]<-original.data.matrix[!single.outliers]
          change.theta <- sum(abs(theta.outliers.old-theta.outliers)) #sum will be total of changes
          change.single <- sum(abs(single.outliers.old-single.outliers)) #sum will be total of changes
          theta.outliers.old <- theta.outliers
        }
        ##PHI OUTLIERS
        else{
          phi.outliers <- sigma.phi > large.threshold*quantile(sigma.phi,normal.array.quantile) | phi^2/sum(phi^2) > large.variation | phi <0
          cJ <- sum(!phi.outliers)
          if(cJ<3) {
            warning("No convergence achieved, too many outliers")
            flag2 <- FALSE
          }
          single.outliers[,phi.outliers] <- rep(FALSE,II)
          data.matrix[single.outliers] <- data.matrixhat[single.outliers]
          data.matrix[!single.outliers]<-original.data.matrix[!single.outliers]
          change.phi <- sum(abs(phi.outliers.old-phi.outliers))
          change.single <- sum(abs(single.outliers.old-single.outliers)) #sum will be total of changes
          phi.outliers.old <- phi.outliers
        }
      }
      if(verbose){
        cat("chips used=",cI,", probes used=",cJ,", single outler=",sum(single.outliers),"\n")
        cat("Number of changes: single=",change.single,", theta=",change.theta,", phi=",change.phi,"\n",sep="")
      }
    }
    if(outer.iter>=outer.maxit){
      warning("No convergence achieved in outlier loop\n")
      flag2 <- FALSE
    }
    all.outliers <- outer(theta.outliers,phi.outliers,FUN="|") | single.outliers
    sigma <- sqrt(sum(resid[!all.outliers]^2)/sum(!all.outliers))
    ##in case we leave iteration and these havent been defined
    sigma.theta <- sqrt(rowSums(resid[,!phi.outliers, drop=FALSE]^2)/(cJ - 1))
    sigma.phi <- sqrt(colSums(resid[!theta.outliers, ,drop=FALSE]^2)/(cI - 1))
  }
  ###code for NO OUTLIER REMOVAL
  else{
    flag1 <- TRUE
    phi <- colMeans(data.matrix)
    c <- sqrt(J/sum(phi^2))
    phi <- c * phi
    theta <- (data.matrix %*% phi)/J

    iter <- 0
    change <- 1
    theta.old <- rep(0, II)
    while(change > delta & iter < maxit) {
      iter <- iter + 1
      phi <- t(data.matrix) %*% theta
      c <- sqrt(J/sum(phi^2))
      phi <- c * phi
      theta <- (data.matrix %*% phi)/J
      change <- max(abs(theta - theta.old))
      if(verbose) cat(paste("Iteration:",iter,"chage=",change,"\n"))
      theta.old <- theta
    }
    if(iter>=maxit){
      warning(paste("No convergence after",iter,"iterations.\n"))
      flag1 <- FALSE
    }
    if(mean(phi[!phi.outliers]<0)>.5){
      ##for identifiability.. theta*phi = (-theta)*(-phi), i require that most phis are positive
      theta <- -theta
      phi <- -phi
    }
    theta <- as.vector(theta)
    phi <- as.vector(phi)
    data.matrixhat <- outer(theta, phi)
    sigma.theta <- sqrt(rowSums((data.matrix - data.matrixhat)^2)/(J - 1))
    sigma.phi <- sqrt(colSums((data.matrix - data.matrixhat)^2)/(II - 1))
    sigma <- sqrt(sum((data.matrix - data.matrixhat)^2)/(II * J))
  }
  return(list(theta = theta, phi = phi, sigma.eps = sigma, sigma.theta = sigma.theta, sigma.phi=sigma.phi,theta.outliers=theta.outliers,phi.outliers=phi.outliers,single.outliers=single.outliers,convergence1=flag1,convergence2=flag2,iter = iter, delta = change))
}

generateExprVal.method.playerout <- function(probes, weights=FALSE, optim.method="L-BFGS-B"){
  
  probes <- t(probes)
  nprobes <- ncol(probes)
  
  ## skip if only one probe
  if (nprobes == 1) return(t(probes))
  
  ## I do not know to which extend the use of optim
  ## is really equivalent to the use of nlminb in S-plus
  S1 <- optim(runif(nprobes),
              playerout.costfunction,
              method=optim.method,
              control=list(maxit=500),
              y=probes)
  ##S1 <- nlm(playerout,runif(20),iterlim=500,y=t(y))
  r <- c(probes %*% S1$par / sum(S1$par))
  if (weights)
    attr(r,"weights") <- S1$par
  return(list(exprs=r,se.exprs=rep(NA,length(r))))
}


## The loss function:

playerout.costfunction <- function(w, y) {
  N <- length(w)        # Number of players
  J <- length(y)/N      # Number of games (the number of games is the number of chips used)
  sumw <- sum(w)
  
  tx <- y %*% w    # Full weighted score at each game  
  pl <- matrix(0,J,N)    # Loss at each game due to each player
  
  for(j in 1:J)
    pl[j,] <- w * y[j,] - (tx[j] - w * y[j,]) / (sumw - w)
  
  sum(pl^2)         # Loss
}

"AffyRNAdeg" <-
function (abatch,log.it=TRUE)
{
    {
        data <- pm(abatch, LIST = TRUE)
        if(log.it==TRUE) data <- lapply(data,log2)
        names <- colnames(exprs(abatch))
        probe.set.size <- function(x) {
            size <- dim(x)[1]
            return(size)
        }
        max.num <- sapply(data, probe.set.size)
        tab <- (table(max.num))
        ord <- order(-as.numeric(tab))
        K <- as.numeric(names(tab))[ord[1]]
        data <- data[max.num == K]
    }
    get.row <- function(x, i = 1) {
        return(x[i, ])
    }
    get.col <- function(x, i = 1) {
        return(x[, i])
    }
    rowstack <- function(x, i = 1) {
        return(t(sapply(x, get.row, i)))
    }
    colstack <- function(x, i = 1) {
        return(t(sapply(x, get.col, i)))
    }
    N <- length(data)

    n <- dim(data[[1]])[2]
    mns <- matrix(nrow = n, ncol = K)
    sds <- mns
    for (i in 1:K) {
        data.stack <- rowstack(data, i)
        if(dim(data[[1]])[2]==1) data.stack <- t(data.stack)
        mns[, i] <- colMeans(data.stack)
        sds[, i] <- apply(data.stack, 2, sd)
    }
    mns.orig <- mns
    mn <- mns[, 1]
    mns <- sweep(mns, 1, mn)
    mns <- mns/(sds/sqrt(N))
    lm.stats <- function(x) {
        index <- 0:(length(x) - 1)
        ans <- summary(lm(x ~ index))$coefficients[2, c(1, 4)]
        return(ans)
    }
    stats <- apply(mns, 1, lm.stats)
    answer <- list(N, names, mns.orig, sds/sqrt(N), stats[1,
        ], stats[2, ])
    names(answer) <- c("N", "sample.names", "means.by.number",
        "ses", "slope", "pvalue")
    return(answer)
}
"summaryAffyRNAdeg" <-
function (rna.deg.obj, signif.digits = 3)
{
  temp.table <- rbind(signif(rna.deg.obj$slope, signif.digits),
                      signif(rna.deg.obj$pvalue, signif.digits))
  colnames(temp.table) <- rna.deg.obj$sample.names
  rownames(temp.table) <- c("slope", "pvalue")
  ##write.table(temp.table, file = "", quote = FALSE)
  return(temp.table)
}
"plotAffyRNAdeg" <-
function (rna.deg.obj,transform="shift.scale",cols=NULL, ...)
{
    if(!is.element(transform,c("shift.scale","shift.only","neither"))) stop("Tranform must be 'shift.scale','shift.only', or 'neither'")
    mns <- rna.deg.obj$means.by.number
    if(is.null(cols)) cols=rep(4,dim(mns)[1])
    ylab="Mean Intensity"
    if(transform=="shift.scale"){
    sds <- rna.deg.obj$ses
    mn <- mns[, 1]
    mns <- sweep(mns, 1, mn)
    mns <- mns/(sds)
    mns <- sweep(mns, 1, 1:(dim(mns)[1]), "+")
    ylab <- paste(ylab,": shifted and scaled")
   }else if(transform=="shift.only"){
    mn <- mns[, 1]
    mns <- sweep(mns, 1, mn)
    mns <- sweep(mns, 1, 1:(dim(mns)[1]), "+")
    ylab <- paste(ylab,": shifted")
   }
    plot(-2, -1, pch = "", xlim = range(-1, (dim(mns)[2])),
        ylim = range(min(as.vector(mns)) - 1, max(as.vector(mns)) + 1), xlab = "5' <-----> 3'\n Probe Number ",
        ylab = ylab, axes = FALSE, main = "RNA degradation plot",
        ...)
    axis(1)
    axis(2)
    for (i in 1:dim(mns)[1]) lines(0:((dim(mns)[2]-1)), mns[i, ],col=cols[i])
}

ppset.ttest <- function(ppset, covariate, pmcorrect.fun = pmcorrect.pmonly, ...) {
  probes <- do.call(pmcorrect.fun, list(ppset))
  my.ttest <- function(x) {
    y <- split(x, get(covariate))
    t.test(y[[1]], y[[2]])$p.value
  }
  r <- apply(probes, 1, my.ttest)
  return(r)
}

debug.affy123 <- FALSE

setClass("ProgressBarText", representation(steps = "integer", barsteps = "integer", internals = "environment"))

setMethod("initialize", "ProgressBarText",
          function(.Object, steps, barsteps = 10, internals = NULL) {
            ##.Object <- callNextMethod()
            if ( ! is.null(internals)) {
              stop("slot 'internals' is for internal use !")
            }
            .Object@barsteps = barsteps
            .Object@internals = new.env()
            assign("milestones.i", as.integer(1), envir=.Object@internals)
            assign("increment", as.integer(1), envir=.Object@internals)
            assign("milestones", as.integer(seq(1, steps, length=barsteps)), envir=.Object@internals)
            assign("i", as.integer(0), envir=.Object@internals)
            return(.Object)
          })

setMethod("open", "ProgressBarText",
          function(con, header = TRUE) {
            if (header) {
              cat("|", paste(rep(" ", con@barsteps), collapse=""), "|\n", sep="")
            }
            cat("|")
            increment <- get("increment", con@internals)
            milestones.i <- get("milestones.i", con@internals)
            milestones <- get("milestones", con@internals)
            while(milestones.i > length(milestones)) {
              cat("#")
            }
            if (.Platform$OS.type == "windows")
            flush.console()
          })

## to avoid 'loosing' the default update.
## (not sure this is the most elegant way to do this)

setGeneric("updateMe", function(object, ...) standardGeneric("updateMe"))

setMethod("updateMe", "ProgressBarText",
          function(object) {
            increment <- get("increment", object@internals)
            i <- get("i", object@internals) + increment
            milestones.i <- get("milestones.i", object@internals)
            milestones <- get("milestones", object@internals)
            touched <- FALSE
            while(milestones.i <= length(milestones) && i >= milestones[milestones.i]) {
              cat("#")
              milestones.i <- milestones.i + increment
              touched <- TRUE
            }
            ## the 'touch' thing appears to make it save 0.1 sec / 100000 iteration
            ## (which makes it absolutely mandatory :) ).
            if (touched) {
              assign("milestones.i", milestones.i, envir = object@internals)
              if (.Platform$OS.type == "windows")
                flush.console()
            }
            assign("i", i, , envir = object@internals)
          })


setMethod("close", "ProgressBarText",
          function(con) {
            increment <- get("increment", con@internals)
            milestones.i <- get("milestones.i", con@internals)
            milestones <- get("milestones", con@internals)
            while(milestones.i <= length(milestones)) {
              cat("#")
              milestones.i <- milestones.i + increment
            }
            assign("milestones.i", milestones.i, envir = con@internals)
            cat("|\n")
            if (.Platform$OS.type == "windows")
              flush.console()
          })

##RG added a method to store and update the available normalization methods

 normalize.AffyBatch.methods <- function() 
            .affyInternalEnv[["normalize.AffyBatch.methods"]]

 upDate.normalize.AffyBatch.methods <- function(x) {
            if (is.character(x))
               .affyInternalEnv[["normalize.AffyBatch.methods"]] <- x
  }

 express.summary.stat.methods <- function()
            .affyInternalEnv[["express.summary.stat.methods"]]

 upDate.express.summary.stat.methods <- function(x) {
            if (is.character(x))
               .affyInternalEnv[["express.summary.stat.methods"]] <- x
  }

 generateExprSet.methods <- function()
            .affyInternalEnv[["generateExprSet.methods"]]

 upDate.generateExprSet.methods <- function(x) {
            if (is.character(x))
               .affyInternalEnv[["generateExprSet.methods"]] <- x
  }

 bgcorrect.methods <- function()
            .affyInternalEnv[["bgcorrect.methods"]]

 upDate.bgcorrect.methods <- function(x) {
            if (is.character(x))
               .affyInternalEnv[["bgcorrect.methods"]] <- x
      }
 pmcorrect.methods <- function()
            .affyInternalEnv[["pmcorrect.methods"]]

 upDate.pmcorrect.methods <- function(x) {
            if (is.character(x))
               .affyInternalEnv[["pmcorrect.methods"]] <- x
    }
 
## computeExprSet:
##   - better reporting of errors
##   - better handling of ids (does not crash any longer when unknown id)
##   - use of the progress bar in Biobase 1.4.4
##   - cleanup of the comments in the code
## indexProbes:
##   - deprecated flag 'xy' removed for good


if (debug.affy123) cat("-->initAffyBatch\n")

## Inherits from Affybatch
## The accessor 'intensity' gets what is in the slot 'exprs'
setClass("AffyBatch",
         representation=representation(
           cdfName="character",
           nrow="numeric",
           ncol="numeric"),
         contains="eSet",
         prototype=prototype(
           new("VersionedBiobase",
               versions=c(classVersion("eSet"), AffyBatch="1.2.0"))))


setMethod("initialize",
          signature(.Object="AffyBatch"),
          ## provide a signature consistent with the new defintion, but that picks up the (implicit) old
          function(.Object,
                   cdfName="", nrow=0, ncol=0,
                   phenoData, featureData,
                   experimentData=new("MIAME"),
                   annotation=character(0),
                   protocolData=phenoData[,integer(0)],
                   assayData,
                   exprs=matrix(numeric(0), nrow=nrow, ncol=ncol),
                   ## se.exprs
                   ...) {
              .Object@cdfName <- cdfName
              .Object@nrow <- nrow
              .Object@ncol <- ncol
              dots <- list(...)
              ## remove reporterInfo, description, notes from '...'
              if ("reporterInfo" %in% names(dots)) {
                  if (missing(featureData)) {
                      if (!is.null(dots[["reporterInfo"]])) {
                          if (is(dots[["reporterInfo"]], "data.frame")) {
                            featureData <- new("AnnotatedDataFrame", data=dots[["reporterInfo"]])
                          } else {
                              warning("trying to convert reporterInfo (class '", class(dots[["reporterInfo"]]),
                                      "') to featureData (class 'AnnotatedDataFrame')",
                                      immediate.=TRUE)
                              featureData <- as(dots[["reporterInfo"]], "AnnotatedDataFrame")
                          }
                      } else {
                        stop("use 'featureData' rather than 'reporterInfo' for feature covariates")
                      }
                  }
              }
              if ("description" %in% names(dots)) {
                  if (missing(experimentData))
                    experimentData <- dots[["description"]]
                  else
                    stop("use 'experimentData' rather than 'description' for experiment description")
              }
              if ("notes" %in% names(dots)) {
                  ## warning("addding 'notes' to 'experimentData'")
                  notes(experimentData) <- c(notes(experimentData), dots[["notes"]])
              }
              dots <- dots[!names(dots) %in% c("reporterInfo", "description", "notes")]
              ## update phenoData to AnnotatedDataFrame, if necessary
              if (missing(assayData)) {
                assayData <- do.call(assayDataNew, c(list(exprs=exprs), dots))
              } else if (!missing(exprs)) {
                stop("cannot initialize when both 'assayData' and 'exprs' are specified")
              }
              if (missing(phenoData) || is.null(phenoData))
                phenoData <- annotatedDataFrameFrom(assayData, byrow=FALSE)
              else if (!is(phenoData, "AnnotatedDataFrame"))
                phenoData <- as(phenoData, "AnnotatedDataFrame")
              if (missing(featureData))
                featureData <- annotatedDataFrameFrom(assayData, byrow=TRUE)
              callNextMethod(.Object,
                             assayData=assayData,
                             phenoData=phenoData,
                             featureData=featureData,
                             experimentData=experimentData,
                             annotation=annotation,
                             protocolData=protocolData)
          })

setMethod("updateObject",
          signature(object="AffyBatch"),
          function(object, ..., verbose=FALSE) {
              if (verbose) message("updateObject(object = 'AffyBatch'")
              if (isVersioned(object) && isCurrent(object)["AffyBatch"])
                return(callNextMethod())
              if (!isVersioned(object) || is.null(classVersion(object)[["AffyBatch"]])) {
                  exprs <- slot(object, "exprs")
                  se.exprs <- slot(object, "se.exprs")
                  if (!all(dim(se.exprs) == dim(exprs))) {
                      if (!all(dim(se.exprs)==0))
                        warning("removing 'se.exprs' with dimensions different from 'exprs'")
                      se.exprs <- NULL
                  }
                  if ("reporterInfo" %in% names(attributes(object)) &&
                      any(dim(slot(object, "reporterInfo"))!=0))
                    warning("reporterInfo data not transfered to 'AffyBatch' object")
                  experimentData=updateObject(slot(object, "description"))
                  if ("notes" %in% names(attributes(object)) &&
                      length(slot(object, "notes"))!=0) {
                      warning("adding 'notes' to 'experimentData'")
                      notes(experimentData) <- c(notes(experimentData), object@notes)
                  }
                  if (!is.null(se.exprs))
                    new("AffyBatch",
                        phenoData=as(slot(object, "phenoData"), "AnnotatedDataFrame"),
                        experimentData=experimentData,
                        annotation=slot(object, "annotation"),
                        cdfName=slot(object, "cdfName"),
                        nrow=slot(object, "nrow"),
                        ncol=slot(object, "ncol"),
                        exprs=exprs,
                        se.exprs=se.exprs)
                  else
                    new("AffyBatch",
                        phenoData=as(slot(object, "phenoData"), "AnnotatedDataFrame"),
                        experimentData=experimentData,
                        annotation=slot(object, "annotation"),
                        cdfName=slot(object, "cdfName"),
                        nrow=slot(object, "nrow"),
                        ncol=slot(object, "ncol"),
                        exprs=exprs)
              } else if (!isCurrent(object)[["AffyBatch"]]) {
                  ## remove empty se.exprs, introduced in class version 1.2.0
                  se.exprs <- assayData(object)[["se.exprs"]]
                  if (!is.null(se.exprs) && all(is.na(se.exprs))) {
                      message("removing 'se.exprs' with all NA values")
                      se.exprs(object) <- NULL
                  }
                  object <- callNextMethod()
                  classVersion(object)["AffyBatch"] <- classVersion("AffyBatch")["AffyBatch"]
                  object
              } else object
          })

#######################################################
### accessors
#######################################################

if (debug.affy123) cat("--->accessors\n")

setMethod("exprs",
          signature(object="AffyBatch"),
          function(object) assayData(object)[["exprs"]])

setReplaceMethod("exprs",
          signature(object="AffyBatch"),
          function(object, value) {
    assayDataElementReplace(object, "exprs", value, validate=FALSE)
})

setMethod("se.exprs",
          signature(object="AffyBatch"),
          function(object) {
              obj <- assayData(object)[["se.exprs"]]
              if (is.null(obj))
                new("matrix")
              else
                obj
          })


setReplaceMethod("se.exprs",
          signature(object="AffyBatch"),
          function(object, value) {
    assayDataElementReplace(object, "se.exprs", value, validate=FALSE)
})

if (is.null(getGeneric("cdfName")))
    setGeneric("cdfName", function(object)
               standardGeneric("cdfName"))

setMethod("cdfName", "AffyBatch", function(object)
          object@cdfName)

##intensity
setGeneric("intensity", function(object)
           standardGeneric("intensity"))

setMethod("intensity", signature(object="AffyBatch"),
          function(object) assayData(object)[["exprs"]])

setGeneric("intensity<-", function(object, value)
           standardGeneric("intensity<-"))

setReplaceMethod("intensity", signature(object="AffyBatch"),
                 function(object, value) {
    assayDataElementReplace(object, "exprs", value, validate=FALSE)
})

##for now, there is no accessor for se.exprs. we could use this to store
##sd, but if no one uses it... why do it

setMethod("length",signature(x="AffyBatch"),
          function(x) ncol(exprs(x))) ##RI: assumes matrices

setMethod("dim",
          signature=signature(x="AffyBatch"),
          function(x) c(x@nrow, x@ncol))

#######################################################
### methods
#######################################################

setMethod("featureNames",
          signature=signature(object="AffyBatch"),
          function(object) {
              cdf.envir <- getCdfInfo(object)
              ls(envir=cdf.envir)
          })

setReplaceMethod("featureNames",
                 signature=signature(
                   object="AffyBatch",
                   value="ANY"),
                 function(object, value)
                 stop("Cannot change featureNames of AffyBatch"))

##geneNames method
if (debug.affy123) cat("--->geneNames\n")

if( is.null(getGeneric("geneNames") ))
  setGeneric("geneNames", function(object)
             standardGeneric("geneNames"))

setMethod("geneNames",signature("AffyBatch"),
            function(object){
              cdf.envir <- getCdfInfo(object)
              return(ls(envir=cdf.envir))
            })


##show method
if (debug.affy123) cat("--->show\n")

setMethod("show", "AffyBatch",
          function(object) {
              if (!isVersioned(object) ||
                  classVersion(object)["AffyBatch"] < "1.1.0")
                stop("AffyBatch out-of-date; use 'updateObject(<AffyBatch>)'",
                     call.=FALSE)
              if (!isCurrent(object)['AffyBatch'])
                message("AffyBatch out-of-date; consider 'updateObject(<AffyBatch>)'")
              ## Location from cdf env
              cdf.env <-
                tryCatch(getCdfInfo(object),
                         error=function(err) {
                             warning("missing cdf environment! in show(AffyBatch)", call.=FALSE)
                             NULL
                         })
              num.ids <-
                if (!is.null(cdf.env)) length(ls(envir=cdf.env))
                else num.ids <- "???"

              cat("AffyBatch object\n")
              cat("size of arrays=", nrow(object), "x", ncol(object),
                  " features (", object.size(object) %/% 1024, " kb)\n", sep="")
              cat("cdf=", object@cdfName, " (", num.ids, " affyids)\n", sep="")
              cat("number of samples=", length(object), "\n", sep="")
              cat("number of genes=", length(featureNames(object)), "\n", sep="")
              cat("annotation=", object@annotation, "\n", sep="")
              if(length(notes(object)) > 0)
                cat("notes=",paste(notes(object),collapse="\n\t"), "\n", sep="")
          })


# if (is.null(getGeneric("index2xy"))) {
#   setGeneric("indexProbes", function(object, which, ...)
#              standardGeneric("indexProbes"))
# }


## indexProbes
if( is.null(getGeneric("indexProbes")))
  setGeneric("indexProbes", function(object, which, ...)
             standardGeneric("indexProbes"))

setMethod("indexProbes",
          signature=c(object="AffyBatch", which="missing"),
          function(object, which, ...) indexProbes(object, which="pm", ...))

setMethod("indexProbes", signature("AffyBatch", which="character"),
          function(object, which=c("pm", "mm","both"),
                   genenames=NULL) {

            which <- match.arg(which)

            i.probes <- match(which, c("pm", "mm", "both"))
            ## i.probes will know if "[,1]" or "[,2]"
            ## if both then [,c(1,2)]
            if(i.probes==3) i.probes=c(1,2)

            envir <- getCdfInfo(object)

            if(is.null(genenames))
              genenames <- ls(envir )

            ## note: the variable name genenames could be confusing (the same gene can be
            ## found in several affyid (ex: the 3' and 5' controls)

            ans <-  mget(genenames, envir, ifnotfound=NA)

            ## this kind of thing could be included in 'mget' as
            ## an extra feature. A function could be specified to
            ## process what is 'multi'-get on the fly
            for (i in seq(along=ans)) {

              if ( is.na(ans[[i]][1]) )
                next

              ##as.vector cause it might be a matrix if both
              tmp <- as.vector(ans[[i]][, i.probes])

              ans[[i]] <- tmp
            }

            return(ans)
          })


  ##pmindex method
if( is.null(getGeneric("pmindex")))
  setGeneric("pmindex", function(object,...)
             standardGeneric("pmindex"))

##wrapper
setMethod("pmindex", "AffyBatch",
            function(object, genenames=NULL)
          indexProbes(object, "pm", genenames=genenames))

  ##mmindex method
if( is.null(getGeneric("mmindex")))
  setGeneric("mmindex", function(object,...)
             standardGeneric("mmindex"))

##wrapper
setMethod("mmindex", "AffyBatch",
          function(object,genenames=NULL)
          indexProbes(object, "mm", genenames=genenames))


##probeNames method
if( is.null(getGeneric("probeNames")))
  setGeneric("probeNames", function(object, ...)
             standardGeneric("probeNames"))

setMethod("probeNames","AffyBatch",
          function(object, genenames=NULL, mm=FALSE){
            if(mm) Index <- mmindex(object,genenames)
            else Index <- pmindex(object,genenames)
            reps <- unlist(lapply(Index,length),use.names=FALSE)
            rep(names(Index),reps)
          })


if( is.null(getGeneric("probes")) )
  setGeneric("probes", function(object, ...)
             standardGeneric("probes"))

setMethod("probes", signature("AffyBatch"),
          function(object, which=c("pm", "mm"),
                   genenames=NULL, LISTRUE=FALSE, drop=FALSE){

            which <- match.arg(which)

            index <- indexProbes(object, which, genenames)

            if(LISTRUE)
              ans <- lapply(index, function(i) exprs(object)[i, ,drop=drop])
            else{
              index <- unlist(index)
              ans <- exprs(object)[index, ,drop=drop]
              colnames(ans) <- sampleNames(object)
              rownames(ans) <- names(index)
            }

            return(ans)
          })

##pm method
if( is.null(getGeneric("pm") ))
  setGeneric("pm", function(object, ...)
             standardGeneric("pm"))

setMethod("pm","AffyBatch",
          function(object, genenames=NULL, LISTRUE=FALSE){
            if(is.null(genenames) & !LISTRUE){
              cdfname <- getCdfInfo(object)
              psets<- as.list(cdfname)
              psets<- psets[order(names(psets))]
              index <-unlist(sapply(psets, function(x) x[,1]),use.names=FALSE)
              return(exprs(object)[index,,drop=FALSE])
            }
            else{
              return(probes(object, "pm", genenames, LISTRUE=LISTRUE))
            }
          })

if( is.null(getGeneric("pm<-") ))
  setGeneric("pm<-", function(object, value)
             standardGeneric("pm<-"))

setReplaceMethod("pm", "AffyBatch", function(object, value){
  Dimnames <- dimnames(exprs(object))
  cdfname <- getCdfInfo(object)
  psets<- as.list(cdfname)
  psets<- psets[order(names(psets))]
  pmIndex <-unlist(sapply(psets, function(x) x[,1]),use.names=FALSE)

  exprs(object)[pmIndex,] <- value
  dimnames(exprs(object)) <- Dimnames
  object
})



##mm method
if( is.null(getGeneric("mm") ))
  setGeneric("mm", function(object, ...)
             standardGeneric("mm"))

setMethod("mm",signature("AffyBatch"),
          function(object, genenames=NULL, LISTRUE=FALSE){
            if(is.null(genenames) & !LISTRUE){
              cdfname <- getCdfInfo(object)
              psets<- as.list(cdfname)
              psets<- psets[order(names(psets))]
              index <-unlist(sapply(psets, function(x) x[,2]),use.names=FALSE)
              return(exprs(object)[index,,drop=FALSE])
            }
            else{
              probes(object, "mm", genenames, LISTRUE=LISTRUE)
            }
          })


if( is.null(getGeneric("mm<-") ))
  setGeneric("mm<-", function(object, value)
             standardGeneric("mm<-"))

setReplaceMethod("mm", "AffyBatch", function(object, value){
  Dimnames <- dimnames(exprs(object))
  cdfname <- getCdfInfo(object)
  psets<- as.list(cdfname)
  psets<- psets[order(names(psets))]
  mmIndex <-unlist(sapply(psets, function(x) x[,2]),use.names=FALSE)
  exprs(object)[mmIndex,] <- value
  dimnames(exprs(object)) <- Dimnames
  object
})

###probeset
  setGeneric("probeset", function(object, ...)
             standardGeneric("probeset"))

setMethod("probeset", "AffyBatch", function(object, genenames=NULL,
                                            locations=NULL){
  oldoptions <- getOption("BioC")

  if(is.null(locations)) ##use info in cdf
    envir <- getCdfInfo(object)
  else{
    ##if the user gives a list of locations let them use that as enviromnet
    envir <- new.env()
    multiassign(names(locations), locations, envir)
    object@cdfName <- "envir"
    newoptions <- oldoptions
    newoptions$affy$probesloc[[1]]$what <- "environment"
    newoptions$affy$probesloc[[1]]$where <- parent.env(envir)
    options("BioC"=newoptions)
  }
  if(is.null(genenames))
    genenames <- ls(envir)

  p.pps <- vector("list", length(genenames))
  names(p.pps) <- genenames

  for (i in seq(along=genenames)) {

    i.pm <- indexProbes(object, "pm", genenames[i])[[1]]
    if (is.na(i.pm)[1])
      intensity.pm <- matrix()
    else
      intensity.pm <- intensity(object)[i.pm, , drop=FALSE]

    i.mm <- indexProbes(object, "mm", genenames[i])[[1]]
    if (is.na(i.mm)[1])
      intensity.mm <- matrix()
    else
      intensity.mm <- intensity(object)[i.mm, , drop=FALSE]

    p.pps[[i]] <- new("ProbeSet", id = genenames[i], pm = intensity.pm, mm = intensity.mm)
  }

  options("BioC"=oldoptions)
  return(p.pps)
})

if (debug.affy123) cat("--->[[\n")


##[[: no more [[, because no more cel class
# setMethod("[[", "AffyBatch",
#           function(x, i, j, ...) { ##no need for j
#             return(new("Cel",
#                        intensity = matrix(intensity(x)[, i], ncol(x), nrow(x)),
#                        name = sampleNames(x)[i],
#                        cdfName = x@cdfName,
#                        history = description(x)@preprocessing))
#           })

##[[ we need replacement that takes an entry by the Cel in value

##[ subseting. can only happen by sample. for now not by gene
setMethod("[", "AffyBatch", function(x, i, j,..., drop=FALSE) {
  if (!missing(i) & missing(j)) {
    warning("The use of abatch[i,] and abatch[i] is deprecated. Please use abatch[,i] instead.\n")
    x <- x[,i]
  }

  if (!missing(j)) {
    phenoData(x) <- phenoData(x)[j, , ..., drop=FALSE]
    intensity(x) <- intensity(x)[ ,j, ..., drop=FALSE]
    if (!identical(se.exprs(x), new("matrix"))) { 
      se.exprs(x) <- se.exprs(x)[ ,j, ..., drop=FALSE]
    }
    protocolData(x) <- protocolData(x)[j, , ..., drop=FALSE]
  }

  return(x)
})

setReplaceMethod("[", "AffyBatch", function(x, i, j,..., value) {
  phenoData(x)[i,, ...] <- phenoData(value)[i, , ..., drop=FALSE]
  intensity(x)[,i]      <- intensity(value)[ ,i,... , drop=FALSE]
  protocolData(x)[i,, ...] <- protocolData(value)[i, , ..., drop=FALSE]
  return(x)
})


## --- bg.correct

if (debug.affy123) cat("--->bg.correct\n")

if( is.null(getGeneric("bg.correct") ))
  setGeneric("bg.correct", function(object, method, ...)
             standardGeneric("bg.correct"))

setMethod("bg.correct", signature(object="AffyBatch", method="character"),
          function(object, method=getOption("BioC")$affy$bgcorrect.method, ...) {

            ## simple for system to let one add background correction methods
            ## relies on naming convention

            method <- match.arg(method, bgcorrect.methods())

            methodname <- paste("bg.correct.", method, sep="")

            if (! exists(methodname))
              stop(paste("Unknown method (cannot find function", methodname, ")"))

            r <- do.call(methodname, alist(object, ...))

            return(r)
          })


## --- normalize.methods
if( is.null(getGeneric("normalize.methods")))
  setGeneric("normalize.methods", function(object)
             standardGeneric("normalize.methods"))


setMethod("normalize.methods", signature(object="AffyBatch"),
          function(object) {
            normalize.AffyBatch.methods()
          })

  ## ---normalize
if (is.null(getGeneric("normalize")))
  setGeneric("normalize", function(object, ...) standardGeneric("normalize"))

  setMethod("normalize", signature(object="AffyBatch"),
            function(object, method=getOption("BioC")$affy$normalize.method, ...) {
              method <- match.arg(method, normalize.AffyBatch.methods())
              if (is.na(method))
                stop("unknown method")
              method <- paste("normalize.AffyBatch", method, sep=".")
              object <- do.call(method, alist(object, ...))
              ## collect info in the attribute "normalization"
              preproc <- c(description(object)@preprocessing,
                           list(normalization = attr(object, "normalization")))
              attr(object, "normalization") <- NULL
              ## and store it in MIAME
              MIAME <- description(object)
              MIAME@preprocessing <- preproc
              description(object) <- MIAME
              ##
              return(object)
            })


## --- expression value computation
if (debug.affy123) cat("--->computeExprSet\n")
if( is.null(getGeneric("computeExprSet")))
  setGeneric("computeExprSet",
             function(x, pmcorrect.method, summary.method, ...)
             standardGeneric("computeExprSet"))

setMethod("computeExprSet", signature(x="AffyBatch", pmcorrect.method="character", summary.method="character"),
          function(x, pmcorrect.method, summary.method, ids=NULL,
                   verbose=TRUE, summary.param=list(),
                   pmcorrect.param=list())
          {

            pmcorrect.method<- match.arg(pmcorrect.method, pmcorrect.methods())
            summary.method <- match.arg(summary.method, express.summary.stat.methods())

            ids <- unname(ids)
            
            n <- length(x)

            ## if 'ids' is NULL compute for all ids
            if (is.null(ids))
              ids <- featureNames(x)

            m <- length(ids)
            pps.warnings <- vector("list", length=m)

            ## cheap trick to (try to) save time
            c.pps <- new("ProbeSet",
                         pm=matrix(),
                         mm=matrix())

            ## matrix to hold expression values
            exp.mat <- matrix(NA, m, n)
            se.mat <- matrix(NA, m, n)

            if (verbose) {
              cat(m, "ids to be processed\n")
              countprogress <- 0
            }

            ## loop over the ids
            mycall <- as.call(c(getMethod("express.summary.stat",
                                          signature=c("ProbeSet","character", "character")),
                                list(c.pps, pmcorrect=pmcorrect.method, summary=summary.method,
                                     summary.param=summary.param, pmcorrect.param=pmcorrect.param))
                              )
            ##only one character cause no more bg correct
            ##bg.correct=bg.method, param.bg.correct=bg.param,

            CDFINFO <- getCdfInfo(x) ##do it once!

            if (verbose) {
              pbt <- new("ProgressBarText", length(ids), barsteps = as.integer(20))
              open(pbt)
            }

            for (i in seq(along=ids)) {

              if (verbose) {
                updateMe(pbt)
              }

              id <- ids[i]

              if (! exists(id, envir=CDFINFO)) {
                pps.warnings[[i]] <- paste("Unknown id", id)
              } else {
                ## locations for an id
                loc <- get(id, envir=CDFINFO)
                l.pm <- loc[, 1]
                if (ncol(loc) == 2)
                  l.mm <- loc[ ,2]
                else
                  l.mm <- integer()

                np <- length(l.pm)

                ##names are skipped

                c.pps@pm <- intensity(x)[l.pm, , drop=FALSE]
                c.pps@mm <- intensity(x)[l.mm, , drop=FALSE]

                ## generate expression values
                ## (wrapped in a sort of try/catch)
                mycall[[2]] <- c.pps
                ev <- try(eval(mycall), silent = TRUE)
              }
              if (! inherits(ev, "try-error")) {
                exp.mat[i, ] <- ev$exprs
                se.mat[i,] <- ev$se.exprs
              } else {
                pps.warnings[[i]] <- ev[1]
              }

            }

            if (verbose) {
              close(pbt)
            }

            dimnames(exp.mat) <- list(ids, sampleNames(x))
            dimnames(se.mat) <- list(ids, sampleNames(x))
            eset <- new("ExpressionSet",
                        phenoData=phenoData(x),
                        ## featureData picked up from exprs
                        experimentData=experimentData(x),
                        exprs=exp.mat,
                        se.exprs=se.mat,
                        annotation=annotation(x),
                        protocolData=protocolData(x))

            attr(eset, "pps.warnings") <- pps.warnings
            return(eset)
          })


##some methods i was asked to add

setMethod("image",signature(x="AffyBatch"),
          function(x, transfo=log, col=gray(c(0:64)/64), xlab="",
            ylab="",type=c("exprs","se.exprs"), main, ...){
            scn <- prod(par("mfrow"))
            ask <- dev.interactive()
            which.plot <- 0

            type <- match.arg(type)

            if (type == "se.exprs" && all(!dim(se.exprs(x)))){
              stop("no se.exprs in object")
            }



            x.pos <- (1:nrow(x)) - (1 + getOption("BioC")$affy$xy.offset)
            y.pos <- (1:ncol(x)) - (1 + getOption("BioC")$affy$xy.offset)

            for(i in 1:length(sampleNames(x))){
              which.plot <- which.plot+1;
              if(trunc((which.plot-1)/scn)==(which.plot-1)/scn && which.plot>1 && ask)  par(ask=TRUE)

              if (type == "exprs"){
                m <- exprs(x)[,i]
              } else {
                m <- se.exprs(x)[,i]
              }


              if (is.function(transfo)) {
                m <- transfo(m)
              }
              m <- as.matrix(rev(as.data.frame(matrix(m, nrow=length(x.pos), ncol=length(y.pos)))))
              if( missing(main) ){
                main.cur=sampleNames(x)[i]
              } else {
                main.cur <- main
              }
              image(x.pos, y.pos, m,
                    col=col, main=main.cur,
                    xlab=xlab, ylab=ylab,,xaxt='n',
                      yaxt='n', ...)
              par(ask=FALSE)
            }
          })


##some special handling of main is needed
setMethod("boxplot",signature(x="AffyBatch"),
          function(x, which="both", range=0, main, ...){
            tmp <- description(x)
            if( missing(main) && (is(tmp, "MIAME")) )
                main <- tmp@title

            tmp <- unlist(indexProbes(x,which))
            tmp <- tmp[seq(1,length(tmp),len=5000)]

            boxplot(data.frame(log2(intensity(x)[tmp,])), main=main,
                      range=range, ...)
          })

###hist
if (debug.affy123) cat("--->hist\n")

if( is.null(getGeneric("hist")) )
  setGeneric("hist")

setMethod("hist",signature(x="AffyBatch"),
    function(x,...) plotDensity.AffyBatch(x,...))


if( is.null(getGeneric("mas5calls")) )
  setGeneric("mas5calls", function(object,...) standardGeneric("mas5calls"))

setMethod("mas5calls",signature(object="AffyBatch"),
          function(object,...) mas5calls.AffyBatch(object,...))


##like for ExpressionSet

"$.AffyBatch" <- function(affybatch, val)
    (pData(affybatch))[[as.character(val)]]

ppsetApply <- function(abatch, FUN, genenames=NULL, ...) {

    if (! is(abatch, "AffyBatch"))
        stop("abatch must be inheriting from class AffyBatch")

    if (! is(FUN, "function"))
        stop("FUN must be a function")

  cdfenv <- getCdfInfo(abatch)

  if (is.null(genenames))
    genenames <- ls(cdfenv)

  ##
  e1 <- new.env(parent = environment(FUN))
  multiassign(names(pData(abatch)), pData(abatch), e1)
  environment(FUN) <- e1

  ppset <- new("ProbeSet", pm=matrix(), mm=matrix())

  r <- vector("list", length=length(genenames))
  names(r) <- genenames

  for (i in seq(along=genenames)) {
    ## use mget to get NA when genenames[i] not found
    probes.i <- mget(genenames[i], envir = cdfenv, ifnotfound = NA)[[1]]
    if (all(is.na(probes.i)))
      next
    ppset@pm <- intensity(abatch)[probes.i[, 1], , drop=FALSE]
    ppset@mm <- intensity(abatch)[probes.i[, 2], , drop=FALSE]
    ppset@id <- genenames[i]
    r[[i]] <- FUN(ppset, ...)
  }

  return(r)
}

mas5 <- function(object,normalize=TRUE,sc = 500, analysis = "absolute",...){
  res <- expresso(object,bgcorrect.method="mas",pmcorrect.method="mas",normalize=FALSE,summary.method="mas",...)
  if(normalize)
    res <- affy.scalevalue.exprSet(res,sc=sc,analysis=analysis)
  return(res)
}

mas5calls.ProbeSet <- function(object,
                               tau=0.015, alpha1=0.04, alpha2=0.06,
                               ignore.saturated=TRUE){

  if(alpha1 < 0)      {stop("alpha1 must be  > 0 "); }
  if(alpha1 > alpha2) {stop("alpha2 must be  > alpha1 "); }
  if(alpha2 > 1)      {stop("alpha2 must be  <1 "); }

  ## Saturation:
  ## shouldn't be a problem with new scanners
  ##or those that have had an engineer visit
  if(ignore.saturated) { sat <- 46000; }
  else { sat <- -1; }

  pms <- pm(object)
  mms <- mm(object)
  pns <- rep(object@id,nrow(pms))
  unique.pns <- unique(pns)

  pvals<-sapply(1:length(pms[1,]),function(x) {
    .C("DetectionPValue",as.double(pms[,x]),as.double(mms[,x]),as.character(pns),as.integer(length(mms[,x])),
       as.double(tau),as.double(sat),dpval=double(length(unique.pns)),length(unique.pns), PACKAGE="affy")$dpval;
  });
  calls <- sapply(pvals,function(y) { if(y < alpha1) { return("P") } else { if(y < alpha2) { return("M") } else { return("A") }}});

  return(list(call=calls,pval=pvals))
}

mas5calls.AffyBatch <- function(object, ids=NULL, verbose=TRUE,
                                tau=0.015, alpha1=0.04, alpha2=0.06,
                                ignore.saturated=TRUE) {
  if(alpha1 < 0)      {stop("alpha1 must be  > 0 "); }
  if(alpha1 > alpha2) {stop("alpha2 must be  > alpha1 "); }
  if(alpha2 > 1)      {stop("alpha2 must be  <1 "); }

  if(verbose) cat("Getting probe level data...\n");
  pms <-as.matrix(pm(object));
  mms <-as.matrix(mm(object));

  # Saturation:
  # shouldn't be a problem with new scanners
  ##or those that have had an engineer visit
  if(ignore.saturated) { sat <- 46000; }
  else { sat <- -1; }

  pns <- probeNames(object);
  o <- order(pns)
  pns <- pns[o]
  pms <- pms[o,,drop=FALSE]
  mms <- mms[o,,drop=FALSE]
  unique.pns <- sort(unique(pns));

  if(verbose) cat("Computing p-values\n");
  p<-sapply(1:length(pms[1,]),function(x) {
    .C("DetectionPValue",as.double(pms[,x]),as.double(mms[,x]),as.character(pns),as.integer(length(mms[,x])),
       as.double(tau),as.double(sat),dpval=double(length(unique.pns)),length(unique.pns), PACKAGE="affy")$dpval;
  });
  rownames(p) <- unique.pns;
  colnames(p) <- sampleNames(object)

  if(verbose) cat("Making P/M/A Calls\n");
  calls <- sapply(p,function(y) { if(y < alpha1) { return("P") } else { if(y < alpha2) { return("M") } else { return("A") }}});
  calls <- matrix(calls,nrow=nrow(p),ncol=ncol(p));
  colnames(calls) <- sampleNames(object)
  rownames(calls) <- rownames(p)

  if(!is.null(ids)){
    calls <- calls[ids,,drop=FALSE]
    p <- p[ids,,drop=FALSE]
  }

  eset <- new("ExpressionSet",
              phenoData=phenoData(object),
              ## featureData picked up from object
              experimentData=experimentData(object),
              annotation=annotation(object),
              protocolData=protocolData(object),
              exprs=calls,
              se.exprs=p
              )
  return(eset)
}


mas5.detection <- function(mat, tau=0.015, alpha1=0.04, alpha2=0.06,
                           exact.pvals=FALSE, cont.correct=FALSE) {
  ## CONSTANTS

  saturation.point <- 46000			# not a user parameter
  mat.r <- (mat[,1]-mat[,2])/(mat[,1]+mat[,2])
  ## SANITY CHECKING
  if ( !is.matrix(mat) || length(dim(mat))!=2 || dim(mat)[2]!=2 ||
      dim(mat)[1] < 1 || !is.numeric(mat) )
    stop("Invalid mat matrix.")
  if ( !is.numeric(tau) )
    stop("Invalid tau.")
  if ( !is.numeric(alpha1) || !is.numeric(alpha2) ||
      alpha1 <= 0 || alpha1 >= alpha2 || alpha2 >= 0.5 )
    stop("Invalid alpha1 or alpha2.")
  if ( !is.logical(exact.pvals) )
    stop("Invalid exact.pvals.")
  if ( !is.logical(cont.correct) )
        stop("Invalid cont.correct.")

  ## DEALING WITH SATURATION; COMPUTING THE P-VALUE
  ## According to the Bioinformatics paper:
  ## * If all MM's are saturated, then call present
  ## * Otherwise discard pairs with a saturated MM
  ## According to the Affymetrix whitepaper:
  ## * If all probe-pairs are saturated, then call present with pval=0
  ## * If an MM is saturated, then we discard the pair
  ## * If a PM and MM are within tau of each other, we discard the pair
  ## So we're going with:
  ## * If all MM's are saturated, set pval=0 and don't use Wilcoxon
  ## * Discard probe-pairs when MM is saturated or the PM,MM are within tau
  ##   of each other
  ## * Compute the p-value using Wilcoxon's signed rank test on the retained
  ##   probe-pairs
  is.mm.saturated <- function(probe.pair, saturation.point)
        probe.pair[2] >= saturation.point
  is.retained <- function(probe.pair, saturation.point, tau)
    !(is.mm.saturated(probe.pair,saturation.point) ||
      abs(diff(probe.pair)) <= tau)
  if ( all(apply(mat,1,is.mm.saturated,saturation.point)) )
    pval <- 0
  else {
    retained <- apply(mat, 1, is.retained, saturation.point, tau)
    pval <- wilcox.test(mat.r[retained],
                        alternative="greater", mu=tau, paired=FALSE,
                        exact=exact.pvals, correct=cont.correct,
                        conf.int=FALSE)$p.value
  }

  ## DETECTION CALL
  if ( pval < 0 || pval > 1 )
    warning("Computed an unusual p-value outside the range [0,1].")
  if ( pval < alpha1 )
    call <- "P"
  else if ( pval < alpha2 )
        call <- "M"
  else
        call <- "A"

  ## DONE
  return(list(pval=pval, call=call))
}

   
hlog <- function(x,constant=1){ #constant is where the change occurs
  if(constant<=0){
    warning("constant less than or equal to 0. Returning log(x)\n")
    return(log(x))
  }
  else{
    if(constant==Inf)
      return(x)
    else{
      aux <- (abs(x)<constant)*(x) +
        (abs(x)>=constant)*(sign(x)*(constant*log(abs(x/constant))+constant))
      aux[x==0] <- 0
      return(aux)
    }
  }
}

generateExprVal.method.liwong <- function(probes, ...) {
  probes <- t(probes)
  if (ncol(probes) == 1) {
    warning("method liwong unsuitable when only one probe pair")
    list(exprs=as.vector(probes),se.exprs=rep(NA,length(probes)))
  }
  else {
    tmp <- fit.li.wong(probes, ...)
    list(exprs=tmp$theta,se.exprs=tmp$sigma.theta)
  }
}

normalize.AffyBatch.invariantset <- function(abatch, prd.td=c(0.003,0.007), verbose=FALSE,baseline.type=c("mean","median","pseudo-mean","pseudo-median"),type=c("separate","pmonly","mmonly","together")) {

  do.normalize.Affybatch.invariantset <- function(abatch, pms, prd.td, baseline.type){


    nc  <-  length(abatch)                                 # number of CEL files

    if (baseline.type == "mean"){
                                        # take as a reference the array having the median overall intensity
      m <- vector("numeric", length=nc)
      for (i in 1:nc)
        m[i] <- mean(intensity(abatch)[pms, i])
      refindex <- match(trunc(median(rank(m))), rank(m))
      rm(m)
      baseline.chip <-  c(intensity(abatch)[pms, refindex])
      if (verbose) cat("Data from", sampleNames(abatch)[refindex], "used as baseline.\n")
    }
    else if (baseline.type == "median"){
                                        # take as a reference the array having the median median intensity
      m <- vector("numeric", length=nc)
      for (i in 1:nc)
        m[i] <- median(intensity(abatch)[pms, i])
      refindex <- match(trunc(median(rank(m))), rank(m))
      rm(m)
      baseline.chip <-  c(intensity(abatch)[pms, refindex])
      if (verbose) cat("Data from", sampleNames(abatch)[refindex], "used as baseline.\n")
    } else if (baseline.type == "pseudo-mean"){
                                        # construct a psuedo chip to serve as the baseline by taking probewise means
      refindex <- 0
      baseline.chip <- rowMeans(intensity(abatch)[pms,])
    } else if (baseline.type == "pseudo-median"){
    # construct a pseudo chip to serve as the baseline by taking probewise medians
      refindex <- 0
      baseline.chip <- rowMedians(intensity(abatch)[pms,])
    }


  ##set.na.spotsd(cel.container)

    normhisto <- vector("list", length=nc)
#  normhisto[[refindex]] <- list(name="reference for the invariant set")

  ## loop over the CEL files and normalize them

    for (i in (1:nc)) {
      if (i != refindex){
        if (verbose) cat("normalizing array", sampleNames(abatch)[i], "...")

        ##temporary
        tmp <- normalize.invariantset(c(intensity(abatch)[pms, i]),
                                      c(baseline.chip),
                                      prd.td)
                                        #i.set <- which(i.pm)[tmp$i.set]
        tmp <- as.numeric(approx(tmp$n.curve$y, tmp$n.curve$x,
                               xout=intensity(abatch)[pms, i], rule=2)$y)
        attr(tmp,"invariant.set") <- NULL
        intensity(abatch)[pms, i] <- tmp

        ## storing information about what has been done
                                        #normhisto[[i]] <- list(name="normalized by invariant set",
                                        #                       invariantset=i.set)

        if (verbose) cat("done.\n")

      }
    }
    attr(abatch, "normalization") <- normhisto
    return(abatch)
  }

  type <- match.arg(type)
  baseline.type <- match.arg(baseline.type)

  if (type == "pmonly"){
    pms <- unlist(pmindex(abatch))
    do.normalize.Affybatch.invariantset(abatch, pms, prd.td, baseline.type)
  } else if (type == "mmonly"){
    pms <- unlist(mmindex(abatch))
    do.normalize.Affybatch.invariantset(abatch, pms, prd.td, baseline.type)
  } else if (type == "together"){
    pms <- unlist(indexProbes(abatch,"both"))
    do.normalize.Affybatch.invariantset(abatch, pms, prd.td, baseline.type)
  } else if (type == "separate"){
    pms <- unlist(pmindex(abatch))
    abatch <- do.normalize.Affybatch.invariantset(abatch, pms, prd.td, baseline.type)
    pms <- unlist(mmindex(abatch))
    do.normalize.Affybatch.invariantset(abatch, pms, prd.td, baseline.type)
  }

}



##  The 'common-to-all' part of the algorithm. Operates on two vectors of numeric data
##
normalize.invariantset <- function(data, ref, prd.td=c(0.003,0.007)) {

  np <- length(data)
  r.ref <- rank(ref)
  r.array <- rank(data)

  ## init
  prd.td.adj <- prd.td*10                           # adjusted threshold things
  i.set <- rep(TRUE, np)                            # index all the PM probes as being in the invariant set
  ns <- sum(i.set)                                  # number of probes in the invariant set
  ns.old <- ns+50+1                                 # number of probes previously in the invariant set

  ## iterate while the number of genes in the invariant set (ns) still varies...
  while ( (ns.old-ns) > 50 ) {
    air <- (r.ref[i.set] + r.array[i.set]) / (2*ns)  # average intensity rank for the probe intensities
    prd <- abs(r.ref[i.set] - r.array[i.set]) / ns
    threshold <- (prd.td.adj[2]-prd.td[1]) * air + prd.td.adj[1]
    i.set[i.set] <- (prd < threshold)

    ns.old <- ns
    ns <- sum(i.set)

    if (prd.td.adj[1] > prd.td[1])
      prd.td.adj <- prd.td.adj * 0.9  # update the adjusted threshold parameters
  }

  ## the index i.set corresponds to the 'invariant genes'
  n.curve <- smooth.spline(ref[i.set], data[i.set])
  ## n.curve$x contains smoothed reference intensities
  ## n.curve$y contains smoothed i-th array intensities

  ##data <- as.numeric(approx(n.curve$y, n.curve$x, xout=data)$y)
  ##attr(data,"invariant.set") <- i.set
  ##return(data)
  return(list(n.curve=n.curve, i.set=i.set))
}

plot.ProbeSet <- function(x, which=c("pm", "mm"), xlab="probes", type="l", ylim=NULL, ...) {

  which <- match.arg(which)
  if (which == "pm")
    f <- getMethod("pm", "ProbeSet")
  else
    f <- getMethod("mm", "ProbeSet")
  
  if (is.null(ylim))
    ylim = range(c(f(x)), na.rm=TRUE)
  
  if (is.na(xlab))
    xlab="probes"
  
  matplot(f(x), xlab=xlab, type=type, ylim=ylim, ...)  
}

##
## Basic set of tests for the class AffyBatch
##
library(affy)

## fake environment
##it the below change the environment def must change too
NCOL <- 8
NROW <- 8
n <- NCOL*NROW

cat("---> normalizing an environment...\n")
tmp <- sample(1:50)
dummy <- new.env(hash=T)
index <- cbind(tmp[1:10], tmp[11:20])
assign("gene.a", index, envir=dummy)
index <- cbind(tmp[21:30],tmp[31:40])
assign("gene.b", index, envir=dummy)
cat("done.\n")

cat("---> creating an AffyBatch...\n")
samplenames <- c("sample1","sample2")
signal <- exp(rexp(n,1))
e <- cbind(exp(rnorm(n,4,1))+signal,exp(rnorm(n,4,1))+signal)
colnames(e) <- samplenames
afbatch <- new("AffyBatch",
               exprs=e,
               cdfName="dummy",
               ncol=NCOL,nrow=NROW)
cat("done.\n")


##can i get pms?
pms <- pm(afbatch)
mms <- mm(afbatch)

## normalize the AffyBatch
cat("---> normalizing an AffyBatch...\n")
n.afbatch <- normalize(afbatch, method="constant")
cat("done.\n")

## compute expression values
cat("---> computing expression values...\n")
e.set <- computeExprSet(n.afbatch, pmcorrect.method="pmonly", summary.method="avgdiff")
if (!is(e.set, "ExpressionSet"))
  stop("e.set does not inherit from 'ExpressionSet'!")
cat("done.\n")

####These functions take an AffyBatch object "background correct"
####the pms and return an AffyBatch with the background corrected PMs
###
bg.parameters <- function(pm, n.pts=2^14){
  
  max.density <- function(x, n.pts){
    aux <- density(x, kernel="epanechnikov", n=n.pts, na.rm=TRUE)
    aux$x[order(-aux$y)[1]] 
  }
  
  pmbg <- max.density(pm,n.pts) ##Log helps detect mode
  bg.data <- pm[pm < pmbg]
  ##do it again to really get the mode
  pmbg <- max.density(bg.data,n.pts) 
  bg.data <- pm[pm < pmbg]
  bg.data <- bg.data - pmbg
  
  bgsd <- sqrt(sum(bg.data^2)/(length(bg.data)-1))*sqrt(2)#/.85
  
  sig.data <- pm[pm > pmbg]
  sig.data <- sig.data-pmbg
  
  expmean <- max.density(sig.data,n.pts)
  alpha <- 1/expmean
  mubg <- pmbg
  list(alpha=alpha,mu=mubg,sigma=bgsd)  
}

bg.adjust <- function(pm, n.pts=2^14, ...){
  param <- bg.parameters(pm,n.pts)
  b <- param$sigma
  pm <- pm - param$mu - param$alpha*b^2
  pm + b*((1./sqrt(2*pi))*exp((-1./2.)*((pm/b)^2)))/pnorm(pm/b)
}

bg.correct.none <- function(object, ...)
  object

##bg.correct.subtractmm <- function(object){
##  pm(object) <- pm(object) - mm(object)
##  return(object)
##}

###bg.correct.rma <- function(object, ...){
###  pm(object) <- apply(pm(object),2,bg.adjust)
###  return(object)
##}

##
## this function calls the c code as an alternative to the R code above.
## it should help end the disagreement between rma() and expresso()
##

bg.correct.rma <- function(object,...){
  
	pm(object) <- rma.background.correct(pm(object),copy=FALSE)
        return(object)
}

## --- pmcorrect things

pmcorrect.subtractmm <- function(object){
  pm.corrected <- pm(object) - mm(object)
  return(pm.corrected)
}

pmcorrect.pmonly <- function(object) {
  return(pm(object))
}

##################################################################
##
## file: normalize.quantiles.R
##
## For a description of quantile normalization method see
##
##  Bolstad, B. M., Irizarry R. A., Astrand, M, and Speed, T. P. (2003)(2003)
##  A Comparison of Normalization Methods for High
##  Density Oligonucleotide Array Data Based on Bias and Variance.
##  Bioinformatics 19,2,pp 185-193
##
## History
## Pre Aug 23, 2003 Two years worth of stuff
## Aug 23, 2003 - Added use.log2 to "robust",
##                added ability to pass additional parameters
##                to normalize.AffyBatch.Quantiles.robust
##                changed pmonly parameters on functions
##                so that it is now a string argument "type"
##                the options are pmonly, mmonly, together, separate
## Jan 31, 2004 - put a check for an integer matrix and force coercision to
##                doubles if required in normalize.quantiles
## Mar 13, 2005 - Modifications to normalize.quantiles.robust including removing
##                approx.method which never got implemented. Making it a use a .Call()
##                rather than a .C()
##
## Sep 20, 2006 - fix .Call in normalize.quantiles.robust
## May 20, 2007 - remove the functions that have been moved to preprocessCore
##
##################################################################

normalize.AffyBatch.quantiles <- function(abatch,type=c("separate","pmonly","mmonly","together")) {


  type <- match.arg(type)

  if ((type == "pmonly")|(type == "separate")){
    pms <- unlist(pmindex(abatch))
    ## Change to faster computation of noNA - SDR 11/06/2003
    ##noNA <- apply(intensity(abatch)[pms,,drop=FALSE],1,function(x) all(!is.na(x)))
    noNA <- rowSums(is.na(intensity(abatch)[pms,,drop=FALSE])) == 0
    pms <- pms[noNA]
    intensity(abatch)[pms,] <- normalize.quantiles(intensity(abatch)[pms,,drop=FALSE ],copy=FALSE)
  }
  if((type == "mmonly") | (type == "separate")){
    mms <- unlist(mmindex(abatch))
    ## Change to faster computation of noNA - SDR 11/06/2003
    ##noNA <- apply(intensity(abatch)[mms,,drop=FALSE],1,function(x) all(!is.na(x)))
    noNA <- rowSums(is.na(intensity(abatch)[mms,,drop=FALSE])) == 0
    mms <- mms[noNA]

    intensity(abatch)[mms,] <- normalize.quantiles(intensity(abatch)[mms,,drop=FALSE ],copy=FALSE)
  }
  if (type == "together"){
    pms <- unlist(indexProbes(abatch,"both"))
    intensity(abatch)[pms,]  <- normalize.quantiles(intensity(abatch)[pms,,drop=FALSE ],copy=FALSE)
  }

  ##this is MIAME we need to decide how to do this properly.
  ##for (i in 1:length(abatch)) {
  ##  history(abatch)[[i]]$name <- "normalized by quantiles"
  ##}

  return(abatch)
}

normalize.AffyBatch.quantiles.robust <- function(abatch, type=c("separate","pmonly","mmonly","together"),weights=NULL,remove.extreme=c("variance","mean","both","none"),n.remove=1,use.median=FALSE,use.log2=FALSE) {

  type <- match.arg(type)

  if ((type == "pmonly")|(type == "separate")){
    pms <- unlist(pmindex(abatch))
    intensity(abatch)[pms, ] <- normalize.quantiles.robust(intensity(abatch)[pms, ], copy=FALSE,weights=weights,remove.extreme,n.remove=n.remove,use.median=use.median,use.log2=use.log2)
  }
  if ((type == "mmonly")|(type == "separate")){
    mms <- unlist(mmindex(abatch))
    intensity(abatch)[mms, ] <- normalize.quantiles.robust(intensity(abatch)[mms, ],copy=FALSE,weights=weights,remove.extreme,n.remove=n.remove,use.median=use.median,use.log2=use.log2)
  }

  if (type == "together"){
    pms <- unlist(indexProbes(abatch,"both"))
    intensity(abatch)  <- normalize.quantiles.robust(intensity(abatch)[pms,,drop=FALSE ],copy=FALSE, weights=weights,remove.extreme=remove.extreme,n.remove=n.remove,use.median=use.median,use.log2=use.log2)
  }



  ##this is MIAME we need to decide how to do this properly.
  ##for (i in 1:length(abatch)) {
  ##  history(abatch)[[i]]$name <- "normalized by quantiles"
  ##}

  return(abatch)
}

pairs.AffyBatch <- function(x, panel=points, ..., transfo=I, main=NULL, oma=NULL,
                            font.main = par("font.main"), cex.main = par("cex.main"),
                            cex.labels = NULL, 
                            lower.panel=panel, upper.panel=NULL,
                            diag.panel=NULL,
                                        #text.panel = textPanel,
                                        #label.pos = 0.5 + has.diag/3,                                
                            font.labels = 1, row1attop = TRUE, gap = 1) {

  #label1 <- chipNames(x)
  #label2 <- unlist(lapply(history(x), function(z) z$name))
  
  #textPanel <- function(x = 0.5, y = 0.5, txt, cex, font) {
  #  text(x, y, txt, cex = cex, font = font)
  #}
  
  ##labels <- paste(sampleNames(x), unlist(lapply(history(x), function(z) if (is.null(z$name)) "" else z$name)), sep="\n")
  labels <- sampleNames(x)
  ##y <- matrix(intensity(x)[, , seq(along=x)], ncol=length(x))
  y <- intensity(x)
  
  pairs(transfo(y), labels=labels,
        panel=panel, ..., main=main, oma=oma,
        font.main = font.main, cex.main = cex.main,
        lower.panel=lower.panel, upper.panel=upper.panel, diag.panel=diag.panel,
        #text.panel = text.panel,
        #label.pos = label.pos,
        cex.labels = cex.labels, 
        font.labels = font.labels, row1attop = row1attop, gap = gap
        )
   
}

############################
##MPM Changed delta, Affy SADD states delta as 2e-20
pmcorrect.mas <- function (object, contrast.tau = 0.03, scale.tau = 10, delta = 2^(-20))
#function (object, contrast.tau = 0.03, scale.tau = 10, delta = 9.536743e-07) 
###########################
{
    all.pps.pm <- pm(object)
    all.pps.mm <- mm(object)
    diff <- log2(all.pps.pm) - log2(all.pps.mm)
    delta <- rep(delta, nrow(diff))
    for (i in 1:ncol(diff)) {
        sb <- tukey.biweight(diff[, i])
        pps.pm <- all.pps.pm[, i]
        pps.mm <- all.pps.mm[, i]
        pps.im <- pps.mm
        j <- (pps.mm >= pps.pm) & (sb > contrast.tau)
        pps.im[j] <- pps.pm[j]/2^sb
        j <- (pps.mm >= pps.pm) & (sb <= contrast.tau)
        pps.im[j] <- pps.pm[j]/2^(contrast.tau/(1 + (contrast.tau - 
            sb)/scale.tau))
	#########################
	#MPM SADD Need to substract the PM-IM, I think this is the culprit
	pm.corrected <- apply(cbind(pps.pm-pps.im, delta), 1, 
            max)
        #pm.corrected <- apply(cbind(pps.pm, pps.im, delta), 1, 
        #    max)
	##########################
        diff[, i] <- pm.corrected
    }
    return(diff)
}

library(affy)
library(affydata)

data(Dilution)

meth <- bgcorrect.methods()

cat("background correction:\n")

for (m in meth) {
  cat(m,"...")
  abatch.bgc <- bg.correct(Dilution, method=m)
  cat("done.\n")
}

generateExprVal.method.mas <- function(probes, ...)
{
  
  probes <- log2(probes)
  M <-  ncol(probes)
  slg <- rep(NA,M)
  
  for (i in 1:ncol(probes)) {
    
    slg[i] <- tukey.biweight(probes[ ,i], ...)
    
  }
  
  return(list(exprs=2^slg,se.exprs=rep(NA,M)))
    
}

affy.scalevalue.exprSet <- function(eset, sc=500, analysis="absolute")
{
  
  analysis <- match(analysis, c("absolute", "comparison"))
  
  if(analysis == 1)
    nf <- 1
  else
    stop("sorry! comparison not implemented.")
  for (i in 1:ncol(exprs(eset))) {
    slg <- exprs(eset)[, i]
    sf <- sc / mean(slg, trim=0.02)  
    reported.value <- nf * sf * slg
    exprs(eset)[, i] <- reported.value
  }
  
  return(eset)
}

bg.correct.mas <- function(object, griddim=16)
{
   nchips <- length(object)

   pm.index <- unique(unlist(indexProbes(object, "pm")))
   mm.index <- unique(unlist(indexProbes(object, "mm")))

   ## some chips have some probesets without MM probes
   ## which will return an NA in mm.index

   mm.index <- mm.index[!is.na(mm.index)]

   rows <- nrow(object)
   cols <- ncol(object)

   allintensities <- intensity(object)[c(pm.index, mm.index), ]

   # note that the indexing is +1 more than you'd expect because
   # the c code expects it that way
   ## (note about the remark above: R indexing starts at 1 and not at 0,
   ## that's why the indexing is done this way. The package is primarily done to
   ## be used with R...)

   allx <- c(pm.index-1, mm.index-1) %% nrow(object) +1
   ally <- c(pm.index-1, mm.index-1) %/% nrow(object) + 1

   nprobes <- length(allx)

   corrected <- matrix(.C("affy_background_adjust_R",
                          as.double(as.vector(allintensities)), as.integer(allx), as.integer(ally),
                          as.integer(nprobes), as.integer(nchips), as.integer(rows), as.integer(cols),
                          as.integer(griddim), PACKAGE="affy")[[1]],
                       nprobes, nchips)

   intensity(object)[c(pm.index, mm.index), ] <- corrected
   ## and what with the 'non pm or mm' probes ?
   ## answer: they are not used per Affymetrix Statistical Algorithms Description Document.

   return(object)

 }

## Currently, the input is a 2 matrices a pm and a mm

##avdiff is more like median than mean, it would be nice to actually have
##avfif
##added typical se of the mean as returned se
generateExprVal.method.avgdiff <- function(probes, ...) {
  list(exprs=apply(probes, 2, median),se.exprs=apply(probes,2,sd)/sqrt(nrow(probes)))
}

 ## A ProbeSet holds probe values for a probe pairs set(*) accross a batch of experiments.
  ## methods 'express.summary.stat' returns of expression value per experiement in the
  ## batch, and 'bg.correct' does background correction (in some sense... the MM probes
  ## were created to measure unspecific hybridization. People thought that doing
  ## PM - MM would remove background noise. The method 'bg.correct' accepts extra parameters
  ## through '...' (can be used to pass background correction parameters common to different
  ## ProbeSet)
  ##
  ## -
  ## (*) : a probe pair set is the set of probes pairs(**) related to an affyid. Generally a
  ##       a probe pair set has 20 elements.
  ## (**): a probe pair (or atom) is a pair of PM/MM values
  ##

if (debug.affy123) cat("-->initProbeSet\n")

setClass("ProbeSet",
         representation(id="character", pm="matrix", mm="matrix"),
         prototype=list(pm=matrix(), mm=matrix()))

setMethod("show", "ProbeSet",
          function(object) {
            cat("ProbeSet object:\n")
            cat("  id=", object@id, "\n", sep="")
            cat("  pm=", nrow(object@pm), "probes x ", ncol(object@pm), " chips\n")
          })

##DEBUG: what to do with that ?
## --> with what ?

setMethod("sampleNames", "ProbeSet",
          function(object) colnames(object))

setMethod("colnames", signature(x="ProbeSet"),
          function(x ,do.NULL=FALSE, prefix="row") {
            
            cnames<-colnames(pm(x))
            
            if (is.null(cnames)) {
              
              if (do.NULL) {
                warning("No column names for ProbeSet")
              }
              else {
                cnames <- paste(prefix, 1:ncol(x@pm))
              }
              
            }
            return(cnames)
          })

## pm
if( is.null(getGeneric("pm")))
  setGeneric("pm", function(object) standardGeneric("pm"))

setMethod("pm", "ProbeSet", function(object) object@pm)

if( is.null(getGeneric("pm<-")))
  setGeneric("pm<-", function(object, value) standardGeneric("pm<-"))

setReplaceMethod("pm", signature=c("ProbeSet", "matrix"),
                 function(object, value) {
                   if (! all(dim(value) == dim(object@mm)))
                     stop("dimension mismatch between 'pm' and 'mm'")
                   object@pm <- value
                 })

## mm
if( is.null(getGeneric("mm")))
  setGeneric("mm", function(object) standardGeneric("mm"))

setMethod("mm", "ProbeSet", function(object) object@mm)


if( is.null(getGeneric("mm<-")))
  setGeneric("mm<-", function(object, value) standardGeneric("mm<-"))

setReplaceMethod("mm", signature=c("ProbeSet", "matrix"),
                 function(object, value) {
                   if (sum(dim(value) == dim(object@mm)) != 2)
                     stop("dimension mismatch between 'pm' and 'mm'")
                   object@mm <- value
                 })

## method express.summary.stat
if( is.null(getGeneric("express.summary.stat")))
  setGeneric("express.summary.stat", function(x, pmcorrect, summary, ...)
             standardGeneric("express.summary.stat"))

setMethod("express.summary.stat",signature(x="ProbeSet",  pmcorrect="character", summary="character"),
          function(x, pmcorrect, summary, summary.param=list(), pmcorrect.param=list()) {
            
            pmcorrect <- match.arg(pmcorrect, pmcorrect.methods())
            summary  <- match.arg(summary, express.summary.stat.methods())
            
            ## simple for system to let one add background correction methods
            ## relies on naming convention
            pmcorrect.methodname <- paste("pmcorrect.", pmcorrect, sep="")
            summary.methodname <- paste("generateExprVal.method.", summary, sep="")
            
            if (! exists(summary.methodname))
              stop(paste("Unknown method (cannot find function", summary.methodname, ")"))
            if (! exists(pmcorrect.methodname))
              stop(paste("Unknown method (cannot find function", pmcorrect.methodname, ")"))
            
            ## NOTE: this could change...
                                        #m <- do.call(bg.correct, c(alist(x@pm, x@mm), param.bg.correct))
            pm.corrected <- do.call(pmcorrect.methodname, c(alist(x), pmcorrect.param))
            r <- do.call(summary.methodname, c(alist(pm.corrected), summary.param))
            
            ##DEBUG: name stuff to sort
                                        #names(r) <- names(allprobes)
            
            return(r)
          })


setMethod("barplot",signature(height="ProbeSet"),function(height,...) barplot.ProbeSet(height,...))

if( is.null(getGeneric("mas5calls")) )
  setGeneric("mas5calls", function(object,...) standardGeneric("mas5calls"))

setMethod("mas5calls",signature(object="ProbeSet"),
          function(object,...) mas5calls.ProbeSet(object,...))

% -*- mode: noweb; noweb-default-code-mode: R-mode; -*-
%\VignetteIndexEntry{4. Import Methods}
%\VignetteKeywords{Preprocessing, Affymetrix}
%\VignetteDepends{affy}
%\VignettePackage{affy}
%documentclass[12pt, a4paper]{article}
\documentclass[12pt]{article}

\usepackage{amsmath}
\usepackage{hyperref}
\usepackage[authoryear,round]{natbib}

\textwidth=6.2in
\textheight=8.5in
%\parskip=.3cm
\oddsidemargin=.1in
\evensidemargin=.1in
\headheight=-.3in

\newcommand{\scscst}{\scriptscriptstyle}
\newcommand{\scst}{\scriptstyle}

\author{Laurent}
\begin{document}
\title{affy: Import Methods (HowTo)}

\maketitle
\tableofcontents
\section{Introduction}
This document describes briefly how to write import methods for
the \verb+affy+ package. As one might know, the Affymetrix data are
originally stored in files of type \verb+.CEL+ and \verb+.CDF+. The
package extracts and store the information contained in \verb+R+ data
structures using file parsers. This document outlines how to get the
data from other sources than the current\footnote{today's date is
early 2003} file formats.

As usual, loading the package in your \verb+R+ session is required. 
\begin{Sinput}
R> library(affy) ##load the affy package
\end{Sinput}
<<echo=F,results=hide>>=
library(affy)
@

{\bf note: this document only describes the process for .CEL files}

Knowing the slots of \verb+Cel+ and \verb+AffyBatch+ objects will
probably help you to achieve your goals. You may want to refer to
the respective help pages. Try \verb+help(Cel)+, \verb+help(AffyBatch)+.

\section{How-to}
 \subsection{Cel objects}
The functions \verb+getNrowForCEL+ and \verb+getNcolForCEL+ are assumed to return
the number of rows and the number of columns in the \verb+.CEL+ file respectively

You will also need to have access to the $X$ and $Y$ position for the probes in the
{\verb .CEL} file. The functions \verb+getPosXForCel+ and \verb+getPosYForCEL+
are assumed to return the $X$ and $Y$ positions respectively. The corresponding
probe intensities are assumed to be returned by the function \verb+getIntensitiesForCEL+.

If you stored {\bf all} the $X$ and $Y$ values that were in the \verb+.CEL+, the
functions verb+getNrowForCEL+ and \verb+getNcolForCEL+ can written:
<<<>>=
getNrowForCEL <- function() max(getPosXForCEL())
getNcolForCEL <- function() max(getPosYForCEL())
@ 

You will also need the name for the corresponding \verb+.CDF+ (although you will
probably no need the \verb+.CDF+ file itself, the cdf packages available for download
will probably be enough).

\begin{Sinput}
import.celfile <- function(celfile, ...) {

  cel.nrow <- getNrowForCEL(celfile)
  cel.ncol <- getNcolForCEL(celfile)
  x <- matrix(NA, nr=cel.nrow, nc=cel.ncol)

  cel.intensities <- getIntensitiesForCEL(celfile)

  cel.posx <- getPosXForCEL(celfile) # +1 if indexing starts at 0 (like in .CEL)
  cel.posy <- getPosYForCEL(celfile) # idem

  x[cbind(cel.posx, cel.posy)] <- cel.intensities

  mycdfName <- whatcdf("aCELfile.CEL")

  myCel <- new("Cel", exprs=x, cdfName=mycdfName)

  return(myCel)
}
\end{Sinput}

The function \verb+import.celfile+ can now replace the function \verb+read.celfile+
in the \verb+affy+ package


 \subsection{AffyBatch objects}

(scratch)
the use of \verb+...+ should make you able to override the function
read.celfile by a hack like:
\begin{Sinput}
read.celfile <- import.celfile
\end{Sinput}
The function \verb+read.affybatch+ should now function using your \verb+import.celfile+


\end{document}

% -*- mode: noweb; noweb-default-code-mode: R-mode; -*-
%\VignetteIndexEntry{3. Custom Processing Methods}
%\VignetteKeywords{Preprocessing, Affymetrix}
%\VignetteDepends{affy}
%\VignettePackage{affy}
%documentclass[12pt, a4paper]{article}
\documentclass[12pt]{article}

\usepackage{amsmath}
\usepackage{hyperref}
\usepackage[authoryear,round]{natbib}

\textwidth=6.2in
\textheight=8.5in
%\parskip=.3cm
\oddsidemargin=.1in
\evensidemargin=.1in
\headheight=-.3in

\newcommand{\scscst}{\scriptscriptstyle}
\newcommand{\scst}{\scriptstyle}

\newcommand{\Rfunction}[1]{{\texttt{#1}}}
\newcommand{\Robject}[1]{{\texttt{#1}}}
\newcommand{\Rpackage}[1]{{\textit{#1}}}
\newcommand{\Rmethod}[1]{{\texttt{#1}}}
\newcommand{\Rfunarg}[1]{{\texttt{#1}}}
\newcommand{\Rclass}[1]{{\textit{#1}}}

\author{Laurent}
\begin{document}
\title{affy: Custom Processing Methods (HowTo)}

\maketitle
\tableofcontents
\section{Introduction}
This document describes briefly how to customize the affy package by
adding one's own processing methods. The types of processing methods
are background correction, normalization, perfect match correction 
and summary expression value computation. 
We tried our best to make this as easy as we could, but we are aware
that it is far from being perfect. We are still working on things to
improve them.
Hopefully this document should let you extend the
package with supplementary processing methods easily.

As usual, loading the package in your \verb+R+ session is required. 
\begin{Sinput}
R> library(affy) ##load the affy package
\end{Sinput}
<<echo=F,results=hide>>=
library(affy)
@

\section{How-to}
For each processing step, labels for the methods known to the package
are stored in variables. 

<<>>=
normalize.AffyBatch.methods()
bgcorrect.methods()
pmcorrect.methods()
express.summary.stat.methods()
@
We would recommend the use of the method \verb+normalize.methods+ to access
the list of available normalization methods (as a scheme for normalization
methods that would go beyond 'affy' is thought).
<<>>=
library(affydata)
data(Dilution)
normalize.methods(Dilution)
@

For each processing step, a naming convention exists between the method label
and the function name in \verb+R+ (see table~\ref{table:summary.labels}).
Each processing methods should be passed objects (and return objects)
corresponding to the processing step (see table~\ref{table:summary.methods}). 
\begin{table}
 \begin{tabular}{|c|c|}
\hline
variable for labels & naming convention \\
\hline
bgcorrect.methods & bg.correct.<label> \\
%\hline
normalize.AffyBatch.methods & normalize.AffyBatch.<label> \\
%\hline
pmcorrect.methods & pmcorrect.<label> \\
%\hline
express.summary.stat.methods & generateExprset.methods.<label>\\
\hline
 \end{tabular}
\caption{\label{table:summary.labels}Summary table for the processing methods.}
\end{table}

\begin{table}
 %\begin{tabular}{|c|c|p{0.5\textwidth}|}
 \begin{tabular}{|c|c|p{5cm}|}
\hline
step & argument(s) & returns \\
\hline
background correction & \verb+AffyBatch+ & \verb+AffyBatch+ \\
%\hline
normalization & \verb+AffyBatch+ & \verb+AffyBatch+ \\
%\hline
{\it pm} correction &  \verb+ProbeSet+ & a \verb+matrix+ of corrected PM values
(one probe per row, one column per chip).\\
%\hline
expression value & a \verb+matrix+ of PM values & a list of two elements
\verb+exprs+ and \verb+se.exprs+\\
\hline
 \end{tabular}
\caption{\label{table:summary.methods}Summary table for the processing methods.}
\end{table}

Practically, this means that to add a method one has to 
\begin{enumerate}
 \item create an appropriate method with a name satisfying the convention.
 \item register the method by adding the label name to the corresponding
 variable.
\end{enumerate}

\section{Examples}
As an example we show how to add two simple new methods. The first one does
{\it pm} correction. The method subtract {\it mm} values to the {\it pm} values,
except when the result is negative. In this case the {\it pm} value remains
unchanged.

We create a function using the label name \verb+subtractmmsometimes+.
<<>>=
pmcorrect.subtractmmsometimes <- function(object) {

  ## subtract mm
  mm.subtracted <- pm(object) - mm(object)

  ## find which ones are unwanted and fix them
  invalid <- which(mm.subtracted <= 0)
  mm.subtracted[invalid] <- pm(object)[invalid]

  return(mm.subtracted)
}
@ 
Once the method defined, we just register the \emph{label name} in the
corresponding variable.
<<>>=
upDate.pmcorrect.methods(c(pmcorrect.methods(), "subtractmmsometimes"))
@

The second new method intends to be an robust alternative to the summary
expression value computation \verb+avgdiff+. The idea is to use the function
\verb+huber+ of the package \verb+MASS+.
<<>>=
huber <- function (y, k = 1.5, tol = 1e-06) {
    y <- y[!is.na(y)]
    n <- length(y)
    mu <- median(y)
    s <- mad(y)
    if (s == 0) 
        stop("cannot estimate scale: MAD is zero for this sample")
    repeat {
        yy <- pmin(pmax(mu - k * s, y), mu + k * s)
        mu1 <- sum(yy)/n
        if (abs(mu - mu1) < tol * s) 
            break
        mu <- mu1
    }
    list(mu = mu, s = s)
}
@
This method returns P.J. Huber's location estimate with MAD scale. You
think this is what you want to compute the summary expression value
from the probe intensities. What is needed to have as a processing method
is a simple wrapper:
<<>>=
computeExprVal.huber <- function(probes) {
  res <- apply(probes, 2, huber)
  mu <- unlist(lapply(res, function(x) x$mu))
  s <- unlist(lapply(res, function(x) x$s))
  return(list(exprs=mu, se.exprs=s))
}

upDate.generateExprSet.methods(c(generateExprSet.methods(), "huber"))
@ 

From now the package is aware of the two new methods\ldots in this session.

The code for the methods included in the package can be informative if you plan
to develop methods. An example that demonstrates how a normalization method can
be added is given by the function \Rfunction{normalize.AffyBatch.vsn} in the
package \Rpackage{vsn}; see its help file.

\end{document}

% -*- mode: noweb; noweb-default-code-mode: R-mode; -*-
%\VignetteIndexEntry{1. Primer}
%\VignetteKeywords{Preprocessing, Affymetrix}
%\VignetteDepends{affy}
%\VignettePackage{affy}
%documentclass[12pt, a4paper]{article}
\documentclass[12pt]{article}

\usepackage{amsmath}
\usepackage{hyperref}
\usepackage[authoryear,round]{natbib}

\textwidth=6.2in
\textheight=8.5in
%\parskip=.3cm
\oddsidemargin=.1in
\evensidemargin=.1in
\headheight=-.3in

\newcommand{\scscst}{\scriptscriptstyle}
\newcommand{\scst}{\scriptstyle}
\newcommand{\Rfunction}[1]{{\texttt{#1}}}
\newcommand{\Robject}[1]{{\texttt{#1}}}
\newcommand{\Rpackage}[1]{{\textit{#1}}}

\author{Laurent Gautier, Rafael Irizarry, Leslie Cope, and Ben Bolstad}
\begin{document}
\title{Description of affy}

\maketitle
\tableofcontents
\section{Introduction}
The \Rpackage{affy} package is part of the
BioConductor\footnote{\url{http://bioconductor.org/}} project.
It is meant to be an extensible, interactive environment for data analysis
and exploration of Affymetrix oligonucleotide array probe level data.

The software utilities provided with the Affymetrix software suite
summarizes the probe set intensities to form one {\it expression measure}
for each gene. The expression measure is the data available for analysis.
However, as pointed out by \cite{li:wong:2001a}, much can be learned from
studying the individual probe intensities, or as we call them, the
{\it probe level data}. This is why we developed this package. The package
includes plotting functions for the probe level data useful for quality
control, RNA degradation assessments, different probe level normalization
and background correction procedures, and flexible functions that permit
the user to convert probe level data to expression measures. The package
includes utilities for computing expression measures similar to MAS 4.0's
AvDiff \citep{affy4}, MAS 5.0's signal \citep{affy5},
DChip's MBEI \citep{li:wong:2001a}, and RMA \citep{iriz:etal:2003}.

We assume that the reader is already familiar with oligonucleotide
arrays and with the design of the Affymetrix GeneChip arrays.
If you are not, we recommend the Appendix of the Affymetrix MAS manual
\cite{affy4,affy5}.

The following terms are used throughout this document:
\begin{description}
\item[probe] oligonucleotides of 25 base pair length used to
probe RNA targets.
\item[perfect match] probes intended to match perfectly the
target sequence.
\item[$PM$] intensity value read from the perfect matches.
\item[mismatch] the probes having one base mismatch
  with the target sequence intended to account for non-specific binding.
\item[$MM$] intensity value read from the mis-matches.
\item[probe pair] a unit composed of a  perfect
match and its mismatch.
\item[affyID] an identification for a probe set (which can be a gene
or a fraction of a gene) represented on the array.
\item[probe pair set] $PM$s and $MM$s related to a common {\it affyID}.
\item[{\it CEL} files] contain measured intensities and locations for
an array that has been hybridized.
\item[{\it CDF} file] contain the information relating probe pair sets
to locations on the array.
\end{description}

Section \ref{whatsnew} describes the main differences between version
1.5 and
this version (1.6). Section \ref{sec:get.started} describes a quick
way of getting started and getting expression measures. Section
\ref{qc} describes some quality control tools. Section \ref{s1.4}
describes normalization routines. Section \ref{classes} describes
the different classes in the package. \ref{sec:probesloc} describes
our strategy to map probe locations to probe set membership. Section
\ref{configure.options} describes how to change the package's default
options. Section \ref{whatwasnew} describes earlier changes.

%%%make sure to change this when we get a publication about version 2.
{\bf Note:} If you use this package please cite
\cite{gaut:cope:bols:iriz:2003} and/or \cite{iriz:gaut:cope:2003}.

\section{Changes for affy in BioC 1.8 release}
\label{whatsnew}
There were relatively few changes.

\begin{itemize}
\item MAplot now accepts the argument \Rfunction{plot.method} which can be
used to call smoothScatter.
\item \Rfunction{normalize.quantiles.robust} has had minor changes.
\item \Rfunction{ReadAffy} can optionally return the SD values stored in
the cel file.
\item The C parsing code has been moved to the \Rpackage{affyio} package,
which is now a dependency of the affy package. This change should be
transparent to users as \Rpackage{affyio} will be automatically loaded
when affy is loaded.
\item Added a cdfname argument to \Rfunction{justRMA} and
\Rfunction{ReadAffy} to allow for the use of alternative cdf packages.
\end{itemize}

\section{Getting Started: From probe level data to expression values}
\label{sec:get.started}

The first thing you need to do is {\bf load the package}.
\begin{Sinput}
R> library(affy) ##load the affy package
\end{Sinput}
<<echo=F,results=hide>>=
library(affy)
@
This release of the \Rpackage{affy} package will automatically
download the appropriate cdf environment when you require it. However,
if you wish you may download and install the cdf environment you need
from
\url{http://bioconductor.org/help/bioc-views/release/data/annotation/}
manually. If there is no cdf environment currently built for your
particular chip and you have access to the CDF file then you may use
the \Rpackage{makecdfenv} package to create one yourself. To make the
cdf packaes, Microsoft Windows users will need to use the tools
described in \url{http://www.murdoch-sutherland.com/Rtools/}.


\subsection{Quick start}
If all you want is to go from probe level data ({\it Cel} files) to
expression measures here are some quick ways.

If you want is RMA, the quickest way of reading in data and getting
expression measures  is the following:
\begin{enumerate}
\item Create a directory, move all the relevant
{\it CEL} files to that directory
\item If using linux/unix, start R in that directory.
\item If using the Rgui for
Microsoft Windows make sure your working directory contains the {\it
Cel} files (use ``File -> Change Dir'' menu item).
\item Load the library.
\begin{Sinput}
R> library(affy) ##load the affy package
\end{Sinput}
\item Read in the data and create an expression, using RMA for example.
\begin{Sinput}
R> Data <- ReadAffy() ##read data in working directory
R> eset <- rma(Data)
\end{Sinput}
\end{enumerate}

Depending on the size of your dataset and on the memory available to your
system, you might experience errors like `Cannot allocate vector \ldots'.
An obvious option is to increase the memory available to your R process
(by adding memory and/or closing external applications\footnote{UNIX-like
systems users might also want to check {\it ulimit} and/or compile
{\bf R} and the package for 64 bits when possible.}.
An another option is to use the function \Rfunction{justRMA}.
\begin{Sinput}
R> eset <- justRMA()
\end{Sinput}
This reads the data and performs the `RMA' way to preprocess them at
the {\it C} level. One does not need to call \verb+ReadAffy+, probe level
data is never stored in an AffyBatch. \verb+rma+ continues to be the
recommended function for computing RMA.


The \Rfunction{rma} function was written in C for speed and
efficiency. It uses the expression measure described in \cite{iriz:etal:2003}.

For other popular methods use \Rfunction{expresso} instead of
\Rfunction{rma} (see Section \ref{expresso}). For example for our
version of MAS 5.0 signal uses
expresso (see code). To get mas 5.0  you can use
\begin{Sinput}
R> eset <- mas5(Data)
\end{Sinput}
which will also normalize the expression values. The normalization can
be turned off through the \verb+normalize+ argument.

In all the above examples, the variable \Robject{eset} is an object of
class \Robject{ExpressionSet} described in the Biobase vignette. Many
of the packages in BioConductor work on objects of this class. See the
\Rpackage{genefilter} and \Rpackage{geneplotter} packages for some examples.

If you want to use some other analysis package, you can write out the
expression values to file using the following command:
\begin{Sinput}
R> write.exprs(eset, file="mydata.txt")
\end{Sinput}

\subsection{Reading CEL file information}
The function \Rfunction{ReadAffy} is quite flexible. It lets you specify the
filenames, phenotype, and MIAME information. You can enter them by reading
files (see the help file) or widgets (you need to have the tkWidgets package
installed and working).

\begin{Sinput}
R> Data <- ReadAffy(widget=TRUE) ##read data in working directory
\end{Sinput}
This function call will pop-up a file browser widget, see Figure \ref{fig:widget.filechooser},
that provides an easy way of choosing cel files.

\newpage
\begin{figure}[htbp]
  \begin{center}
    \includegraphics{widgetfilechooser}
    \caption{\label{fig:widget.filechooser}Graphical display for selecting
      {\it CEL} files. This widget is part of the {\it tkWidgets} package.
      (function written by Jianhua (John) Zhang).
    }
  \end{center}
\end{figure}

Next, a widget (not shown) permits the user to enter the \verb+phenoData+.
%%See Figure \ref{fig:widget.pd}.
%% \begin{figure}[htbp]
%% \begin{center}
%% \begin{tabular}{c}
%% \includegraphics{numcovariates}\\
%% \includegraphics{namecovariates}\\
%% \includegraphics{assigncovariates}
%% \end{tabular}
%% \caption{\label{fig:widget.pd}Graphical display for entering phenoData
%% This widget is part
%% of the {\it tkWidgets} package.}
%% % (functions written by Majnu John.}
%% \end{center}
%% \end{figure}
Finally the a widget is presented for the user to enter MIAME information.
%%Seen in Figure \ref{fig:widget.tkMIAME}.


%% \begin{figure}[htbp]
%% \begin{center}
%% \includegraphics[width=0.5\textwidth]{widgettkMIAME}
%% \caption{\label{fig:widget.tkMIAME}Graphical display for entering {\it
%%     MIAME} informations. This widget is part of the {\it tkWidgets}
%%     package.}
%% % (function written by Majnu John).}
%% \end{center}
%% \end{figure}

Notice that it is not necessary to use widgets to enter this
information. Please read the help file for more information on how to
read it from flat files or to enter it programmatically.

The function \Rfunction{ReadAffy} is a wrapper for the functions
\Rfunction{read.affybatch}, \Rfunction{tkSampleNames},
\Rfunction{read.AnnotatedDataFrame}, and \Rfunction{read.MIAME}.
The function \Rfunction{read.affybatch} has some nice feature that
make it quite flexible. For example, the \verb+compression+ argument
permit the user to read compressed {\it CEL} files. The argument
{\it compress} set to {\it TRUE} will inform the readers that your files
are compressed and let you read them while they remain compressed.
The compression formats {\it zip} and {\it gzip} are known to be recognized.

A comprehensive description of all these options is found in
the help file:
\begin{Sinput}
R> ?read.affybatch
R> ?read.AnnotatedDataFrame
R> ?read.MIAME
\end{Sinput}


\subsection{Expression measures}

The most common operation is certainly to convert  probe level data to
expression values.
Typically this is achieved through the following sequence:
\begin{enumerate}
\item reading in probe level data.
\item background correction.
\item normalization.
\item probe specific background correction, e.g. subtracting $MM$.
\item summarizing the probe set values into one expression measure
and, in some cases, a standard error for this summary.
\end{enumerate}
We detail what we believe is a good way to proceed below.
As mentioned the function \Rfunction{expresso} provides many
options. For example,
\begin{Sinput}
R> eset <- expresso(Dilution, normalize.method="qspline",
               bgcorrect.method="rma",pmcorrect.method="pmonly",
               summary.method="liwong")
\end{Sinput}

This will store expression values, in the object \Robject{eset}, as an
object of class \Robject{ExpressionSet} (see the \Rpackage{Biobase}
package). You can either use R and the BioConductor packages to analyze
your expression data or if you rather use another package you can write
it out to a tab delimited file like this

\begin{Sinput}
R> write.exprs(eset, file="mydata.txt")
\end{Sinput}

In the \verb+mydata.txt+ file, row will represent genes and columns
will represent samples/arrays. The first row will be a header describing
the columns. The first column will have the {\it affyID}s. The
\Rfunction{write.exprs} function is quite flexible on what it writes (see
the help file).


\subsubsection{expresso}
\label{expresso}
The function \Rfunction{expresso} performs the steps background
correction, normalization, probe specific correction, and summary
value computation. We now show this using an \Robject{AffyBatch}
included in the package for examples. The command
\verb+data(Dilution)+ is used to load these data.

Important parameters for the expresso function are:

\begin{description}
\item[bgcorrect.method]. The background correction method to use.
The available methods are
<<>>=
bgcorrect.methods()
@

\item[normalize.method]. The normalization method to use. The
   available methods can be queried by using
\verb+normalize.methods+.
<<>>=
library(affydata)
data(Dilution) ##data included in the package for examples
normalize.methods(Dilution)
@

\item[pmcorrect.method]
The method for probe specific correction. The available methods are
<<>>=
pmcorrect.methods()
@

\item[summary.method]. The summary method to use. The available
methods are
<<>>=
express.summary.stat.methods()
@
Here we use \Rfunction{mas} to refer to the methods described in the
Affymetrix manual version 5.0.

\item[widget]
Making the \verb+widget+ argument \verb+TRUE+,  will let you select
missing parameters (like the normalization method, the background correction
method or the summary method).
Figure \ref{fig:expressochooser} shows the widget for the selection of
preprocessing methods for each of the steps.

\begin{Sinput}
R> expresso(Dilution, widget=TRUE)
\end{Sinput}

\begin{figure}[htbp]
\begin{center}
\includegraphics[width=0.5\textwidth]{EWSnap}
\caption{\label{fig:expressochooser}Graphical display for
selecting expresso methods.}
\end{center}
\end{figure}

\end{description}

There is a separate vignette {\bf affy: Built-in Processing Methods}
which explains in more detail what each of the preprocessing options
does.

\subsubsection{MAS 5.0}
To obtain expression values that correspond to those from MAS 5.0, use
\Rfunction{mas5}, which wraps \Rfunction{expresso} and
\Rfunction{affy.scalevalue.exprSet}.

<<>>=
eset <- mas5(Dilution)
@

To obtain MAS 5.0 presence calls you can use the \verb+mas5calls+ method.

<<>>=
Calls <- mas5calls(Dilution)
@
This returns an \verb+ExpressionSet+ object containing P/M/A calls and
their associated Wilcoxon p-values.

\subsubsection{Li and Wong's MBEI (dchip)}
To obtain our version of Li and Wong's MBEI one can use
\begin{Sinput}
R> eset <- expresso(Dilution, normalize.method="invariantset",
                 bg.correct=FALSE,
                 pmcorrect.method="pmonly",summary.method="liwong")
\end{Sinput}

This gives the current $PM$-only default. The reduced model (previous
default) can be obtained using \verb+pmcorrect.method="subtractmm"+.

\subsubsection{C implementation of RMA}
One of the quickest ways to compute expression using the \Rpackage{affy}
package is to use the \Rfunction{rma} function. We have found that this
method allows a user to compute the RMA expression measure in a matter
of minutes for datasets that may have taken hours in previous versions
of \Rpackage{affy}. The function serves as an interface to a hard coded C
implementation of the RMA method \citep{iriz:etal:2003}. Generally, the
following would be sufficient to compute RMA expression measures:
<<>>=
eset <- rma(Dilution)
@

Currently the \Rfunction{rma} function implements RMA in the following manner
\begin{enumerate}
\item Probe specific correction of the PM probes using a model based
  on observed intensity being the sum of signal and noise
\item Normalization of corrected PM probes using quantile
  normalization \citep{bols:etal:2003}
\item Calculation of Expression measure using median polish.
\end{enumerate}

The \Rfunction{rma} function is likely to be improved and extended in the
future as the RMA method is fine-tuned.

\newpage


\section{Quality Control through Data Exploration}
\label{qc}

For the users convenience we have included the \verb+Dilution+ sample data set:

<<>>=
Dilution
@

This will create the \verb+Dilution+ object of class \Robject{AffyBatch}.
\Rfunction{print} (or \Rfunction{show}) will display summary information.
These objects represent data from one experiment. The \Robject{AffyBatch}
class combines the information of various {\it CEL} files with a common
{\it CDF} file. This class is designed to keep information of one experiment.
The probe level data is contained in this object.

The data in \verb+Dilution+ is a small sample of probe sets from 2
sets of duplicate arrays hybridized with different concentrations of the
same RNA. This information is part of the \Robject{AffyBatch} and can be
accessed with the \verb+phenoData+ and \verb+pData+ methods:
<<>>=
phenoData(Dilution)
pData(Dilution)
@

Several of the functions for plotting summarized probe level data are
useful for diagnosing problems with the data. The plotting functions
\Rfunction{boxplot} and \Rfunction{hist} have methods for
\Robject{AffyBatch} objects.  Each of these functions presents
side-by-side graphical summaries of intensity information from each
array.  Important differences in the distribution of intensities are
often evident in these plots. The function \Rfunction{MAplot}
(applied, for example, to \verb+pm(Dilution)+), offers pairwise
graphical comparison of intensity data. The option \verb+pairs+ permits
you to chose between all pairwise comparisons (when \verb+TRUE+) or
compared to a reference array (the default). These plots can be particularly
useful in diagnosing problems in replicate sets of arrays. The function
argument \verb+plot.method+ can be used to create a MAplot using a
smoothScatter, rather than the default method which is to draw every point.

\begin{figure}[htbp]
\begin{center}
<<fig=TRUE>>=
data(Dilution)
MAplot(Dilution,pairs=TRUE,plot.method="smoothScatter")
@
\end{center}
\caption{Pairwise MA plots}
\end{figure}


\subsection{Accessing $PM$ and $MM$ Data}
The $PM$ and $MM$  intensities and corresponding {\it affyID} can be
accessed with the \Rfunction{pm}, \Rfunction{mm}, and
\Rfunction{probeNames} methods. These will be matrices with rows
representing probe pairs and columns representing arrays. The gene
name associated with the probe pair in row $i$ can be found in the
$i$th entry of the vector returned by \Rfunction{probeNames}.

<<>>=
Index <- c(1,2,3,100,1000,2000) ##6 arbitrary probe positions
pm(Dilution)[Index,]
mm(Dilution)[Index,]
probeNames(Dilution)[Index]
@
\verb+Index+ contains six arbitrary probe positions.

Notice that the column names of $PM$ and $MM$ matrices are the sample names
and the row names are the {\it affyID}, e.g. \verb+1001_at+ and
\verb+1000_at+ together with the probe number (related to position in the
target sequence).

<<>>=
sampleNames(Dilution)
@

{\bf Quick example:} To see what percentage of the $MM$ are larger
than the $PM$ simply type
<<>>=
mean(mm(Dilution)>pm(Dilution))
@

The \Rfunction{pm} and \Rfunction{mm} functions can be used to extract
specific probe set intensities.
<<>>=
gn <- geneNames(Dilution)
pm(Dilution, gn[100])
@
The method \Rfunction{geneNames} extracts the unique {\it affyID}s. Also
notice that the 100th probe set is different from the 100th probe! The
100th probe is not part of the the 100th probe set.

The methods \Rfunction{boxplot}, \Rfunction{hist}, and \Rfunction{image}
are useful for quality control. Figure \ref{f3} shows kernel density
estimates (rather than histograms) of $PM$ intensities for the 1st and
2nd array of the \verb+Dilution+ also included in the package.

\subsection{Histograms, Images, and Boxplots}
\begin{figure}[htbp]
\begin{center}
<<fig=TRUE>>=
hist(Dilution[,1:2]) ##PM histogram of arrays 1 and 2
@
\caption{\label{f3} Histogram of $PM$ intensities for 1st and 2nd array}
\end{center}
\end{figure}

As seen in the previous example, the sub-setting method \verb+[+ can be
used to extract specific arrays. {\bf NOTE: Sub-setting is different in
this version. One can no longer subset by gene. We can only define subsets
by one dimension: the columns, i.e. the arrays. Because the \verb+Cel+
class is no longer available \verb+[[+ is no longer available.} %]]

The method \verb+image()+ can be used to detect spatial artifacts.
By default we look at log transformed intensities. This can be
changed through the \verb+transfo+ argument.

<<eval=FALSE>>=
par(mfrow=c(2,2))
image(Dilution)
@

\begin{figure}[htbp]
\begin{center}
\includegraphics{image}
\caption{\label{f1} Image of the log intensities.}
\end{center}
\end{figure}

These images are quite useful for quality control. We recommend
examining these images as a first step in data exploration.

The method \Rfunction{boxplot} can be used to show $PM$, $MM$ or both intensities.
\begin{figure}[htbp]
\begin{center}
<<fig=TRUE>>=
par(mfrow=c(1,1))
boxplot(Dilution, col=c(2,3,4))
@
\caption{\label{f4}Boxplot of arrays in Dilution data.}
\end{center}
\end{figure}
As discussed in the next section this plot shows that we need to
normalize these arrays.


\subsection{RNA degradation plots}
The functions \Rfunction{AffyRNAdeg}, \Rfunction{summaryAffyRNAdeg}, and
\Rfunction{plotAffyRNAdeg} aid in assessment of RNA quality. Individual
probes in a probeset are ordered by location relative to the $5'$ end of
the targeted RNA molecule.\cite{affy4}  Since RNA degradation typically
starts from the $5'$ end of the molecule, we would expect probe
intensities to be systematically lowered at that end of a probeset when
compared to the $3'$ end.  On each chip, probe intensities are averaged
by location in probeset, with the average taken over probesets. The
function \Rfunction{plotAffyRNAdeg} produces a side-by-side plots of these
means, making it easy to notice any $5'$ to $3'$ trend.  The function
\Rfunction{summaryAffyRNAdeg} produces a single summary statistic for each
array in the batch, offering a convenient measure of the severity of
degradation and significance level.  For an example
<<>>=
deg <- AffyRNAdeg(Dilution)
names(deg)
@
does the degradation analysis and returns a list with various
components. A summary can be obtained using
<<>>=
summaryAffyRNAdeg(deg)
@

Finally a plot can be created using \Rfunction{plotAffyRNAdeg}, see Figure \ref{f4.3}.

\begin{figure}[htbp]
\begin{center}
<<fig=TRUE>>=
plotAffyRNAdeg(deg)
@
\caption{\label{f4.3} Side-by-side plot produced by plotAffyRNAdeg.}
\end{center}
\end{figure}

\newpage

\section{Normalization}
\label{s1.4}
Various researchers have pointed out the need for normalization of Affymetrix
arrays. See for example \cite{bols:etal:2003}. The method \verb+normalize+
lets one normalize at the probe level

<<>>=
Dilution.normalized <- normalize(Dilution)
@

For an extended example on normalization please refer to the vignette
in the affydata package.

\section{Classes}
\label{classes}
\verb+AffyBatch+ is the main class in this package. There are three
other auxiliary classes that we also describe in this Section.
\subsection{AffyBatch}
The AffyBatch class has slots to keep all the probe level information
for a batch of {\it Cel} files, which usually represent an experiment.
It also stores phenotypic and MIAME information as does the
\verb+ExpressionSet+ class in the Biobase package (the base package for
BioConductor). In fact, \verb+AffyBatch+ extends \verb+ExpressionSet+.

The expression matrix in \verb+AffyBatch+ has columns representing the
intensities read from the different arrays. The rows represent the
{\it cel} intensities for all position on the array. The cel intensity
with physical coordinates\footnote{Note that in the {\it .CEL} files
the indexing starts at zero while it starts at 1 in the package
(as indexing starts at 1 in {\bf R}).} $(x,y)$ will be in row
\[i = x + \mathtt{nrow} \times (y - 1)\]. The \verb+ncol+ and
\verb+nrow+ slots contain the physical rows of the array. Notice that
this is different from the dimensions of the expression matrix. The
number of row of the expression matrix is equal to
\verb+ncol+$\times$\verb+nrow+. We advice the use of the functions
\verb+xy2indices+ and \verb+indices2xy+ to shuttle from X/Y
coordinates to indices.

For compatibility with previous versions the accessor method
\verb+intensity+ exists for obtaining the expression matrix.

The \verb+cdfName+ slot contains the necessary information for the
package to find the locations of the probes for each probe set. See
Section \ref{sec:probesloc} for more on this.

\subsection{ProbeSet}
The \verb+ProbeSet+ class holds the information of all
the probes related to an {\it affyID}. The components are \verb+pm+ and
\verb+mm+.

The method \verb+probeset+ extracts probe sets from \verb+AffyBatch+
objects. It takes as arguments an \verb+AffyBatch+ object and a vector of
{\it affyIDs} and returns a list of objects of class \verb+ProbeSet+
<<>>=
gn <- featureNames(Dilution)
ps <- probeset(Dilution, gn[1:2])
#this is what i should be using: ps
show(ps[[1]])
@

The \verb+pm+ and \verb+mm+ methods can be used to extract these
matrices (see below).

This function is general in the way it defines a probe set. The
default is to use the definition of a probe set given by Affymetrix in
the CDF file. However, the user can define arbitrary probe sets. The
argument \verb+locations+ lets the user decide the row numbers in the
\verb+intensity+ that define a probe set. For example, if we are
interested in redefining the \verb+AB000114_at+ and \verb+AB000115_at+
probe sets, we could do the following:

First, define the locations of the $PM$ and $MM$ on the array of the
\verb+1000_at+ and \verb+1001_at+ probe sets
<<>>=
mylocation <- list("1000_at"=cbind(pm=c(1,2,3),mm=c(4,5,6)),
                   "1001_at"=cbind(pm=c(4,5,6),mm=c(1,2,3)))
@
The first column of the matrix defines the location of the $PM$s and
the second column the $MM$s.

Now we are ready to extract the \verb+ProbSet+s using the
\verb+probeset+ function:
<<>>=
ps <- probeset(Dilution, genenames=c("1000_at","1001_at"),
                 locations=mylocation)
@
Now, \verb+ps+ is list of \verb+ProbeSet+s. We can see the $PM$s and
$MM$s of each component using the \verb+pm+ and \verb+mm+ accessor methods.
<<>>=
pm(ps[[1]])
mm(ps[[1]])
pm(ps[[2]])
mm(ps[[2]])
@

This can be useful in situations where the user wants to determine if
leaving out certain probes improves performance at the expression level.
It can also be useful to combine probes from different human chips, for
example by considering only probes common to both arrays.

Users can also define their own environment for probe set location
mapping. More on this in Section \ref{sec:probesloc}.

An example of a \verb+ProbeSet+ is included in the package. A spike-in
data set is included in the package in the form of a list of
\verb+ProbeSet+s. The help file describes the data set.
Figure \ref{f5.3} uses this data set to demonstrate that the $MM$
also detect transcript signal.

\begin{figure}[htbp]
\begin{center}
<<fig=TRUE>>=
data(SpikeIn) ##SpikeIn is a ProbeSets
pms <- pm(SpikeIn)
mms <- mm(SpikeIn)

##pms follow concentration
par(mfrow=c(1,2))
concentrations <- matrix(as.numeric(sampleNames(SpikeIn)),20,12,byrow=TRUE)
matplot(concentrations,pms,log="xy",main="PM",ylim=c(30,20000))
lines(concentrations[1,],apply(pms,2,mean),lwd=3)
##so do mms
matplot(concentrations,mms,log="xy",main="MM",ylim=c(30,20000))
lines(concentrations[1,],apply(mms,2,mean),lwd=3)
@
\caption{\label{f5.3}PM and MM intensities plotted against SpikeIn concentration}
\end{center}
\end{figure}


\section{Location to ProbeSet Mapping}
\label{sec:probesloc}
On Affymetrix GeneChip arrays, several probes are used to represent genes
in the form of probe sets. From a {\it CEL} file we get for each physical
location, or cel, (defined by $x$ and $y$ coordinates) an intensity. The
{\it CEL} file also contains the name of the {\it CDF} file needed for the
location-probe-set mapping. The {\it CDF} files store the probe set related
to each location on the array. The computation of a summary expression values
from the probe intensities requires a fast way to map an {\it affyid} to
corresponding probes. We store this mapping information in {\bf R}
environments\footnote{Please refer to the {\bf R} documentation to know more about environments.}.
They only contain a part of the information that can be found in the {\it CDF}
files. The {\it cdfenvs} are sufficient to perform the numerical processing
methods included in the package. For each {\it CDF} file there is package,
available from \url{http://bioconductor.org/help/bioc-views/release/data/annotation/}, that
contains exactly one of these environments. The {\it cdfenvs} we store the
$x$ and $y$ coordinates as one number (see above).

In instances of {\it AffyBatch}, the {\it cdfName} slot gives the name of the
appropriate {\it CDF} file for arrays represented in the \verb+intensity+ slot.
The functions \verb+read.celfile+, \verb+read.affybatch+, and \verb+ReadAffy+
extract the {\it CDF} filename from the {\it CEL} files being read. Each
{\it CDF} file corresponds to exactly one environment. The function
\verb+cleancdfname+ converts the Affymetrix given {\it CDF} name to a
BioConductor environment and annotation name. Here are two examples:

These give environment names:
<<>>=
cat("HG_U95Av2 is",cleancdfname("HG_U95Av2"),"\n")
cat("HG-133A is",cleancdfname("HG-133A"),"\n")
@

This gives annotation name:
<<>>=
cat("HG_U95Av2 is",cleancdfname("HG_U95Av2",addcdf=FALSE),"\n")
@

An environment representing the corner of an Hu6800 array is available with
the package. In the following, we load the environment, look at the names for
the first 5 objects defined in the environment, and finally look at the first
object in the environment:
<<>>=
data(cdfenv.example)
ls(cdfenv.example)[1:5]
get(ls(cdfenv.example)[1],cdfenv.example)
@

The package needs to know what locations correspond to which probe
sets. The \verb+cdfName+ slot contains the necessary information to
find the environment with this location information. The method
\verb+getCdfInfo+ takes as an argument an \verb+AffyBatch+ and
returns the necessary environment. If \verb+x+ is an \verb+AffyBatch+,
this function will look for an environment with name
\verb+cleancdfname(x@cdfName)+.

<<>>=
print(Dilution@cdfName)
myenv <- getCdfInfo(Dilution)
ls(myenv)[1:5]
@

By default we search for the environment first in the global
environment, then in a package named \verb+cleancdfname(x@cdfName)+.

Various methods exist to obtain locations of probes as demonstrated
in the following examples:
<<>>=
Index <- pmindex(Dilution)
names(Index)[1:2]
Index[1:2]
@
\verb+pmindex+ returns a list with probe set names as names and
locations in the components. We can also get specific probe sets:
<<>>=
pmindex(Dilution, genenames=c("1000_at","1001_at"))
@
The locations are ordered from 5' to 3' on the target transcript.
The function \verb+mmindex+ performs in a similar way:
<<>>=
mmindex(Dilution, genenames=c("1000_at","1001_at"))
@
They both use the method \verb+indexProbes+
<<>>=
indexProbes(Dilution, which="pm")[1]
indexProbes(Dilution, which="mm")[1]
indexProbes(Dilution, which="both")[1]
@
The \verb+which="both"+ options returns the location of the $PM$s
followed by the $MM$s.


\section{Configuring the package options}
\label{configure.options}
Package-wide options can be configured, as shown below through examples.

\begin{itemize}
 \item Getting the names for the options:
<<>>=
opt <- getOption("BioC")
affy.opt <- opt$affy
print(names(affy.opt))
@
%$
 \item Default processing methods:
<<>>=
opt <- getOption("BioC")
affy.opt <- opt$affy
affy.opt$normalize.method <- "constant"
opt$affy <- affy.opt
options(BioC=opt)
@
%$
\item Compression of files: if you are always compressing your CEL files,
you might find annoying to specify it each time you call a reading function.
It can be specified once for all in the options.
<<>>=
opt <- getOption("BioC")
affy.opt <- opt$affy
affy.opt$compress.cel <- TRUE
opt$affy <- affy.opt
options(BioC=opt)
@
%$
\item Priority rule for the use of a cdf environment:
The option {\it probesloc} is a list. Each element of the list is itself a
list with two elements {\it what} and {\it where}. When looking for the
information related to the locations of the probes on the array, the elements
in the list will be looked at sequentially. The first one leading to the
information is used (an error message is returned if none permits to find the
information). The element {\it what} can be one of {\it package},
{\it environment}.
\end{itemize}


\section{Where can I get more information?}
\label{moreinfo}

There are several other vignettes addressing more specialised topics related to the {\tt affy} package.
\begin{itemize}
\item {\bf affy: Custom Processing Methods (HowTo)}: A description of how to use custom preprocessing methods with the package. This document gives examples of how you might write your own preprocessing method and use it with the package.
\item {\bf affy: Built-in Processing Methods}: A document giving fuller descriptions of each of the preprocessing methods that are available within the {\tt affy} package.
\item {\bf affy: Import Methods (HowTo)}: A discussion of the data structures used and how you might import non standard data into the package.
\item {\bf affy: Loading Affymetrix Data (HowTo)}: A quick guide to loading Affymetrix data into R.
\item {\bf affy: Automatic downloading of cdfenvs (HowTo)}: How you can configure the automatic downloading of the appropriate {\it cdfenv} for your analysis.
\end{itemize}


\appendix

\section{Previous Release Notes}

\subsection{Changes in versions 1.6.x}
There were very few changes.

\begin{itemize}
\item The function \verb+MAplot+ has been added. It works on instances of AffyBatch.
You can decide if you want to make all pairwise MA plots or compare to a
reference array using the pairs argument.
\item Minor bugs fixed in the parsers.
\item The path of celfiles is now removed by ReadAffy.
\end{itemize}


\subsection{Changes in versions 1.5.x}
There are some minor differences in what you can do but little
functionality has disappeared. Memory efficiency and speed have improved.

\begin{itemize}
\item The widgets used by ReadAffy have changed.
\item The path of celfiles is now removed by ReadAffy.
\end{itemize}

\subsection{Changes in versions 1.4.x}
There are some minor differences in what you can do but little
functionality has disappeared. Memory efficiency and speed have improved.

\begin{itemize}
\item For instances of \verb+AffyBatch+ the subsetting has
  changed. For consistency with \verb+exprSets+ one can only subset by
  the second dimension. So to obtain the first array, \verb+abatch[1]+
  and \verb+abatch[1,]+ will give warnings (errors in the next
  release). The correct code is \verb+abatch[,1]+.
\item mas5calls is now faster and reproduces Affymetrix's official
  version much better.
\item If you use \verb+pm+ and \verb+mm+ to get the entire set of
  probes, e.g. by typing \verb+pm(abatch)+ then the method will be, on
  average, about 2-3 times faster than in version 1.3.
\end{itemize}

\bibliographystyle{plainnat}
\bibliography{affy}

\end{document}

% -*- mode: noweb; noweb-default-code-mode: R-mode; -*-
%\VignetteIndexEntry{2. Built-in Processing Methods}
%\VignetteKeywords{Preprocessing, Affymetrix}
%\VignetteDepends{affy}
%\VignettePackage{affy}
%documentclass[12pt, a4paper]{article}
\documentclass[12pt]{article}

\usepackage{amsmath}
\usepackage{hyperref}
\usepackage[authoryear,round]{natbib}

\textwidth=6.2in
\textheight=8.5in
%\parskip=.3cm
\oddsidemargin=.1in
\evensidemargin=.1in
\headheight=-.3in

\newcommand{\scscst}{\scriptscriptstyle}
\newcommand{\scst}{\scriptstyle}
\author{Ben Bolstad}
\begin{document}
\title{affy: Built-in Processing Methods}
\maketitle
\tableofcontents
\section{Introduction}

This document describes the preprocessing methods that have currently
been built into the \verb+affy+ package.  Hopefully it will clarify
for the reader what each of the routines does. There is a separate
vignette which describes how you might write your own routines and use
them in combination with the built-in routines. 

As usual, loading the package in your \verb+R+ session is required. 
\begin{Sinput}
R> library(affy) ##load the affy package
\end{Sinput}
<<echo=F,results=hide>>=
library(affy)
@

\section{Background methods}

You can see the background correction methods that are built into the
package by examining the variable \verb+bgcorrect.method+. 

<<>>=
bgcorrect.methods()
@

\subsection{none}

Calling this method actually does nothing. It returns the object
unchanged. May be used as a placeholder. 

\subsection{rma/rma2}

These are background adjustment implementations for the rma method
\cite{iriz:etal:2003}. They differ only in how they estimate a set of
parameters (generally you should use \verb+rma+ in preference to
\verb+rma2+. In both cases PM probe intensities are corrected by using
a global model for the distribution of probe intensities. The model is
suggested by looking at plots of the empirical distribution of probe
intensities.  In particular the observed PM probes are modeled as the
sum of a normal noise component N (Normal with mean $\mu$ and variance
$\sigma^2$) and a exponential signal component S (exponential with
mean $\alpha$). To avoid any possibility of negatives, the normal is
truncated at zero. Given we have O the observed intensity, this then
leads to an adjustment. 

\begin{equation*}
E\left(s \lvert O=o\right) = a + b \frac{\phi\left(\frac{a}{b}\right)
  - \phi\left(\frac{o-a}{b}\right)}{\Phi\left(\frac{a}{b}\right) +
  \Phi\left(\frac{o-a}{b}\right) - 1 } 
\end{equation*}
where $a =  s- \mu - \sigma^2\alpha$ and $b = \sigma$. Note that
$\phi$ and $\Phi$ are the standard normal distribution density and
distribution functions respectively. 

Note that MM probe intensities are not corrected by either of these routines.

\subsection{mas}

This is an implementation of the background correction method outlined
in the Statistical Algorithms Description Document
\cite{affy:tech:2002}. The chip is broken into a grid of 16
rectangular regions. For each region the lowest 2\% of probe
intensities are used to compute a background value for that grid. Each
probe is then adjusted based upon a weighted average of the
backgrounds for each of the regions. The weights are based on the
distances between the location of the probe and the centriods of 16
different regions. Note this method corrects both PM and MM probes. 

\section{Normalization Methods}

You can see the background correction methods that are built into the
package by examining the variable \verb+bgcorrect.method+. 
<<>>=
normalize.AffyBatch.methods()
@

The Quantile, Contrast and Loess normalizations have been discussed
and compared in \cite{bols:etal:2003}. 

\subsection{quantiles/quantiles.robust}

The quantile method was introduced by \cite{bols:etal:2003}. The goal
is to give each chip the same empirical distribution. To do this we
use the following algorithm where $X$ is a matrix of probe intensities
(probes by arrays):  

\begin{enumerate}
\item Given $n$ array of length $p$, form $X$  of dimension $p \times n$  where
each array is a column
\item Sort each column of $X$ to give $X_{\mbox{sort}}$
\item Take the means across rows of $X_{\mbox{sort}}$ and assign this mean to
each element in the row to get $X'_{\mbox{sort}}$
\item Get $X_{\mbox{normalized}}$ by rearranging each column of
$X'_{\mbox{sort}}$ to have the same ordering as original $X$
\end{enumerate}
 
The quantile normalization method is a specific case of the
transformation $x'_{i} = F^{-1}\left(G\left(x_{i}\right)\right)$,
where we estimate $G$ by the empirical distribution of each array and
$F$ using the empirical distribution of the averaged sample quantiles.
Quantile normalization is pretty fast. 

The {\tt quantiles} function performs the algorithm as above. The
{\tt quantile.robust} function allows you to exclude or down-weight
arrays in the computation of $\hat G$ above. In most cases we have
found that the {\tt quantiles} method is sufficient for use and
{\tt quantiles.robust} not required.   

\subsection{loess}

There is a discussion of this method in \cite{bols:etal:2003}. It
generalizes the $M$ vs $A$ methodology proposed in \cite{Dudoit:2002}
to multiple arrays. It works in a pairwise manner and is thus slow
when used with a large number of arrays. 

\subsection{contrasts}

This method was proposed by \cite{astr:2003}. It is also a variation
on the  $M$ vs $A$ methodology, but the normalization is done by
transforming the data to a set of contrasts, then normalizing. 

\subsection{constant}

A scaling normalization. This means that all the arrays are scaled so
that they have the same mean value. This would be typical of the
approach taken by Affymetrix. However, the Affymetrix normalization is
usually done after summarization (you can investigate
\verb+affy.scalevalue.exprSet+ if you are interested) and this
normalization is carried out before summarization. 

\subsection{invariantset}

A normalization similar to that used in the dChip software
\cite{li:wong:2001a}. Using a baseline array, arrays are normalized by
selecting invariant sets of genes (or probes) then using them to fit a
non-linear relationship between the ``treatment'' and ``baseline''
arrays. The non-linear relationship is used to carry out the
normalization. 

\subsection{qspline}

This method is documented in \cite{workman:etal:2002}. Using a target array
(either one of the arrays or a synthetic target), arrays are normalized by
fitting splines to the quantiles, then using the splines to perform the
normalization.

\section{PM correct methods}

<<>>=
pmcorrect.methods()
@
\subsection{mas}

An {\it ideal mismatch} is subtracted from PM. The ideal mismatch is
documented by \cite{affy:tech:2002}. It has been designed so that you
subtract MM when possible (ie MM is less than PM) or something else
when it is not possible. The Ideal Mismatch will always be less than
the corresponding PM and thus we can safely subtract it without risk
of negative values.  

\subsection{pmonly}

Make no adjustment to the pm values.

\subsection{subtractmm}

Subtract MM from PM. This would be the approach taken in MAS 4
\cite{affy4}. It could also be used in conjunction with the Li-Wong
model. 

\section{Summarization methods}

<<>>=
express.summary.stat.methods()
@

\subsection{avgdiff}

Compute the average. This is the approach that was taken in \cite{affy4}.

\subsection{liwong}

This is an implementation of the methods proposed in
\cite{li:wong:2001a} and \cite{li:wong:2001b}. The Li-Wong MBEI is
based upon fitting the following multi-chip model to each probeset 
\begin{equation}
y_{ij} = \phi_i \theta_j + \epsilon_{ij}
\end{equation}
where $y_{ij}$ is $PM_{ij}$ or the difference between
$PM_{ij}-MM_{ij}$. The $\phi_i$ parameter is a probe response
parameter and $\theta_j$ is the expression on array $j$. 


\subsection{mas}

As documented in \cite{affy:tech:2002}, a robust average using 1-step
Tukey biweight on $\log_2$ scale. 

\subsection{medianpolish}

This is the summarization used in the RMA expression summary
\cite{iriz:etal:2003}. A multichip linear model is fit to data from
each probeset. In particular for a probeset $k$ with $i=1,\dots,I_k$
probes and data from $j=1,\dots,J$ arrays we fit the following model 
\begin{equation*}
\log_2\left(PM^{(k)}_{ij}\right) = \alpha_i^{(k)} + \beta_j^{(k)} + \epsilon_{ij}^{(k)}
\end{equation*}
where $\alpha_i$ is a probe effect and $\beta_j$ is the $\log_2$
expression value. The medianpolish is an algorithm (see
\cite{tukey:1977}) for fitting this model robustly. Please note that
expression values you get using this summary measure will be in
$\log_2$ scale. 

\subsection{playerout}

This method is detailed in \cite{Lazardis:etal:2002}. A non-parametric
method is used to determine weights. The expression value is then the
weighted average. 

\section{Putting it altogether using {\tt expresso}}

The function that you should use is {\tt expresso}. It is important to
note that not every preprocessing method can be combined together. In
particular the \verb+rma+ backgrounds adjust only PM probe intensities
and so they should only be used in conjunction with the \verb+pmonly+
PM correction. Also remember that the \verb+mas+ and
\verb+medianpolish+ summarization methods $\log_2$ transform the data,
thus they should not be used in conjunction with any preprocessing
steps that are likely to yield negatives like the \verb+subtractmm+ pm
correction method. The following is a typical call to
\verb+expresso+. 

\begin{Sinput}
 library(affydata)
 data(Dilution)
 eset <- expresso(Dilution,bgcorrect.method="rma",
                  normalize.method="quantiles",
                  pmcorrect.method="pmonly",
                  summary.method="medianpolish")
\end{Sinput}
%@ 

This would give you the RMA expression measure, but of course there
are other ways of computing RMA (chiefly \verb+rma+). The true power
of \verb+expresso+ becomes apparent when you start combining different
methods. By choosing a method for each of the four steps ({\tt
  bgcorrect.method}, {\tt normalize.method}, {\tt pmcorrect.method},
{\tt summary.method}) you can create quite a variety of expression
measures. For instance 

\begin{Sinput}
eset <- expresso(Dilution,bgcorrect.method="mas",
                  normalize.method="qspline",
                  pmcorrect.method="subtractmm",
                  summary.method="playerout")
\end{Sinput}

would be a valid way of computing an expression measure (it is up to
the user to decide whether such a concoction is sensible or not). 


\bibliographystyle{plainnat}
\bibliography{affy}

\end{document}

%\name{matdensity}
\name{plotDensity}
%\alias{matdensity}
\alias{plotDensity}
\alias{plotDensity.AffyBatch}
\title{Plot Densities}
\description{
Plots the non-parametric density estimates using values contained in the
columns of a matrix.
}
\usage{
%matdensity(mat, ylab = "density", xlab="x", type="l", ...)

plotDensity(mat, ylab = "density", xlab="x", type="l", col=1:6,
            na.rm = TRUE, ...)

plotDensity.AffyBatch(x, col = 1:6, log = TRUE,
                      which=c("pm","mm","both"),
                      ylab = "density",
                      xlab = NULL, ...)
}
\arguments{
  \item{mat}{a matrix containing the values to make densities in the columns.}
  \item{x}{an object of class \code{\link[affy:AffyBatch-class]{AffyBatch}}.}
  \item{log}{logical value. If \code{TRUE} the log of the intensities in
    the \code{AffyBatch} are plotted.}
  \item{which}{should a histogram of the PMs, MMs, or both be made?}
  \item{col}{the colors to use for the different arrays.}
  \item{ylab}{a title for the y axis.}
  \item{xlab}{a title for the x axis.}
  \item{type}{type for the plot.}
  \item{na.rm}{handling of \code{NA} values.}
  \item{\dots}{graphical parameters can be given as arguments to
    \code{\link{plot}}.}
}
\details{
  The list returned can be convenient for plotting large input matrices
  with different colors/line types schemes
  (the computation of the densities can take some time).

  To match other functions in base R, this function should probably be
  called \code{matdensity}, as it is sharing similarities with
  \code{matplot} and \code{matlines}. 
}
\value{
  It returns invisibly a list of two matrices `x' and `y'.
}
\author{Ben Bolstad and Laurent Gautier}
\examples{
if (require(affydata)) {
  data(Dilution)
  plotDensity(exprs(Dilution), log="x")
}
}
\keyword{hplot}

\name{plotLocation}
\alias{plotLocation}
\title{Plot a location on a cel image}
\description{
  Plots a location on a previously plotted cel image. This can be used
  to locate the physical location of probes on the array.
}
\usage{
plotLocation(x, col="green", pch=22, ...)
}
\arguments{
  \item{x}{a `location'. It can be obtained by the method of \code{AffyBatch}
    \code{indexProbes}, or made elsewhere (basically a location is nrows and
    two columns array. The first column corresponds to the x positions
    and the second columns corresponds to the y positions of n elements
    to locate).}
  \item{col}{colors for the plot.}
  \item{pch}{plotting type (see function \code{plot}).}
  \item{\dots}{other parameters passed to the function \code{points}.}
}
\author{
  Laurent
}
\seealso{
  \code{\link[affy:AffyBatch-class]{AffyBatch}}
}
\examples{
if (require(affydata)) {
  data(Dilution)

  ## image of the celfile
  image(Dilution[, 1])

  ## genenames, arbitrarily pick the 101th
  n <- geneNames(Dilution)[101]

  ## get the location for the gene n
  l <- indexProbes(Dilution, "both", n)[[1]]
  ## convert the index to X/Y coordinates
  xy <- indices2xy(l, abatch=Dilution) 

  ## plot
  plotLocation(xy)
}
}
\keyword{aplot}

\name{fit.li.wong}
\alias{fit.li.wong}
\alias{li.wong}
\title{Fit Li and Wong Model to a Probe Set}
\description{Fits the model described in Li and Wong (2001) to a probe
  set with I chips and J probes.
}
\usage{
fit.li.wong(data.matrix, remove.outliers=TRUE, normal.array.quantile=0.5,
            normal.resid.quantile=0.9, large.threshold=3, large.variation=0.8,
            outlier.fraction=0.14, delta=1e-06, maxit=50,
            outer.maxit=50,verbose=FALSE, ...)

li.wong(data.matrix,remove.outliers=TRUE, normal.array.quantile=0.5,
            normal.resid.quantile=0.9, large.threshold=3, large.variation=0.8,
            outlier.fraction=0.14, delta=1e-06, maxit=50,
            outer.maxit=50,verbose=FALSE)
}
\arguments{
  \item{data.matrix}{an I x J matrix containing the probe set data. Typically
    the i,j entry will contain the PM-MM value for probe pair j in chip i.
    Another possible use, is to use PM instead of PM-MM.}
  \item{remove.outliers}{logical value indicating if the algorithm
    will remove outliers according to the procedure described in Li and
    Wong (2001).}
  \item{large.threshold}{used to define outliers.}
  \item{normal.array.quantile}{quantile to be used when determining what
    a normal SD is. probes or chips having estimates with SDs bigger
    than the quantile \code{normal.array.quantile} of all SDs x
    \code{large.threshold}.}
  \item{normal.resid.quantile}{any residual bigger than the
    \code{normal.resid.quantile} quantile of all residuals x
    \code{large.threshold} is considered an outlier.}
  \item{large.variation}{any probe or chip describing more than this
    much total variation is considered an outlier.}
  \item{outlier.fraction}{this is the maximum fraction of single
    outliers that can be in the same probe or chip.}
  \item{delta}{numerical value used to define the stopping criterion.}
  \item{maxit}{maximum number of iterations when fitting the model.}
  \item{outer.maxit}{maximum number of iterations of defined outliers.}
  \item{verbose}{logical value. If \code{TRUE} information is given of
    the status of the algorithm.}
  \item{\dots}{additional arguments.}
}
\details{
  This is Bioconductor's implementation of the Li and Wong algorithm. The
  Li and Wong PNAS 2001 paper was followed. However, you will not get
  the same results as you would get with dChip. dChip is not open source
  so it is not easy to reproduce. 

  Notice that this iterative algorithm will not always converge.
  If you run the algorithm on thousands of probes expect some
  non-convergence warnings. These are more likely when few arrays are used.
  We recommend using this method only if you have 10 or more arrays.

  Please refer to references for more details.
}

\value{\code{li.wong} returns a vector of expression measures (or column
  effects) followed by their respective standard error estimates. It
  was designed to work with \code{express} which is no longer part of
  the package.

  \code{fit.li.wong} returns much more. Namely, a list containing the
  fitted parameters and relevant information. 
  \item{theta}{fitted thetas.}
  \item{phi}{fitted phis.}
  \item{sigma.eps}{estimated standard deviation of the error term.}
  \item{sigma.theta}{estimated standard error of theta.}
  \item{sigma.phi}{estimated standard error of phis.}
  \item{theta.outliers}{logical vector describing which chips (thetas) are
    considered outliers (\code{TRUE}).}
  \item{phi.outliers}{logical vector describing which probe sets (phis) are
    considered outliers (\code{TRUE})}
  \item{convergence1}{logical value. If \code{FALSE} the algorithm did
    not converge when fitting the phis and thetas.}
  \item{convergence2}{logical value. If \code{FALSE} the algorithm did
    not converge in deciding what are outliers.}
  \item{iter}{number of iterations needed to achieve convergence.}
  \item{delta}{difference between thetas when iteration stopped.}
}
\examples{
    x <- sweep(matrix(2^rnorm(600),30,20),1,seq(1,2,len=30),FUN="+")
    fit1 <- fit.li.wong(x)
    plot(x[1,])
    lines(fit1$theta)
}
\references{
  Li, C. and Wong, W.H. (2001) \emph{Genome Biology} \bold{2}, 1--11.\cr

  Li, C. and Wong, W.H. (2001) \emph{Proc. Natl. Acad. Sci USA}
  \bold{98}, 31--36.
}
\author{Rafael A. Irizarry, Cheng Li, Fred A. Wright, Ben Bolstad}
\seealso{\code{\link{li.wong}}, \code{\link{expresso}}}
\keyword{manip}
\keyword{models}

\name{mva.pairs}
\alias{mva.pairs}
\title{M vs. A Matrix}
\description{
 A matrix of M vs. A plots is produced. Plots are made on the upper
 triangle and the IQR of the Ms are displayed in the lower triangle
}
\usage{
mva.pairs(x, labels=colnames(x), log.it=TRUE,span=2/3,family.loess="gaussian",
          digits=3,line.col=2,main="MVA plot",cex=2,...)
}
\arguments{
  \item{x}{a matrix containing the chip data in the columns.}
  \item{labels}{the names of the variables.}
  \item{log.it}{logical. If \code{TRUE}, uses log scale.}
  \item{span}{span to be used for loess fit.}
  \item{family.loess}{\code{"gaussian"} or \code{"symmetric"} as in
    \code{\link[stats]{loess}}.}
  \item{digits}{number of digits to use in the display of IQR.}
  \item{line.col}{color of the loess line.}
  \item{main}{an overall title for the plot.}
  \item{cex}{size for text.}
  \item{\dots}{graphical parameters can be given as arguments to
    \code{mva.plot}}.}
\examples{
    x <- matrix(rnorm(4000),1000,4)
    x[,1] <- x[,1]^2
    dimnames(x) <- list(NULL,c("chip 1","chip 2","chip 3","chip 4"))
    mva.pairs(x,log=FALSE,main="example")
}
\seealso{\code{\link{pairs}}}
\keyword{hplot}

\name{generateExprSet-method}
\docType{methods}
\alias{generateExprSet-methods}
\alias{computeExprSet}
\alias{generateExprSet.methods}
\alias{upDate.generateExprSet.methods}
\title{generate a set of expression values}
\description{
  Generate a set of expression values from the probe pair
  information. The set of expression is returned as an
  \code{\link[Biobase:class.ExpressionSet]{ExpressionSet}} object.
}
\usage{
  computeExprSet(x, pmcorrect.method, summary.method, ...)

  generateExprSet.methods()

  upDate.generateExprSet.methods(x)
}
\arguments{
  \item{x}{a \code{\link[=AffyBatch-class]{AffyBatch}} holding the probe level
    informations to generate the expression values, for computeExprSet,
    and for upDate.generateExprSet.methods it is a character vector..}
  \item{pmcorrect.method}{the method used to correct PM values (see
    section 'details').}
  \item{summary.method}{the method used to generate the expression value (see
    section 'details').}
  \item{\dots}{any of the options of the normalization you would like to
    modify.}
}
\details{
  An extra argument \code{ids=} can be passed. It must be a vector of
  affids. The expression values will only be computed and returned for
  these affyids.
  
  The different methods available through this mechanism can be accessed
  by calling the method \code{generateExprSet.methods} with an object of
  call \code{Cel.container} as an argument.

  In the Affymetrix design, \emph{MM} probes were included to measure
  the noise (or background signal). The original algorithm for
  background correction was to subtract the \emph{MM} signal to the
  \emph{PM} signal. The methods currently included in the package are
  "bg.correct.subtractmm", "bg.correct.pmonly" and "bg.correct.adjust".

  To alter the available methods for generating ExprSets use
  upDate.generateExprSet.methods.
}
\seealso{
  method \code{generateExprSet} of the class
  \code{\link[=AffyBatch-class]{AffyBatch}}\cr
  \code{\link{expresso}} 
}
\examples{
if (require(affydata)) {
  data(Dilution)

  ids <- c( "1000_at","1001_at")

  eset <- computeExprSet(Dilution, pmcorrect.method="pmonly",
                         summary.method="avgdiff", ids=ids)
}
}
\keyword{manip}

\name{barplot.ProbeSet}
\alias{barplot.ProbeSet}
\title{show a ProbeSet as barplots}
\description{
  Displays the probe intensities in a ProbeSet as a barplots
}
\usage{
\method{barplot}{ProbeSet}(height, xlab = "Probe pair", ylab = "Intensity",
    main = NA, col.pm = "red", col.mm = "blue", beside = TRUE, names.arg = "pp",
    ask = TRUE, scale, ...)
}
\arguments{
  \item{height}{an object of class \code{ProbeSet}.}
  \item{xlab}{label for x axis.}
  \item{ylab}{label for y axis.}
  \item{main}{main label for the figure.}
  \item{col.pm}{color for the `pm' intensities.}
  \item{col.mm}{color for the `mm' intensities.}
  \item{beside}{bars beside each others or not.}
  \item{names.arg}{names to be plotted below each bar or group of bars.}
  \item{ask}{ask before ploting the next barplot.}
  \item{scale}{put all the barplot to the same scale.}
  \item{\dots}{extra parameters to be passed to \code{\link{barplot}}.}
}
\examples{
if (require(affydata)) {
  data(Dilution)
  gn <- geneNames(Dilution)
  pps <- probeset(Dilution, gn[1])[[1]]

  barplot.ProbeSet(pps)
}
}
\keyword{hplot}

\name{probeNames-methods}
\docType{methods}
\title{Methods for accessing the Probe Names}
\alias{probeNames-methods}
\alias{probeNames}
\alias{probeNames<-}
%\alias{probeName}
%\alias{probeName<-}
\description{Methods for accessing Probe Names}
\section{Methods}{\describe{
\item{object = Cdf}{an accessor function for the \code{name} slot.}
\item{object = probeNames}{returns the probe names associated with the
  rownames of the intensity matrices one gets with the \code{pm} and
  \code{mm} methods.}
}}
\keyword{methods}

\name{cleancdfname}
\alias{cleancdfname}
\alias{mapCdfName}
\title{Clean Affymetrix's CDF name}
\description{
This function converts Affymetrix's names for CDF files to the names
used in the annotation package and in all Bioconductor.
}
\usage{
cleancdfname(cdfname, addcdf = TRUE)
}
\arguments{
  \item{cdfname}{A \code{character} denoting Affymetrix'x CDF file name }
  \item{addcdf}{A \code{logical}. If \code{TRUE} it adds the string "cdf"
    at the end of the cleaned CDF name. This is used to name the
    \code{cdfenvs} packages.}
}
\details{
  This function takes a CDF filename obtained from an Affymetrix file
  (from a CEL file for example) and convert it to a convention of ours:
  all small caps and only alphanumeric characters. The details of the rule
  can be seen in the code.
  We observed exceptions that made us create a set of special cases for
  mapping CEL to CDF. The object \code{mapCdfName} holds information
  about these cases. It is a \code{data.frame} of three elements: the 
  first is the name as found in the CDF file, the second the name in the
  CEL file and the third the name in Bioconductor. \code{mapCdfName} can
  be loaded using \code{data(mapCdfName)}.
}
\value{
A \code{character}
}
\examples{
cdf.tags <- c("HG_U95Av2", "HG-133A")
for (i in cdf.tags)
  cat(i, "becomes", cleancdfname(i), "\n")
}
\keyword{character}

\name{tukey.biweight}
\alias{tukey.biweight}
\title{One-step Tukey's biweight}
\description{
  One-step Tukey's biweight on a matrix.
}
\usage{
tukey.biweight(x, c = 5, epsilon = 1e-04)
}
\arguments{
  \item{x}{a matrix.}
  \item{c}{tuning constant (see details).}
  \item{epsilon}{fuzzy value to avoid division by zero (see details).}
}
\details{
  The details can be found in the given reference.
}
\value{
  a vector of values (one value per column in the input matrix).
}
\references{Statistical Algorithms Description Document, 2002, Affymetrix.}
\seealso{
  \code{\link{pmcorrect.mas}} and \code{\link{generateExprVal.method.mas}} 
}
\keyword{manip}

\name{.setAffyOptions}
\alias{.setAffyOptions}
\title{ ~~function to set options ~~ }
\description{
  ~~ Set the options for the package 
}
\usage{
.setAffyOptions(affy.opt = NA)
}
\arguments{
  \item{affy.opt}{ A list structure of options. If \code{NA}, the
    default options are set.}
}
\details{
  See the vignettes to know more. This function could disappear in favor
  of a more general one the package Biobase.
}
\value{
  The function is used for its side effect. Nothing is returned.
}
\author{ Laurent }
\examples{
affy.opt <- getOption("BioC")$affy

.setAffyOptions(affy.opt)
}
\keyword{manip}

\name{normalize.constant}
\alias{normalize.constant}
\alias{normalize.AffyBatch.constant}
\title{Scale probe intensities}
\description{
  Scale array intensities in a \code{\link[affy:AffyBatch-class]{AffyBatch}}.
}
\usage{
normalize.AffyBatch.constant(abatch, refindex=1, FUN=mean, na.rm=TRUE)
normalize.constant(x, refconstant, FUN=mean, na.rm=TRUE)
}
\arguments{
  \item{abatch}{ an instance of the \code{\link{AffyBatch-class}}.}
  \item{x}{a vector of intensities on a chip (to normalize to the reference).}
  \item{refindex}{the index of the array used as a reference.}
  \item{refconstant}{the constant used as a reference.}
  \item{FUN}{a function generating a value from the intensities on an
    array. Typically \code{mean} or \code{median}.}
  \item{na.rm}{parameter passed to the function FUN.}
}
\value{
  %A \code{\link[Biobase]{container-class}} of normalized objects.
  An \code{\link[affy:AffyBatch-class]{AffyBatch}} with an attribute "constant"
  holding the value of the factor used for scaling.
}
\author{
  L. Gautier <laurent@cbs.dtu.dk>
}
\seealso{
  \code{\link[affy:AffyBatch-class]{AffyBatch}}
}
\keyword{manip}

\name{normalize.qspline}
\alias{qspline-normalize}
\alias{normalize.qspline}
\alias{normalize.AffyBatch.qspline}
\title{Normalize arrays}
\description{
  normalizes arrays in an AffyBatch each other or to a set of target intensities
}
\usage{
normalize.AffyBatch.qspline(abatch,type=c("together", "pmonly", "mmonly",
                 "separate"), ...)

normalize.qspline(x, target = NULL, samples = NULL, 
                  fit.iters = 5, min.offset = 5, 
                  spline.method = "natural", smooth = TRUE,
                  spar = 0, p.min = 0, p.max = 1.0, 
                  incl.ends = TRUE, converge = FALSE, 
                  verbose = TRUE, na.rm = FALSE)
}
\arguments{
  \item{x}{a \code{data.matrix} of intensities}
  \item{abatch}{an \code{AffyBatch}}
  \item{target}{numerical vector of intensity values to normalize to.
                (could be the name for one of the celfiles in 'abatch').}
  \item{samples}{numerical, the number of quantiles to be used for spline.
                 if (0,1], then it is a sampling rate.}
  \item{fit.iters}{number of spline interpolations to average.}
  \item{min.offset}{minimum span between quantiles (rank difference) for the
                    different fit iterations.}
  \item{spline.method}{specifies the type of spline to be used. Possible values are 
                       `"fmm"', `"natural"', and `"periodic"'.}
  \item{smooth}{logical, if `TRUE', smoothing splines are used on the quantiles.}
  \item{spar}{smoothing parameter for `splinefun', typically in (0,1].}
  \item{p.min}{minimum percentile for the first quantile.}
  \item{p.max}{maximum percentile for the last quantile.}
  \item{incl.ends}{include the minimum and maximum values from the normalized
                   and target arrays in the fit.}
  \item{converge}{(currently unimplemented)}
  \item{verbose}{logical, if `TRUE' then normalization progress is reported.}
  \item{na.rm}{logical, if `TRUE' then handle NA values (by ignoring
    them).}
  \item{type}{a string specifying how the normalization should be
    applied. See details for more.}
  \item{\dots}{optional parameters to be passed through.}
}
\value{
  a normalized \code{AffyBatch}.
}
\details{
  This normalization method uses the quantiles from each array and the 
  target to fit a system of cubic splines to normalize the data. The 
  target should be the mean (geometric) or median of each probe but could 
  also be the name of a particular chip in the \code{abatch} object.

  Parameters setting can be of much importance when using this method.
  The parameter \code{fit.iter} is used as a starting point to find a
  more appropriate value. Unfortunately the algorithm used do not
  converge in some cases. If this happens, the \code{fit.iter} value is
  used and a warning is thrown. Use of different settings for the
  parameter \code{samples} was reported to give good results. More
  specifically, for about 200   data points use
  \code{samples = 0.33}, for about 2000  data points use
  \code{samples = 0.05}, for about 10000 data points use
  \code{samples = 0.02}
  (thanks to Paul Boutros).
  
  The \code{type} argument should be one of
  \code{"separate","pmonly","mmonly","together"} which indicates whether
  to normalize only one probe type (PM,MM) or both together or separately.
}
\author{
  Laurent and Workman C.
}
\references{
  Christopher Workman, Lars Juhl Jensen, Hanne Jarmer, Randy Berka,
     Laurent Gautier, Henrik Bjorn Nielsen, Hans-Henrik Saxild, Claus
     Nielsen, Soren Brunak, and Steen Knudsen. A new non-linear normal-
     ization method for reducing variability in dna microarray experiments.
     Genome Biology, accepted, 2002
}
\keyword{manip}

\name{MAplot}
\alias{ma.plot}
\alias{Mbox}
\alias{MAplot}
\alias{Mbox,AffyBatch-method}
\alias{MAplot,AffyBatch-method}

\title{Relative M vs. A plots}
\description{
  Create boxplots of M or M vs A plots. Where M is determined relative
  to a specified chip or to a pseudo-median reference chip.
}
\usage{
MAplot(object,...)
Mbox(object,...)
ma.plot(A, M, subset = sample(1:length(M), min(c(10000, length(M)))),
        show.statistics = TRUE, span = 2/3, family.loess = "gaussian",
        cex = 2, plot.method = c("normal","smoothScatter","add"),
        add.loess = TRUE, lwd = 1, lty = 1, loess.col = "red", ...)
}
\arguments{
  \item{object}{an \code{\link[affy]{AffyBatch-class}}.}
  \item{\dots}{additional parameters for the routine.}
  \item{A}{a vector to plot along the horizontal axis.}
  \item{M}{a vector to plot along vertical axis.}
  \item{subset}{a set of indices to use when drawing the loess curve.}
  \item{show.statistics}{logical. If TRUE, some summary statistics of the M
    values are drawn.}
  \item{span}{span to be used for loess fit.}
  \item{family.loess}{\code{"guassian"} or \code{"symmetric"} as in
    \code{\link[stats]{loess}}.}
  \item{cex}{size of text when writing summary statistics on plot.}
  \item{plot.method}{a string specifying how the plot is to be drawn.
    \code{"normal"} plots points, \code{"smoothScatter"} uses the
    \code{\link[graphics]{smoothScatter}} function. Specifying
    \code{"add"} means that the MAplot should be added to the current plot.}
\item{add.loess}{add a loess line to the plot.}
\item{lwd}{width of loess line.}
\item{lty}{line type for loess line.}
\item{loess.col}{color for loess line.}
}
\examples{
if (require(affydata)) {
   data(Dilution)
   MAplot(Dilution)
   Mbox(Dilution)
}
}
\seealso{\code{\link[affy]{mva.pairs}}}
\keyword{hplot}

\name{generateExprVal.method.playerout}
\alias{generateExprVal.method.playerout}
\alias{playerout.costfunction}
\title{Generate an expression value from the probes informations}
\description{
  Generate an expression from the probes
}
\usage{
  generateExprVal.method.playerout(probes, weights=FALSE, optim.method="L-BFGS-B")
}
\arguments{
  \item{probes}{a list of \code{probes} slots from \code{PPSet.container}}
  \item{weights}{Should the resulting weights be returned ?}
  \item{optim.method}{see parameter 'optim' for the function \code{\link{optim}}}
}
\value{
  A vector of expression values.
}
\details{
  A non-parametric method to weight each perfect match probe in the set and
  to compute a weighted mean of the perfect match values. One will notice
  this method only makes use of the perfect matches. (see function
  \code{playerout.costfunction} for the cost function).
}
\author{
  Laurent <laurent@cbs.dtu.dk>\cr
  (Thanks to E. Lazaridris for the original playerout code and the
  discussions about it)
}
\references{
  Emmanuel N. Lazaridis, Dominic Sinibaldi, Gregory Bloom, Shrikant Mane and Richard Jove
  A simple method to improve probe set estimates from oligonucleotide
  arrays, Mathematical Biosciences, Volume 176, Issue 1, March 2002, Pages 53-58
}
\keyword{manip}

\name{normalize.loess}
\alias{normalize.loess}
\alias{normalize.AffyBatch.loess}
\title{Scale microarray data}
\description{Normalizes arrays using loess.}
\usage{
normalize.loess(mat, subset = sample(1:(dim(mat)[1]), min(c(5000,
                 nrow(mat)))), epsilon = 10^-2, maxit = 1, log.it =
                 TRUE, verbose = TRUE, span = 2/3, family.loess =
                 "symmetric")
normalize.AffyBatch.loess(abatch,type=c("together","pmonly","mmonly","separate"), ...)
}
\arguments{
  \item{mat}{a matrix with columns containing the values of the chips
    to normalize.}
  \item{abatch}{an \code{\link[affy:AffyBatch-class]{AffyBatch}} object.}
  \item{subset}{a subset of the data to fit a loess to.}
  \item{epsilon}{a tolerance value (supposed to be a small value - used
    as a stopping criterion).}
  \item{maxit}{maximum number of iterations.}
  \item{log.it}{logical. If \code{TRUE} it takes the log2 of \code{mat}}
  \item{verbose}{logical. If \code{TRUE} displays current pair of chip being
    worked on.}
  \item{span}{parameter to be passed the function \code{\link[stats]{loess}}}
  \item{family.loess}{parameter to be passed the function
    \code{\link[stats]{loess}}. \code{"gaussian"} or \code{"symmetric"}
    are acceptable values for this parameter.}
  \item{type}{A string specifying how the normalization should be
    applied. See details for more.}
  \item{\dots}{any of the options of normalize.loess you would like to
    modify (described above).}
}
\details{
  The type argument should be one of
  \code{"separate","pmonly","mmonly","together"} which indicates whether
  to normalize only one probe type (PM,MM) or both together or separately.
}
\seealso{
  \code{\link{normalize}}
}
\examples{
if (require(affydata)) {
  #data(Dilution)
  #x <- pm(Dilution[,1:3])
  #mva.pairs(x)
  #x <- normalize.loess(x,subset=1:nrow(x))
  #mva.pairs(x)
}
}
\keyword{smooth}

\name{cdfenv.example}
\alias{cdfenv.example}
\title{Example cdfenv}
\description{
  Example cdfenv (environment containing the probe locations).
}
\usage{data(cdfenv.example)}
\format{ An
  \code{\link{environment}} \code{cdfenv.example} containing the probe
  locations}
\source{Affymetrix CDF file for the array Hu6800}
\keyword{datasets}

\name{normalize-methods}
\title{Normalize Affymetrix Probe Level Data - methods}
\docType{methods}
\alias{normalize.AffyBatch}
\alias{normalize.methods}
\alias{normalize.AffyBatch.methods}
\alias{upDate.normalize.AffyBatch.methods}
\alias{normalize,AffyBatch-method}
\alias{normalize.methods,AffyBatch-method}
\alias{normalize.AffyBatch.methods}
\alias{bgcorrect.methods}
\alias{upDate.bgcorrect.methods}
\alias{pmcorrect.methods}
\alias{upDate.pmcorrect.methods}
\description{
  Method for normalizing Affymetrix Probe Level Data
}
\usage{
normalize.methods(object)
bgcorrect.methods()
upDate.bgcorrect.methods(x)
pmcorrect.methods()
upDate.pmcorrect.methods(x)
}
\arguments{
  \item{object}{An \code{\link[affy:AffyBatch-class]{AffyBatch}}.}
  \item{x}{A character vector that will replace the existing one.}
}
\details{
  If \code{object} is an
  \code{\link[affy:AffyBatch-class]{AffyBatch}} object, then
  \code{normalize(object)} returns an
  \code{\link[affy:AffyBatch-class]{AffyBatch}} object with the
  intensities normalized using the methodology specified by
  \code{getOption("BioC")$affy$normalize.method}. The affy package
  default is \code{quantiles}.

  Other methodologies can be used by specifying them with the
  \code{method} argument. For example to use the invariant set
  methodology described by Li and Wong (2001) one would type:
  \code{normalize(object, method="invariantset")}.

  Further arguments passed by \code{...}, apart from \code{method},  are
  passed along to the function responsible for the methodology defined by
  the \code{method} argument. 

  A character vector of \emph{nicknames} for the methodologies available
  is returned by \code{normalize.methods(object))}, where \code{object} 
  is an  \code{\link[affy:AffyBatch-class]{AffyBatch}}, or simply by
  typing \code{normalize.AffyBatch.methods}. If the nickname of a method
  is called "loess", the help page for that specific methodology can
  be accessed by typing \code{?normalize.loess}. 

  For more on the normalization methodologies currently implemented
  please refer to the vignette `Custom Processing Methods'.

  To add your own normalization procedures please refer to the
  customMethods vignette.

  The functions: \code{bgcorrect.methods}, \code{pmcorrect.methods},
  provide access to internal vectors listing the corresponding capabilities.
}
\seealso{
  \code{\link{AffyBatch-class}}, \code{\link{normalize}}.
}
\examples{
if (require(affydata)) {
  data(Dilution)
  normalize.methods(Dilution)
  generateExprSet.methods()
  bgcorrect.methods()
  pmcorrect.methods()
}
}
\keyword{manip}

\name{pmcorrect}
\alias{pmcorrect}
\alias{pmcorrect.pmonly} %took out .pmonly casue rma is pm-only
\alias{pmcorrect.mas}
\alias{pmcorrect.subtractmm}
\title{PM Correction}
\description{
  Corrects the PM intensities in a \code{\link[affy:ProbeSet-class]{ProbeSet}}
  for non-specific binding.
}
\usage{
pmcorrect.pmonly(object)

pmcorrect.subtractmm(object)

pmcorrect.mas(object, contrast.tau=0.03, scale.tau=10, delta=2^(-20))
}
\arguments{
  \item{object}{An object of class \code{\link[affy:ProbeSet-class]{ProbeSet}}.}
  \item{contrast.tau}{a number denoting the contrast tau parameter in
    the MAS 5.0 pm correction algorithm.}
  \item{scale.tau}{a number denoting the scale tau parameter in
    the MAS 5.0 pm correction algorithm.}
  \item{delta}{a number denoting the delta parameter in
    the MAS 5.0 pm correction algorithm.}
}
\details{
  These are the pm correction methods perfromed by Affymetrix MAS 4.0
  (subtractmm) and MAS 5.0 (mas). See the Affymetrix Manual for details.
  pmonly does what you think: does not change the PM values.}
  \value{A \code{\link[affy:ProbeSet-class]{ProbeSet}} for which the
  \code{pm} slot contains the  corrected  PM values.}
\references{Affymetrix MAS 4.0 and 5.0 manual}
\examples{
if (require(affydata)) {
  data(Dilution)
  gn <- geneNames(Dilution)
  pps <- probeset(Dilution, gn[1])[[1]]

  pps.pmonly <- pmcorrect.pmonly(pps)
  pps.subtractmm <- pmcorrect.subtractmm(pps)
  pps.mas5 <- pmcorrect.mas(pps)
}
}
\keyword{manip}

\name{ProgressBarText-class}
\docType{class}
\alias{ProgressBarText-class}
\alias{close,ProgressBarText-method}
\alias{initialize,ProgressBarText-method}
\alias{open,ProgressBarText-method}
\alias{updateMe}
\alias{updateMe,ProgressBarText-method}
\title{Class "ProgressBarText" }
\description{A class to handle progress bars in text mode.}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("ProgressBarText", steps)}.
}
\section{Slots}{
  \describe{
    \item{\code{steps}:}{Object of class \code{"integer"}. The total number of
      steps the progress bar should represent.}
    \item{\code{barsteps}:}{Object of class \code{"integer"}. The size of the
      progress bar.}
    \item{\code{internals}:}{Object of class \code{"environment"}.
      For internal use.}
  }
}
\section{Methods}{
  \describe{
    \item{close}{\code{signature(con = "ProgressBarText")}: Terminate
      the progress bar (i.e. print what needs to be printed). Note that
      closing the instance will ensure the progress bar is plotted to
      its end.}
    \item{initialize}{\code{signature(.Object = "ProgressBarText")}:
      initialize a instance.}
    \item{open}{\code{signature(con = "ProgressBarText")}: Open a
      progress bar (i.e. print things). In the case open is called on
      a progress bar that was 'progress', the progress bar is resumed
      (this might be useful when one wishes to insert text output while
      there is a progress bar running).}
    \item{updateMe}{\code{signature(object = "ProgressBarText")}: Update
      the progress bar (see examples).}
  }
}
\author{ Laurent }
\examples{
f <- function(x, header = TRUE) {
  pbt <- new("ProgressBarText", length(x), barsteps = as.integer(20))

  open(pbt, header = header)

  for (i in x) {
    Sys.sleep(i)
    updateMe(pbt)
  }
  close(pbt)
}

## if too fast on your machine, change the number
x <- runif(15)

f(x)
f(x, header = FALSE)

## 'cost' of the progress bar:
g <- function(x) {
  z <- 1
  for (i in 1:x) {
    z <- z + 1
  }
}
h <- function(x) {
  pbt <- new("ProgressBarText", as.integer(x), barsteps = as.integer(20))
  open(pbt)
  for (i in 1:x) {
    updateMe(pbt)
  }
  close(pbt)
}

system.time(g(10000))
system.time(h(10000))

}
\keyword{classes}

\name{hlog}
\alias{hlog}
\title{Hybrid Log}
\description{Given a constant \code{c} this function returns
\code{x} if \code{x} is less than \code{c} and \code{sign(x)*(c*log(abs(x)/c)
  + c)} if its not. Notice this is a continuous odd ( f(-x)=-f(x) )
function with continuous first derivative. The main purpose is to perform log
transformation when one has negative numbers, for example for PM-MM.}
\usage{
hlog(x, constant=1)
}
\arguments{
  \item{x}{a number.}
  \item{constant}{the constant c (see description).}
  }
\details{
If \code{constant} is less than or equal to 0 \code{log(x)} is
returned for all \code{x}. If \code{constant} is infinity \code{x} is
returned for all \code{x}. 
}
\author{Rafael A. Irizarry}
\keyword{math}

\name{summary}
\alias{summary}
\alias{medianpolish}
\alias{tukeybiweight}
\alias{avdiff}
\title{Probe Set Summarizing Functions}
\description{
  These were used with the function \code{express}, which is no longer part
  of the package. Some are still used by the generateExprVal functions, but
  you should avoid using them directly.
}  
\seealso{\code{\link[affy]{expresso}}}
\keyword{manip}

\name{maffy.subset}
\alias{maffy.subset}
\title{Select Subset}
\description{Select a subset of rows with small rank-range over columns.}
\usage{
maffy.subset(data,subset.size=5000,maxit=100,
             subset.delta=max(round(subset.size/100),25),verbose=FALSE)}
\arguments{
  \item{data}{a matrix}
  \item{subset.size}{desired size of subset}
  \item{maxit}{maximum number of iterations}
  \item{subset.delta}{maximum deviation from subset.size}
  \item{verbose}{logical value.}
}
\details{
Please refer to references.
}
\value{ 
  A list with component \code{subset}, the indexes for subset.
}
\references{Astrand, M. (2001) \url{http://www.math.chalmers.se/~magnusaa/maffy/}}
\author{Magnus Astrand}
\seealso{ \code{\link{maffy.normalize}}}
\examples{
if (require(affydata)) {
  #data(Dilution)
  #x <- log2(pm(Dilution)[,1:3])
  #Index <- maffy.subset(x,subset.size=100)$subset
  #mva.pairs(x[Index,])
}
}
\keyword{internal}

\name{normalize.invariantset}
\alias{normalize.invariantset}
\alias{normalize.AffyBatch.invariantset}
\title{Invariant Set normalization}
\description{
Normalize arrays in an  \code{\link[affy:AffyBatch-class]{AffyBatch}} using an
invariant set.
}
\usage{
normalize.AffyBatch.invariantset(abatch, prd.td = c(0.003, 0.007),
                                 verbose = FALSE,
                                 baseline.type = c("mean","median","pseudo-mean","pseudo-median"),
                                 type = c("separate","pmonly","mmonly","together"))

normalize.invariantset(data, ref, prd.td=c(0.003,0.007))
}
\arguments{
  \item{abatch}{an \code{\link[affy:AffyBatch-class]{AffyBatch}} object.}
  \item{data}{a vector of intensities on a chip (to normalize to the reference).}
  \item{ref}{a vector of reference intensities.}
  \item{prd.td}{cutoff parameter (details in the bibliographic
    reference).}
  \item{baseline.type}{specifies how to determine the baseline array.}
  \item{type}{a string specifying how the normalization should be
    applied. See details for more.}
  \item{verbose}{logical indicating printing throughout the normalization.}
}
\value{
  Respectively a \code{\link[affy:AffyBatch-class]{AffyBatch}} of normalized
  objects, or a vector of normalized intensities, with an attribute
  "invariant.set" holding the indexes of the 'invariant' intensities.
}
\details{
  The set of invariant intensities between \code{data} and \code{ref} is
  found through an iterative process (based on the respective ranks the
  intensities). This set of intensities is used to generate a normalization
  curve by smoothing.

  The \code{type} argument should be one of
  \code{"separate","pmonly","mmonly","together"} which indicates whether
  to normalize only one probe type (PM,MM) or both together or separately.
}
\author{
  L. Gautier <laurent@cbs.dtu.dk>
  (Thanks to Cheng Li for the discussions about the algorithm.)
}
\references{
  Cheng Li and Wing Hung Wong, Model-based analysis of
  oligonucleotides arrays: model validation, design issues and
  standard error application. Genome Biology 2001, 2(8):research0032.1-0032.11
}
\seealso{
  \code{\link{normalize}} to normalize \code{\link[affy:AffyBatch-class]{AffyBatch}} objects.
}
\keyword{manip}

\name{expresso}
\alias{expresso}
\alias{bgcorrect}
\title{ From raw probe intensities to expression values }
\description{
  Goes from raw probe intensities to expression values
}
\usage{
expresso(
        afbatch,
    # background correction
        bg.correct = TRUE,
        bgcorrect.method = NULL,
        bgcorrect.param = list(),
    # normalize
        normalize = TRUE,
        normalize.method = NULL,
        normalize.param = list(),
    # pm correction
        pmcorrect.method = NULL,
        pmcorrect.param = list(),
    # expression values
        summary.method = NULL,
        summary.param = list(),
        summary.subset = NULL,
    # misc.
        verbose = TRUE,
%        warnings = TRUE,
        widget = FALSE)
}
\arguments{
  \item{afbatch}{an \code{\link[affy:AffyBatch-class]{AffyBatch}} object.}
  \item{bg.correct}{a boolean to express whether background correction
    is wanted or not.}
  \item{bgcorrect.method}{the name of the background adjustment method.}
  \item{bgcorrect.param}{a list of parameters for bgcorrect.method
    (if needed/wanted).}
  \item{normalize}{ normalization step wished or not.}
  \item{normalize.method}{the normalization method to use.}
  \item{normalize.param}{a list of parameters to be passed to the
    normalization method (if wanted).}
  \item{pmcorrect.method}{the name of the PM adjustment method.}
  \item{pmcorrect.param}{a list of parameters for pmcorrect.method
    (if needed/wanted).}
  \item{summary.method}{the method used for the computation of expression
    values.}
  \item{summary.param}{a list of parameters to be passed to the
    \code{summary.method} (if wanted).}
  \item{summary.subset}{a list of 'affyids'. If \code{NULL}, an
    expression summary value is computed for everything on the chip.}
  \item{verbose}{logical value. If \code{TRUE}, it writes out some
    messages.}
%  \item{warnings}{warning when something goes wrong}
  \item{widget}{a boolean to specify the use of widgets (the package
    tkWidget is required).}
}
\details{
  Some arguments can be left to \code{NULL} if the \code{widget=TRUE}.
  In this case, a widget pops up and let the user choose with the mouse.
  The arguments are: \code{AffyBatch}, \code{bgcorrect.method},
  \code{normalize.method}, \code{pmcorrect.method} and \code{summary.method}.

  For the mas 5.0 and 4.0 methods ones need to normalize after obtaining
  expression. The function \code{\link{affy.scalevalue.exprSet}} does this.

  For the Li and Wong summary method notice you will not get
  the same results as you would get with dChip. dChip is not open source
  so it is not easy to reproduce. 
  Notice also that this iterative algorithm will not always converge.
  If you run the algorithm on thousands of probes expect some non-convergence
  warnings. These are more likely when few arrays are used. We recommend
  using this method only if you have 10 or more arrays.
  Please refer to the \code{\link{fit.li.wong}} help page for more details.
}
\value{
  An object of class \code{\link[Biobase:class.ExpressionSet]{ExpressionSet}},
  with an attribute \code{pps.warnings} as returned by the method
  \code{\link{computeExprSet}}.
}
\seealso{\code{\link[affy:AffyBatch-class]{AffyBatch}}}
\examples{
if (require(affydata)) {
  data(Dilution)

  eset <- expresso(Dilution, bgcorrect.method="rma",
                   normalize.method="constant",pmcorrect.method="pmonly",
                   summary.method="avgdiff")

  ##to see options available for bg correction type:
  bgcorrect.methods()
}
}
\keyword{manip}

\name{read.probematrix}
\alias{read.probematrix}
\title{Read CEL file data into PM or MM matrices}
\description{
  Read CEL data into matrices.
}
\usage{
read.probematrix(..., filenames = character(0),
                 phenoData = new("AnnotatedDataFrame"),
                 description = NULL,
                 notes = "",
                 compress = getOption("BioC")$affy$compress.cel,
                 rm.mask = FALSE, rm.outliers = FALSE, rm.extra = FALSE,
                 verbose = FALSE, which = "pm", cdfname = NULL)
}
\arguments{
  \item{\dots}{file names separated by comma.}
  \item{filenames}{file names in a character vector.}
  \item{phenoData}{a \code{\link[Biobase:class.AnnotatedDataFrame]{AnnotatedDataFrame}}
    object.}
  \item{description}{a \code{\link[Biobase:class.MIAME]{MIAME}} object.}
  \item{notes}{notes.}
  \item{compress}{are the CEL files compressed?}
  \item{rm.mask}{should the spots marked as 'MASKS' set to \code{NA}?}
  \item{rm.outliers}{should the spots marked as 'OUTLIERS' set to \code{NA}?}
  \item{rm.extra}{if \code{TRUE}, overrides what is in \code{rm.mask} and
    \code{rm.oultiers}.}
  \item{verbose}{verbosity flag.}
  \item{which}{should be either "pm", "mm" or "both".}
  \item{cdfname}{Used to specify the name of an alternative cdf package.
    If set to \code{NULL}, the usual cdf package based on Affymetrix's
    mappings will be used.}
}
\value{
  A list of one or two matrices. Each matrix is either PM or MM data. No
  \code{\link[affy:AffyBatch-class]{AffyBatch}} is created.
}
\author{Ben Bolstad \email{bmb@bmbolstad.com}}
\seealso{\code{\link[affy:AffyBatch-class]{AffyBatch}},
  \code{\link[affy:read.affybatch]{read.affybatch}}}
\keyword{manip}

\name{read.affybatch}
\alias{read.affybatch}
\alias{AllButCelsForReadAffy}
\alias{ReadAffy}
\title{Read CEL files into an AffyBatch}
\description{
  Read CEL files into an Affybatch.
}
\usage{
read.affybatch(\dots, filenames = character(0),
               phenoData = new("AnnotatedDataFrame"),
               description = NULL,
               notes = "",
               compress = getOption("BioC")$affy$compress.cel,
               rm.mask = FALSE, rm.outliers = FALSE, rm.extra = FALSE,
               verbose = FALSE,sd=FALSE, cdfname = NULL)

ReadAffy(\dots, filenames=character(0),
              widget=getOption("BioC")$affy$use.widgets,
              compress=getOption("BioC")$affy$compress.cel,
              celfile.path=NULL,
              sampleNames=NULL,
              phenoData=NULL,
              description=NULL,
              notes="",
              rm.mask=FALSE, rm.outliers=FALSE, rm.extra=FALSE,
              verbose=FALSE,sd=FALSE, cdfname = NULL)
}
\arguments{
  \item{\dots}{file names separated by comma.}
  \item{filenames}{file names in a character vector.}
  \item{phenoData}{an \code{\link[Biobase:class.AnnotatedDataFrame]{AnnotatedDataFrame}}
     object, a \code{character} of length one, or a \code{data.frame}.}
  \item{description}{a \code{\link[Biobase:class.MIAME]{MIAME}} object.}
  \item{notes}{notes.}
  \item{compress}{are the CEL files compressed?}
  \item{rm.mask}{should the spots marked as 'MASKS' set to \code{NA}?}
  \item{rm.outliers}{should the spots marked as 'OUTLIERS' set to \code{NA}?}
  \item{rm.extra}{if \code{TRUE}, then overrides what is in \code{rm.mask} and
    \code{rm.oultiers}.}
  \item{verbose}{verbosity flag.}
  \item{widget}{a logical specifying if widgets should be used.}
  \item{celfile.path}{a character denoting the path \code{ReadAffy}
    should look for cel files.}
  \item{sampleNames}{a character vector of sample names to be used in
    the \code{AffyBatch}.}
  \item{sd}{should the standard deviation values in the CEL file be read
    in? Since these are typically not used default is not to read them
    in. This also save lots of memory.}
   \item{cdfname}{used to specify the name of an alternative cdf package.
     If set to \code{NULL}, then the usual cdf package based on Affymetrix's
     mappings will be used.}
}
\details{
\code{ReadAffy} is a wrapper for \code{read.affybatch} that permits the
user to read in phenoData, MIAME information, and CEL files using
widgets. One can also define files where to read phenoData and MIAME
information.

If the function is called with no arguments \code{ReadAffy()} all the CEL
files in the working directory are read and put into an \code{AffyBatch}.
However, the arguments give the user great flexibility.
  
If \code{phenoData} is a character vector of length 1, the function
\code{\link[Biobase]{read.AnnotatedDataFrame}} is called to read a file
of that name and produce the \code{AnnotationDataFrame} object with the
sample metadata. If \code{phenoData} is a \code{data.frame}, it is
converted to an \code{AnnotatedDataFrame}.
If it is \code{NULL} and \code{widget=FALSE} (\code{widget=TRUE} is not currently
supported), then a default object of class
\code{\link[Biobase:class.AnnotatedDataFrame]{AnnotatedDataFrame}} is created,
whose \code{pData} is a data.frame with rownames being the names
of the CEL files, and with one column \code{sample} with an integer index.

\code{AllButCelsForReadAffy} is an internal function that gets called
by \code{ReadAffy}. It gets all the information except the cel intensities.

\code{description} is read using \code{\link[Biobase]{read.MIAME}}. If a
character is given, then it tries to read the file with that name to obtain a
\code{MIAME} instance. If left \code{NULL} but \code{widget=TRUE}, then
widgets are used. If left \code{NULL} and \code{widget=FALSE}, then an
empty instance of \code{MIAME} is created.
}
\value{
  An \code{AffyBatch} object.
}
\author{Ben Bolstad \email{bmb@bmbolstad.com} (read.affybatch),
  Laurent Gautier, and Rafael A. Irizarry (ReadAffy)}
\seealso{\code{\link[affy:AffyBatch-class]{AffyBatch}}}
\examples{
if(require(affydata)){
     celpath <- system.file("celfiles", package="affydata")
     fns <- list.celfiles(path=celpath,full.names=TRUE)
 
     cat("Reading files:\n",paste(fns,collapse="\n"),"\n")
     ##read a binary celfile
     abatch <- ReadAffy(filenames=fns[1])
     ##read a text celfile
     abatch <- ReadAffy(filenames=fns[2])
     ##read all files in that dir
     abatch <- ReadAffy(celfile.path=celpath)
}
}
\keyword{manip}

\name{mas5}
\alias{mas5}
\title{MAS 5.0 expression measure}
\description{
  This function converts an instance of \code{\link{AffyBatch}}
  into an instance of \code{\link[Biobase:class.ExpressionSet]{ExpressionSet}} using
  our implementation of Affymetrix's MAS 5.0 expression measure.
}
\usage{
mas5(object, normalize = TRUE, sc = 500, analysis = "absolute", ...)
}
\arguments{
  \item{object}{an instance of \code{\link{AffyBatch}}}
  \item{normalize}{logical. If \code{TRUE} scale normalization is used
    after we obtain an instance of \code{\link[Biobase:class.ExpressionSet]{ExpressionSet}}}
  \item{sc}{Value at which all arrays will be scaled to.}
  \item{analysis}{should we do absolute or comparison analysis, although
    "comparison" is still not implemented.}
  \item{\dots}{other arguments to be passed to \code{\link{expresso}}.}
}
\details{
  This function is a wrapper for \code{\link{expresso}} and
  \code{\link{affy.scalevalue.exprSet}}.}
\value{
  \code{\link[Biobase:class.ExpressionSet]{ExpressionSet}} 

  The methods used by this function were implemented based upon available
  documentation. In particular a useful reference is Statistical Algorithms
  Description Document by Affymetrix.
  Our implementation is based on what is written in the documentation and, as
  you might appreciate, there are places where the documentation is less than
  clear. This function does not give exactly the same results.
  All source code of our implementation is available. You are free to
  read it and suggest fixes. 

  For more information visit this URL:
  \url{http://stat-www.berkeley.edu/users/bolstad/}
}
\seealso{\code{\link{expresso}},\code{\link{affy.scalevalue.exprSet}}}
\examples{
if (require(affydata)) {
  data(Dilution)
  eset <- mas5(Dilution)
}
}
\keyword{manip}

\name{normalize.quantiles}
\alias{normalize.AffyBatch.quantiles}
\title{Quantile Normalization}
\description{
  Using a normalization based upon quantiles, this function
  normalizes a matrix of probe level intensities.
}
\usage{
  normalize.AffyBatch.quantiles(abatch, type=c("separate","pmonly","mmonly","together"))
}
\arguments{
  \item{abatch}{an \code{\link[affy:AffyBatch-class]{AffyBatch}} object.}
  \item{type}{A string specifying how the normalization should be
    applied. See details for more.}
}
\details{This method is based upon the concept of a quantile-quantile
  plot extended to n dimensions. No special allowances are made for
  outliers. If you make use of quantile normalization either through
  \code{\link{rma}} or \code{\link{expresso}}
  please cite Bolstad et al, Bioinformatics (2003).

  The type argument should be one of
  \code{"separate","pmonly","mmonly","together"} which indicates whether
  to normalize only one probe type (PM,MM) or both together or separately. 
}

\value{
  A normalized \code{AffyBatch}.
}
\references{
  Bolstad, B (2001) \emph{Probe Level Quantile Normalization of High Density
    Oligonucleotide Array Data}. Unpublished manuscript
  \url{http://bmbolstad.com/stuff/qnorm.pdf}

  Bolstad, B. M., Irizarry R. A., Astrand, M, and Speed, T. P. (2003)
  \emph{A Comparison of Normalization Methods for High Density
    Oligonucleotide Array Data Based on Bias and Variance.}
   Bioinformatics 19(2) ,pp 185-193. \url{http://bmbolstad.com/misc/normalize/normalize.html}
}
\author{Ben Bolstad, \email{bmb@bmbolstad.com}}
\seealso{\code{\link{normalize}}}
\keyword{manip}

\name{generateExprVal.method.avgdiff}
\alias{generateExprVal.method.avgdiff}
\alias{generateExprVal.method.medianpolish}
\alias{generateExprVal.method.liwong}
\alias{generateExprVal.method.mas}

\title{Generate an expression value from the probes informations}
\description{
  Generate an expression from the probes
}
\usage{
generateExprVal.method.avgdiff(probes, ...)
generateExprVal.method.medianpolish(probes, ...)
generateExprVal.method.liwong(probes, ...)
generateExprVal.method.mas(probes, ...)
}
\arguments{
  \item{probes}{a matrix of probe intensities with rows representing
    probes and columns representing samples. Usually \code{pm(probeset)}
    where \code{probeset} is a of class
    \code{\link[affy:ProbeSet-class]{ProbeSet}}.}
  \item{\dots}{extra arguments to pass to the respective function.}
}
\value{
  A list containing entries:
  \item{exprs}{The expression values.}
  \item{se.exprs}{The standard error estimate.}
}
\examples{
  data(SpikeIn) ##SpikeIn is a ProbeSets
  probes <- pm(SpikeIn) 
  avgdiff <- generateExprVal.method.avgdiff(probes)
  medianpolish <- generateExprVal.method.medianpolish(probes)
  liwong <- generateExprVal.method.liwong(probes)
  playerout <- generateExprVal.method.playerout(probes)
  mas <- generateExprVal.method.mas(probes)
  
  concentrations <- as.numeric(sampleNames(SpikeIn))
  plot(concentrations,avgdiff$exprs,log="xy",ylim=c(50,10000),pch="a",type="b")
  points(concentrations,2^medianpolish$exprs,pch="m",col=2,type="b",lty=2)
  points(concentrations,liwong$exprs,pch="l",col=3,type="b",lty=3)
  points(concentrations,playerout$exprs,pch="p",col=4,type="b",lty=4)
  points(concentrations,mas$exprs,pch="p",col=4,type="b",lty=4)
}
\seealso{
  \code{\link[affy]{generateExprSet-methods}},
    \code{\link[affy]{generateExprVal.method.playerout}},
    \code{\link[affy]{fit.li.wong}}
  
}

\keyword{manip}

\name{normalize.quantiles.robust}
\alias{normalize.AffyBatch.quantiles.robust}
\title{Robust Quantile Normalization}
\description{Using a normalization based upon quantiles, this function
  normalizes a matrix of probe level intensities. Allows weighting of chips}
\usage{
    normalize.AffyBatch.quantiles.robust(abatch,
                                         type = c("separate","pmonly","mmonly","together"),
                                         weights = NULL,
                                         remove.extreme = c("variance","mean","both","none"),
                                         n.remove = 1, use.median = FALSE,
                                         use.log2 = FALSE)
}
\arguments{
  \item{abatch}{an \code{\link[affy:AffyBatch-class]{AffyBatch}} object.}
  \item{type}{a string specifying how the normalization should be applied.
    See details for more.}
  \item{weights}{a vector of weights, one for each chip.}
  \item{remove.extreme}{if weights is NULL, then this will be used for
    determining which chips to remove from the calculation of the
    normalization distribution. See details for more info.}
  \item{n.remove}{number of chips to remove.}
  \item{use.median}{if TRUE, the use the median to compute normalization
    chip; otherwise uses a weighted mean.}
  \item{use.log2}{work on log2 scale. This means we will be using the
    geometric mean rather than ordinary mean.}
}
\details{This method is based upon the concept of a quantile-quantile 
  plot extended to n dimensions. Note that the matrix is of intensities
  not log intensities. The function performs better with raw
  intensities.
  
  Choosing \bold{variance} will remove chips with variances much higher
  or lower than the other chips, \bold{mean} removes chips with the mean
  most different from all the other means, \bold{both} removes first
  extreme variance and then an extreme mean. The option \bold{none} does
  not remove any chips, but will assign equal weights to all chips.
 
  The type argument should be one of
  \code{"separate","pmonly","mmonly","together"} which indicates whether
  to normalize only one probe type (PM,MM) or both together or separately.
 }
\note{This function is still experimental.}
\value{a matrix of normalized intensities}
\author{Ben Bolstad, \email{bmb@bmbolstad.com}}
\seealso{\code{\link{normalize}}, \code{\link[preprocessCore:normalize.quantiles]{normalize.quantiles}}}
\keyword{manip}

\name{probeMatch-methods}
\docType{methods}
\title{Methods for accessing perfect matches and mismatches}
\alias{probeMatch-methods}
\alias{probeMatch}
\alias{pm}
\alias{pm<-}
\alias{mm}
\alias{mm<-}
\description{Methods for perfect matches and mismatches probes}
\section{Methods}{\describe{
    \item{object = AffyBatch}{All the \emph{perfect match} (pm) or
      \emph{mismatch} (mm) probes on the arrays the object represents
      are returned.}
    \item{object = ProbeSet}{The \code{pm} or \code{mm} of the object
      are returned.}
  }
}
\keyword{methods}

\name{generateExprVal}
\alias{express.summary.stat}
\alias{express.summary.stat-methods}
\alias{express.summary.stat.methods}
\alias{upDate.express.summary.stat.methods}
\title{Compute a summary expression value from the probes intensities}
\description{
  Compute a summary expression value from the probes intensities
}
\usage{
express.summary.stat(x, pmcorrect, summary, ...)
express.summary.stat.methods() # vector of names of methods
upDate.express.summary.stat.methods(x)
}
\arguments{
  \item{x}{a (\code{ProbeSet}}
  \item{pmcorrect}{the method used to correct the PM values before
    summarizing to an expression value.}
  \item{summary}{the method used to generate the expression value.}
  \item{\dots}{other parameters the method might need... (see the
    corresponding methods below...)}
}
\value{
  Returns a vector of expression values.
}
\examples{
if (require(affydata)) {
  data(Dilution)

  p <- probeset(Dilution, "1001_at")[[1]]

  par(mfcol=c(5,2))
  mymethods <- express.summary.stat.methods()
  nmet <- length(mymethods)
  nc <- ncol(pm(p))

  layout(matrix(c(1:nc, rep(nc+1, nc)), nc, 2), width = c(1, 1))

  barplot(p)

  results <- matrix(0, nc, nmet)
  rownames(results) <- paste("sample", 1:nc)
  colnames(results) <- mymethods

  for (i in 1:nmet) {
    ev <- express.summary.stat(p, summary=mymethods[i], pmcorrect="pmonly")
    if (mymethods[[i]] != "medianpolish")
      results[, i] <- 2^(ev$exprs)
    else
      results[, i] <- ev$exprs
  }

  dotchart(results, labels=paste("sample", 1:nc))
}
}
\keyword{manip}

\name{merge.AffyBatch}
\alias{merge.AffyBatch}
\title{merge two AffyBatch objects}
\description{
  merge two AffyBatch objects into one.
}
\usage{
\method{merge}{AffyBatch}(x, y, annotation = paste(annotation(x),
                 annotation(y)), description = NULL, notes =
                 character(0), ...)
}
\arguments{
  \item{x}{an \code{AffyBatch} object.}
  \item{y}{an \code{AffyBatch} object.}
  \item{annotation}{a \code{character} vector.}
  \item{description}{a \code{characterORmiame}, eventually \code{NULL}.}
  \item{notes}{a \code{character} vector.}
  \item{\dots}{additional arguments.}
}
\details{
  To be done.
}
\value{
  A object if class \code{\link[affy:AffyBatch-class]{AffyBatch}}.
}
\seealso{\code{\link{AffyBatch-class}}}
\keyword{manip}

\name{AffyBatch-class}
\docType{class}
\alias{AffyBatch-class}
\alias{AffyBatch,ANY}
\alias{AffyBatch}
\alias{probes}
\alias{geneNames}
\alias{geneNames<-}
\alias{getCdfInfo}
\alias{image}
\alias{indexProbes}
\alias{intensity<-}
\alias{intensity}
\alias{pmindex}
\alias{mmindex}
\alias{probeset}
\alias{$.AffyBatch}
\alias{cdfName}
\alias{cdfName,AffyBatch-method}
\alias{checkValidFilenames}
\alias{probes,AffyBatch-method}
\alias{exprs,AffyBatch-method}
\alias{exprs<-,AffyBatch,ANY-method}
\alias{se.exprs,AffyBatch-method}
\alias{se.exprs<-,AffyBatch-method}
\alias{featureNames,AffyBatch-method}
\alias{featureNames<-,AffyBatch-method}
\alias{geneNames,AffyBatch-method}
\alias{geneNames<-,AffyBatch,ANY-method}
\alias{getCdfInfo,AffyBatch-method}
\alias{image,AffyBatch-method}
\alias{initialize,AffyBatch-method}
\alias{indexProbes,AffyBatch-method}
\alias{intensity<-,AffyBatch-method}
\alias{intensity,AffyBatch-method}
\alias{pmindex,AffyBatch-method}
\alias{mmindex,AffyBatch-method}
\alias{probeset,AffyBatch-method}
\alias{boxplot,AffyBatch-method}
\alias{dim,AffyBatch-method}
\alias{row,AffyBatch-method}
\alias{col,AffyBatch-method}
\alias{show,AffyBatch-method}
\alias{pm,AffyBatch-method}
\alias{pm<-,AffyBatch,ANY-method}
\alias{mm,AffyBatch-method}
\alias{mm<-,AffyBatch,ANY-method}
\alias{probeNames,AffyBatch-method}
\alias{hist,AffyBatch-method}
\alias{[<-,AffyBatch-method}
\alias{[,AffyBatch-method}
\alias{[[,AffyBatch-method}
\alias{length,AffyBatch-method}
\alias{bg.correct,AffyBatch,character-method}
\alias{indexProbes,AffyBatch,character-method}
\alias{indexProbes,AffyBatch,missing-method}
\alias{computeExprSet,AffyBatch,character,character-method}
\alias{cdfName,AffyBatch-method}
\alias{updateObject,AffyBatch-method}
\title{Class AffyBatch}
\description{This is a class representation for Affymetrix GeneChip probe
  level data. The main component are the intensities from multiple arrays
  of the same \code{CDF} type. It extends
  \code{\link[Biobase:class.eSet]{eSet}}.}
\section{Objects from the Class}{
  Objects can be created using the function \code{\link{read.affybatch}}
  or the wrapper \code{\link{ReadAffy}}.
}
\section{Slots}{
  \describe{
    \item{\code{cdfName}:}{Object of class \code{character} representing
      the name of \code{CDF} file associated with the arrays in the
      \code{AffyBatch}.}
    \item{\code{nrow}:}{Object of class \code{integer} representing the
      physical number of rows in the arrays.}
    \item{\code{ncol}:}{Object of class \code{integer} representing the
      physical number of columns in the arrays.}
    \item{\code{assayData}:}{Object of class \code{AssayData}
      containing the raw data, which will be at minimum a matrix of
      intensity values. This slot can also hold a matrix of standard
      errors if the 'sd' argument is set to \code{TRUE} in the call to
      \code{ReadAffy}.} 
    \item{\code{phenoData}:}{Object of class \code{AnnotatedDataFrame}
      containing phenotypic data for the samples.}
    \item{\code{annotation}}{A character string identifying the
      annotation that may be used for the \code{ExpressionSet}
      instance.}
    \item{\code{protocolData}:}{Object of class \code{AnnotatedDataFrame}
      containing protocol data for the samples.}
    \item{\code{featureData}}{Object of class \code{AnnotatedDataFrame}
      containing feature-level (e.g., probeset-level) information.}
    \item{\code{experimentData}:}{Object of class "MIAME" containing
      experiment-level information.} 
    \item{\code{.__classVersion__}:}{Object of class \code{Versions}
      describing the R and Biobase version number used to create the
      instance. Intended for developer use.} 
  }
}
\section{Extends}{
Class \code{"eSet"}, directly.
}
\section{Methods}{
  \describe{
    \item{cdfName}{\code{signature(object = "AffyBatch")}: obtains the
      cdfName slot.}
    \item{pm<-}{\code{signature(object = "AffyBatch")}: replaces the
      perfect match intensities.}
    \item{pm}{\code{signature(object = "AffyBatch")}: extracts the pm
      intensities.}
    \item{mm<-}{\code{signature(object = "AffyBatch")}: replaces the
      mismatch intensities.}
    \item{mm}{\code{signature(object = "AffyBatch")}: extracts the mm
      intensities.}
    \item{probes}{\code{signature(object = "AffyBatch", which)}: extract
      the perfect match or mismatch probe intensities. Uses which can be
      "pm" and "mm".}
    \item{exprs}{\code{signature(object = "AffyBatch")}: extracts the
      expression matrix.}
    \item{exprs<-}{\code{signature(object = "AffyBatch", value = "matrix")}:
      replaces the expression matrix.}
    \item{se.exprs}{\code{signature(object = "AffyBatch")}: extracts the
      matrix of standard errors of expression values, if available.}
    \item{se.exprs<-}{\code{signature(object = "AffyBatch", value = "matrix")}:
      replaces the matrix of standard errors of expression values.}
    \item{[<-}{\code{signature(x = "AffyBatch")}: replaces subsets.}
    \item{[}{\code{signature(x = "AffyBatch")}: subsets by array.}
    \item{boxplot}{\code{signature(x = "AffyBatch")}:  creates a
      \code{\link{boxplot}}s of log base 2 intensities (pm, mm or both).
      Defaults to both.}
    \item{hist}{\code{signature(x = "AffyBatch")}:  creates a plot showing
      all the histograms of the pm,mm or both data. See
      \code{\link{plotDensity}}.}
    \item{computeExprSet}{\code{signature(x = "AffyBatch",
      summary.method = "character")}: For each probe set computes an
      expression value using \code{summary.method}. }
    \item{featureNames}{\code{signature(object = "AffyBatch")}: return the
      probe set names also referred to as the Affymetrix IDs. Notice that
      one can not assign \code{featureNames}. You must do this by changing
      the cdfenvs.}
    \item{geneNames}{\code{signature(object="AffyBatch'")}: deprecated,
      use \code{featureNames}.}
    \item{getCdfInfo}{\code{signature(object = "AffyBatch")}: retrieve
      the environment that defines the location of probes by probe set.}
    \item{image}{\code{signature(x = "AffyBatch")}: creates an image for
      each sample.}
    \item{indexProbes}{
      \code{signature(object = "AffyBatch", which = "character")}:
      returns a  list with locations of the probes in each probe set. The
      affyID corresponding to the probe set to retrieve can be specified in
      an optional parameter \code{genenames}. By default, all the affyIDs
      are retrieved. The names of the elements in the list returned are the
      affyIDs. \code{which} can be "pm", "mm", or "both". If "both" then
      perfect match locations are given followed by mismatch locations.

      \code{signature(object = "AffyBatch", which = "missing")} (i.e.,
      calling \code{indexProbes} without a "which" argument) is the
      same as setting "which" to "pm".}
    \item{intensity<-}{\code{signature(object = "AffyBatch")}: a
      replacement method for the \code{exprs} slot, i.e. the intensities.}
    \item{intensity}{\code{signature(object = "AffyBatch")}: extract the
      \code{exprs} slot, i.e. the intensities.}
    \item{length}{\code{signature(x = "AffyBatch")}: returns the number
      of samples.}
    \item{pmindex}{\code{signature(object = "AffyBatch")}: return the
      location of perfect matches in the intensity matrix.}
    \item{mmindex}{\code{signature(object = "AffyBatch")}: return the
      location of the mismatch intensities.}
    \item{dim}{\code{signature(x = "AffyBatch")}: Row and column dimensions.}
    \item{ncol}{\code{signature(x = "AffyBatch")}: An accessor function
      for \code{ncol}.}
    \item{nrow}{\code{signature(x = "AffyBatch")}: an accessor function
      for \code{nrow}.}
    \item{normalize}{\code{signature(object = "AffyBatch")}: a method to
      \code{\link{normalize}}. The method accepts an argument
      \code{method}. The default methods is specified in package options
      (see the main vignette).}
    \item{normalize.methods}{\code{signature(object = "AffyBatch")}:
      returns the normalization methods defined for this class. See
      \code{\link{normalize}}.}
    \item{probeNames}{\code{signature(object = "AffyBatch")}: returns
      the probe set associated with each row of the intensity matrix. }
    \item{probeset}{\code{signature(object = "AffyBatch",genenames=NULL,
	  locations=NULL)}: Extracts \code{\link[affy:ProbeSet-class]{ProbeSet}}
	  objects related to the probe sets given in genenames. If an alternative
	  set of locations defining pms and mms a list with those locations should
      be passed via the \code{locations} argument.}
    \item{bg.correct}{\code{signature(object = "AffyBatch",
	  method="character")} applies background correction methods defined by
	  method.}
    \item{updateObject}{\code{signature(object = "AffyBatch", ...,
	  verbose=FALSE)}: update, if necessary, an object of class
      AffyBatch to its current class definition. \code{verbose=TRUE}
      provides details about the conversion process.}
  }
}
\note{This class is better described in the vignette.}

\seealso{related methods \code{\link[affy]{merge.AffyBatch}},
  \code{\link[affy]{pairs.AffyBatch}}, and
  \code{\link[Biobase:class.eSet]{eSet}}}

\examples{
if (require(affydata)) {
  ## load example
  data(Dilution)

  ## nice print
  print(Dilution)

  pm(Dilution)[1:5,]
  mm(Dilution)[1:5,]

  ## get indexes for the PM probes for the affyID "1900_at" 
  mypmindex <- pmindex(Dilution,"1900_at")
  ## same operation using the primitive
  mypmindex <- indexProbes(Dilution, which="pm", genenames="1900_at")[[1]]
  ## get the probe intensities from the index
  intensity(Dilution)[mypmindex, ]

  description(Dilution) ##we can also use the methods of eSet
  sampleNames(Dilution)
  abstract(Dilution)
}
}
\keyword{classes}

\name{expressoWidget}
\alias{expressoWidget}
\title{A widget for users to pick correction methods}
\description{
  This widget is called by expresso to allow users to select
  correction methods that will be used to process affy data.
}
\usage{
expressoWidget(BGMethods, normMethods, PMMethods, expMethods, BGDefault,
normDefault, PMDefault, expDefault) 
}
\arguments{
  \item{BGMethods}{a vector of character strings for the available methods
    that can be used as a background correction method of affy data.}
  \item{normMethods}{a vector of character strings for the available methods
    that can be used as a normalization method of affy data.}
  \item{PMMethods}{a vector of character strings for the available methods
    that can be used as a PM correction method of affy data.}
  \item{expMethods}{a vector of character strings for the available methods
    that can be used as a summary method of affy data.}
  \item{BGDefault}{a character string for the name of a default background
    correction method.}
  \item{normDefault}{a character string for the name of a default
    normalization method.}
  \item{PMDefault}{a character string for the name of a default PM correction
    method.}
  \item{expDefault}{a character string for the name of a default summary
    method.}
}
\details{
  The widget will be invoked when expresso is called with argument
  "widget" set to TRUE. Default values can be changed using the drop
  down list boxes. Double clicking on an option from the drop-down list
  makes an selection. The first element of the list for available methods
  will be the default method if no default is provided.
}
\value{
  The widget returns a list of selected correction methods.
  \item{BG}{background correction method}
  \item{NORM}{normalization method}
  \item{PM}{PM correction method}
  \item{EXP}{summary method}
}
\references{Documentations of affy package}
\author{Jianhua Zhang}

\seealso{\code{\link{expresso}}}
\examples{
if(interactive()){
  require(widgetTools)
  expressoWidget(c("mas", "none", "rma"), c("constant", "quantiles"),
c("mas", "pmonly"), c("liwong", "playerout"))
}
}
\keyword{interface}

\name{list.celfiles}
\alias{list.celfiles}
\title{List the Cel Files in a Directory/Folder}
\description{
This function produces a vector containing the names of files in the
named directory/folder ending in .cel or .CEL. 
}
\usage{
list.celfiles(...)
}
\arguments{
  \item{\dots}{arguments to  pass along to
    \code{\link[base]{list.files}}}
}
\value{
A character vector of file names.
}
\seealso{list.files}
\examples{
list.celfiles()
}
\keyword{character}

\name{normalize.contrasts}
\alias{normalize.contrasts}
\alias{normalize.AffyBatch.contrasts}
\title{Normalize intensities using the contrasts method}
\description{
  Scale chip objects in an \code{\link{AffyBatch-class}}.
}
\usage{
%normalize.contrast() ## currently maffy.normalize
normalize.AffyBatch.contrasts(abatch,span=2/3, choose.subset=TRUE,
                              subset.size=5000, verbose=TRUE,
                              family="symmetric",
                              type=c("together","pmonly","mmonly","separate"))
}
\arguments{
  \item{abatch}{an \code{\link{AffyBatch-class}} object.}
  \item{span}{parameter to be passed to the function
    \code{\link[stats]{loess}}.} 
  \item{choose.subset}{Boolean. Defaults to \code{TRUE}}
  \item{subset.size}{Integer. Number of probesets to use in each subset.}
  \item{verbose}{verbosity flag.}
  \item{family}{parameter to be passed to the function
    \code{\link[stats]{loess}}.} 
  \item{type}{a string specifying how the normalization should be applied.}
}
\value{
  An object of the same class as the one passed.
}
\seealso{
  \code{\link{maffy.normalize}}
}
\keyword{manip}

\name{SpikeIn}
\alias{SpikeIn}
\alias{concentrations}
\title{SpikeIn Experiment Data: ProbeSet Example}
\description{
This \code{\link[affy:ProbeSet-class]{ProbeSet}} represents part of SpikeIn
experiment data set. 
}
\usage{data(SpikeIn)}
\format{\code{SpikeIn} is \code{\link[affy:ProbeSet-class]{ProbeSet}} containing the
  $PM$ and $MM$ intensities for a gene spiked in at different
  concentrations (given in the vector \code{colnames(pm(SpikeIn))}) in 12
  different arrays.}
\source{This comes from an experiments where 11 different cRNA
fragments have been added to the hybridization mixture of the GeneChip
arrays at different pM concentrations.  The 11 control cRNAs were BioB-5,
BioB-M, BioB-3, BioC-5, BioC-3, BioDn-5 (all \emph{E. coli}), CreX-5, CreX-3
(phage P1), and  DapX-5, DapX-M, DapX-3 (\emph{B. subtilis})
The cRNA were chosen to match the target sequence for each of the
Affymetrix control probe sets. For example, for DapX (a \emph{B. subtilis}
gene), the 5', middle and 3' target sequences (identified by DapX-5,
DapX-M, DapX-3) were each synthesized separately and spiked-in at a
specific concentration. Thus, for example, DapX-3 target sequence may be
added to the total hybridization solution of 200 micro-liters to give a final
concentration of 0.5 pM.

For this example we have the $PM$ and $MM$ for BioB-5
obtained from the arrays where it was spiked in at
0.0, 0.5, 0.75, 1, 1.5, 2, 3, 5, 12.5, 25, 50, and 150
pM.

For more information see Irizarry, R.A., et al. (2001)
\url{http://biosun01.biostat.jhsph.edu/~ririzarr/papers/index.html}}
\keyword{datasets}

\name{bg.adjust}
\alias{bg.adjust}
\alias{bg.parameters}
\title{Background adjustment (internal function)}
\description{
An internal function to be used by \code{\link{bg.correct.rma}}.
}
\usage{
bg.adjust(pm, n.pts = 2^14, ...)
bg.parameters(pm, n.pts = 2^14)
}
\arguments{
  \item{pm}{a pm matrix}
  \item{n.pts}{number of points to use in call to \code{density}.}
  \item{\dots}{extra arguments to pass to bg.adjust.}
}
\details{Assumes PMs are a convolution of normal and exponential. So we
  observe X+Y where X is background and Y is signal. \code{bg.adjust}
  returns E[Y|X+Y, Y>0] as our background corrected PM.
  \code{bg.parameters} provides ad hoc estimates of the parameters of the
  normal and exponential distributions.}
\value{a matrix}
\seealso{\code{\link{bg.correct.rma}}}
\keyword{manip}

%%%THIS FUNCTION SHOULD BE In BIOBASE or SOMEWHERE ELSE... NEXT RELEASE
%% LG: In such a case, I assume the long awaited agreement about
%% normalization methods will be here and all the normalization methods
%% will be stored together
\name{affy.scalevalue.exprSet}
\alias{affy.scalevalue.exprSet}
\title{Scale normalization for expreSets}
\description{
  Normalizes expression values using the method described in the Affymetrix
  user manual.
}
\usage{
affy.scalevalue.exprSet(eset, sc = 500, analysis="absolute")
}
\arguments{
  \item{eset}{An \code{\link[Biobase:class.ExpressionSet]{ExpressionSet}}
    object.}
  \item{sc}{Value at which all arrays will be scaled to.}
  \item{analysis}{Should we do absolute or comparison analysis, although
    "comparison" is still not implemented.}
}
\details{
  This is function was implemented from the Affymetrix technical
  documentation for MAS 5.0. It can be downloaded from the website of
  the company. Please refer to this document for details.
}
\author{Laurent}
\value{
  A normalized \code{\link[Biobase:class.ExpressionSet]{ExpressionSet}}.
}
\keyword{manip}

\name{ppsetApply}
\alias{ppsetApply}
\alias{ppset.ttest}
\title{ Apply a function over the ProbeSets in an AffyBatch }
\description{
  Apply a function over the ProbeSets in an AffyBatch
}
\usage{
ppsetApply(abatch, FUN, genenames = NULL, ...)

ppset.ttest(ppset, covariate, pmcorrect.fun = pmcorrect.pmonly, ...)
}
\arguments{
  \item{abatch}{an object inheriting from \code{AffyBatch}.}
  \item{ppset}{an object of class \code{ProbeSet}.}
  \item{covariate}{the name a covariate in the slot \code{phenoData}.}
  \item{pmcorrect.fun}{a function to correct PM intensities.}
  \item{FUN}{a function working on a \code{ProbeSet}.}
  \item{genenames}{a list of Affymetrix probesets ids to work with. All
  probe set ids used when \code{NULL}.}
  \item{\dots}{optional parameters to the function \code{FUN}.}
}
\value{
  Returns a \code{list} of objects, or values, as returned by the
  function \code{FUN} for each \code{ProbeSet} it processes.
}
\author{Laurent Gautier <laurent@cbs.dtu.dk>}
\seealso{\code{\link[affy]{ProbeSet-class}} }
\examples{
ppset.ttest <- function(ppset, covariate, pmcorrect.fun = pmcorrect.pmonly, ...) {
  probes <- do.call("pmcorrect.fun", list(ppset))
  my.ttest <- function(x) {
    y <- split(x, get(covariate))
    t.test(y[[1]], y[[2]])$p.value
  }
  r <- apply(probes, 1, my.ttest)
  return(r)
}
##this takes a long time - and rowttests is a good alternative
## eg: rt = rowttests(exprs(Dilution), Dilution$liver)
\dontrun{
  data(Dilution)
  all.ttest <- ppsetApply(Dilution, ppset.ttest, covariate="liver")
}
}
\keyword{ manip }

\name{AffyRNAdeg}
\alias{AffyRNAdeg}
\alias{summaryAffyRNAdeg}
\alias{plotAffyRNAdeg}
\title{Function to assess RNA degradation in Affymetrix GeneChip data.}
\description{
  Uses ordered probes in probeset to detect possible RNA degradation.
  Plots and statistics used for evaluation.
}
\usage{
AffyRNAdeg(abatch,log.it=TRUE)
summaryAffyRNAdeg(rna.deg.obj,signif.digits=3)
plotAffyRNAdeg(rna.deg.obj, transform = "shift.scale", cols = NULL, ...)
}
\arguments{
  \item{abatch}{An object of class \code{\link{AffyBatch-class}}.}
  \item{log.it}{A logical argument: If log.it=T, then probe data is log2
    transformed.}
  \item{rna.deg.obj}{Output from AffyRNAdeg.}
  \item{signif.digits}{Number of significant digits to show.}
  \item{transform}{Possible choices are "shift.scale","shift.only", and
    "neither". "Shift" vertically staggers the plots for individual chips,
    to make the display easier to read. "Scale" normalizes so that standard
    deviation is equal to 1.}
  \item{cols}{A vector of colors for plot, length = number of chips.}
  \item{\dots}{further arguments for \code{\link{plot}} function.}
}
\details{Within each probeset, probes are numbered directionally from
  the 5' end to the 3' end. Probe intensities are averaged by probe
  number, across all genes. If log.it=\code{FALSE} and transform="Neither",
  then  plotAffyRNAdeg simply shows these means for each chip. Shifted and
  scaled versions of the plot can make it easier to see.
}
\value{
  \code{AffyRNAdeg} returns a list with the following components:
  \item{sample.names }{names of samples, derived from affy batch object}
  \item{means.by.number}{average intensity by probe position}
  \item{ses}{standard errors for probe position averages}
  \item{slope}{from linear regression of means.by.number}
  \item{pvalue}{from linear regression of means.by.number}
}
\examples{
if (require(affydata)) {
  data(Dilution)
  RNAdeg<-AffyRNAdeg(Dilution)
  plotAffyRNAdeg(RNAdeg)
}
}
\author{Leslie Cope}
\keyword{hplot}
\keyword{manip}

\name{bg.correct}
\alias{bg.correct}
\alias{bg.correct.none} %took out .pmonly casue rma is pm-only
%\alias{bg.correct.subtractmm}
\alias{bg.correct.rma}
\alias{bg.correct.mas}
\title{Background Correction}
\description{
Background corrects probe intensities in an object of class
\code{\link[affy:AffyBatch-class]{AffyBatch}}.
}
\usage{
bg.correct(object, method, ...)

bg.correct.rma(object,...)
bg.correct.mas(object, griddim)
bg.correct.none(object, ...)
}
\arguments{
  \item{object}{An object of class \code{\link[affy:AffyBatch-class]{AffyBatch}}.}
  \item{method}{A \code{character} that defines what background
    correction method will be used. Available methods are given by
    \code{bg.correct.methods}.}
  \item{griddim}{grid dimension used for mas background estimate. The array
    is divided into griddim equal parts. Default is 16.}
  \item{\dots}{arguments to pass along to the engine function.}
}
\details{
  The name of the method to apply must be double-quoted.
  Methods provided with the package are currently:
  \itemize{
    \item bg.correct.none: returns \code{object} unchanged.
    \item bg.correct.chipwide: noise correction as described in a
    `white paper' from Affymetrix.
    \item bg.correct.rma: the model based correction used by the RMA
    expression measure.
  }

  They are listed in the variable \code{bg.correct.methods}. The user must
  supply the word after "bg.correct", i.e none, subtractmm, rma, etc...

  More details are available in the vignette.

  R implementations similar in function to the internal implementation used by
  \code{bg.correct.rma} are in \code{\link{bg.adjust}}. 
}  
\value{
  An \code{\link[affy:AffyBatch-class]{AffyBatch}} for which the
  intensities have been background adjusted.
  For some methods (RMA), only PMs are corrected and the MMs remain the
  same. 
}
\examples{
if (require(affydata)) {
  data(Dilution)

  ##bgc will be the bg corrected version of Dilution
  bgc <- bg.correct(Dilution, method="rma") 

  ##This plot shows the tranformation
  plot(pm(Dilution)[,1],pm(bgc)[,1],log="xy",
  main="PMs before and after background correction")
}
}
\keyword{manip}

\name{whatcdf}
\alias{whatcdf}
\title{Find which CDF corresponds}
\description{
  Find which kind of CDF corresponds to a CEL file.
}
\usage{
whatcdf(filename, compress = getOption("BioC")$affy$compress.cel)
}
\arguments{
  \item{filename}{a '.CEL' file name.}
  \item{compress}{logical (file compressed or not).}
}
\details{
  Information concerning the corresponding CDF file seems to be found in
  CEL files. This allows us to try to link CDF information automatically.
}
\value{
  a \code{character} with the name of the CDF.
}
\seealso{\code{getInfoInAffyFile}, \code{read.celfile}}
\keyword{manip}

\name{rma}
\alias{rma}
\title{Robust Multi-Array Average expression measure}
\description{
  This function converts an \code{\link[affy:AffyBatch-class]{AffyBatch}}
  object into an \code{\link[Biobase:class.ExpressionSet]{ExpressionSet}}
  object using the robust multi-array average (RMA) expression measure.
}
\usage{
rma(object, subset=NULL, verbose=TRUE, destructive=TRUE, normalize=TRUE,
    background=TRUE, bgversion=2, ...)
}
\arguments{
  \item{object}{an \code{\link[affy:AffyBatch-class]{AffyBatch}} object.}
  \item{subset}{a character vector with the the names of the probesets
    to be used in expression calculation.}
  \item{verbose}{logical value. If \code{TRUE}, it writes out some messages
    indicating progress. If \code{FALSE} nothing should be printed.}
  \item{destructive}{logical value. If \code{TRUE}, works on the PM matrix
    in place as much as possible, good for large datasets.}
  \item{normalize}{logical value. If \code{TRUE}, normalize data using
    quantile normalization.}
  \item{background}{logical value. If \code{TRUE}, background correct
    using RMA background correction.}
  \item{bgversion}{integer value indicating which RMA background to use
1: use background similar to pure R rma background given in affy version 1.0 - 1.0.2
2: use background similar to pure R rma background given in affy version 1.1 and above}
  \item{\dots}{further arguments to be passed (not currently implemented - stub for future use).}
}
\details{
This function computes the RMA (Robust Multichip Average) expression measure
described in Irizarry et al Biostatistics (2003).

Note that this expression measure is given to you in log base 2
scale. This differs from most of the other expression measure methods.

Please note that the default background adjustment method was changed during
the lead up to the Bioconductor 1.2 release. This means that this function and
\code{\link{expresso}} should give results that directly agree.
}
\value{
An \code{\link[Biobase:class.ExpressionSet]{ExpressionSet}}
}
\author{Ben Bolstad \email{bmb@bmbolstad.com}}
\references{
  Rafael. A. Irizarry, Benjamin M. Bolstad, Francois Collin, Leslie M. Cope, Bridget Hobbs and Terence P. Speed (2003), Summaries of Affymetrix GeneChip probe level data Nucleic Acids Research 31(4):e15

  Bolstad, B.M., Irizarry R. A., Astrand M., and Speed, T.P. (2003), A Comparison of Normalization Methods for High Density Oligonucleotide Array Data Based on Bias and Variance. Bioinformatics 19(2):185-193

  Irizarry, RA, Hobbs, B, Collin, F, Beazer-Barclay, YD, Antonellis, KJ, Scherf, U, Speed, TP (2003) Exploration, Normalization, and Summaries of High Density Oligonucleotide Array Probe Level Data. Biostatistics .Vol. 4, Number 2: 249-264 
}
\seealso{\code{\link{expresso}}}
\examples{
if (require(affydata)) {
  data(Dilution)
  eset <- rma(Dilution)
}
}
\keyword{manip}

\name{affy-options}
\alias{affy-options}
\title{Options for the affy package}
\description{
  Description of the options for the affy package.
}
\note{
  The affy package options are contained in the Bioconductor options.
  The options are:
  \itemize{
    \item \code{use.widgets}: a logical used to decide on the default of
    widget use.
    \item \code{compress.cel}: a logical
    \item \code{compress.cdf}: a logical
    \item \code{probes.loc}: a list. Each element of the list is it self a
    list with two elements \emph{what} and \emph{where}. When looking for
    the informations about the locations of the probes on the array, the
    elements in the list will be looked at one after the other. The first
    one for which \emph{what} and \emph{where} lead to the matching
    locations information is used. The element \emph{what} can be one of
    \emph{package}, \emph{environment} or \emph{file}. The element \emph{where}
    depends on the corresponding element \emph{what}.
    \itemize{
      \item if \emph{package}: location for the package (like it would be for
      the argument \code{lib.loc} for the function \code{library}.)
      \item if \emph{environment}: an \code{environment} to look for the
      information (like the argument \code{env} for the function \code{get}).
      \item if \emph{file}: a \code{character} with the path in which a CDF
      file can be found.
    }
  }
}
\examples{
## get the options
opt <- getOption("BioC")
affy.opt <- opt$affy

## list their names
names(affy.opt)

## set the option compress.cel
affy.opt$compress.cel <- TRUE
options(BioC=opt)
}
\keyword{manip}

\name{justRMA}
\alias{justRMA}
\alias{just.rma}
\title{Read CEL files into an ExpressionSet}
\description{
  Read CEL files and compute an expression measure without using an AffyBatch.}
\usage{
just.rma(\dots, filenames = character(0),
               phenoData = new("AnnotatedDataFrame"),
               description = NULL,
               notes = "",
               compress = getOption("BioC")$affy$compress.cel,
               rm.mask = FALSE, rm.outliers = FALSE, rm.extra = FALSE,
               verbose=FALSE, background=TRUE, normalize=TRUE,
               bgversion=2, destructive=FALSE, cdfname = NULL)

justRMA(\dots, filenames=character(0),
              widget=getOption("BioC")$affy$use.widgets,
              compress=getOption("BioC")$affy$compress.cel,
              celfile.path=getwd(),
              sampleNames=NULL,
              phenoData=NULL,
              description=NULL,
              notes="",
              rm.mask=FALSE, rm.outliers=FALSE, rm.extra=FALSE,
              hdf5=FALSE, hdf5FilePath=NULL,verbose=FALSE,
              normalize=TRUE, background=TRUE, 
              bgversion=2, destructive=FALSE, cdfname = NULL)
}
\arguments{
  \item{\dots}{file names separated by comma.}
  \item{filenames}{file names in a character vector.}
  \item{phenoData}{an
              \code{\link[Biobase:class.AnnotatedDataFrame]{AnnotatedDataFrame}}
              object.} 
  \item{description}{a \code{\link[Biobase:class.MIAME]{MIAME}} object.}
  \item{notes}{notes.}
  \item{compress}{are the CEL files compressed?}
  \item{rm.mask}{should the spots marked as 'MASKS' set to \code{NA}?}
  \item{rm.outliers}{should the spots marked as 'OUTLIERS' set to \code{NA}?}
  \item{rm.extra}{if \code{TRUE}, then overrides what is in
              \code{rm.mask} and \code{rm.oultiers}.} 
  \item{hdf5}{use of hdf5 ? (not available yet)}
  \item{hdf5FilePath}{a filename to use with hdf5 (not available yet).}
  \item{verbose}{verbosity flag.}
  \item{widget}{a logical specifying if widgets should be used.}
  \item{celfile.path}{a character denoting the path \code{ReadAffy} should look
    for cel files.}
  \item{sampleNames}{a character vector of sample names to be used in the
    \code{AffyBatch}.}
    \item{normalize}{logical value. If \code{TRUE}, then normalize data using
    quantile normalization.}
  \item{background}{logical value. If \code{TRUE}, then background correct
    using RMA background correction.}
  \item{bgversion}{integer value indicating which RMA background to use
    1: use background similar to pure R rma background given in affy
    version 1.0 - 1.0.2
    
    2: use background similar to pure R rma background given in affy
    version 1.1 and above} 
   \item{destructive}{logical value. If \code{TRUE}, then works on the PM matrix
    in place as much as possible, good for large datasets.}
   \item{cdfname}{Used to specify the name of an alternative cdf package. If set
     to \code{NULL}, then the usual cdf package based on Affymetrix' mappings
     will be used.}
}
\details{
\code{justRMA} is a wrapper for \code{just.rma} that permits the user to read
in phenoData, MIAME information, and CEL files using widgets. One can also
define files where to read phenoData and MIAME information.

If the function is called with no arguments \code{justRMA()}, then all the CEL
files in the working directory are read, converted to an expression measure
using RMA and put into an
\code{\link[Biobase:class.ExpressionSet]{ExpressionSet}}. 
However, the arguments give the user great flexibility.
  
\code{phenoData} is read using \code{\link[Biobase]{read.AnnotatedDataFrame}}.
If a character is given, it tries to read the file with that name to obtain the
\code{AnnotatedDataFrame} object as described in \code{\link[Biobase]{read.AnnotatedDataFrame}}.
If left \code{NULL} and \code{widget=FALSE} (\code{widget=TRUE} is not currently
supported), then a default object is created.
It will be an object of class \code{\link[Biobase:class.AnnotatedDataFrame]{AnnotatedDataFrame}}
with its pData being a data.frame with column x indexing the CEL files.

\code{description} is read using \code{\link[Biobase]{read.MIAME}}. If a
character is given, it tries to read the file with that name to obtain a
\code{MIAME} instance. If left \code{NULL} but \code{widget=TRUE}, then
widgets are used. If left \code{NULL} and \code{widget=FALSE}, then an
empty instance of \code{MIAME} is created.

The arguments \code{rm.masks}, \code{rm.outliers}, \code{rm.extra} are
passed along to the function \code{read.celfile}.
}
\value{
  An \code{ExpressionSet} object, containing expression values identical to
  what one would get from running \code{rma} on an \code{AffyBatch}.
}
\author{In the beginning: James MacDonald <jmacdon@med.umich.edu>
Supporting routines, maintenance and just.rma: Ben Bolstad <bmb@bmbolstad.com>}
\seealso{\code{\link[affy]{rma}}, \code{\link{read.affybatch}}}
\keyword{manip}

\name{xy2indices}
\alias{xy2indices}
\alias{indices2xy}
\title{Functions to convert indices to x/y (and reverse)}
\description{
  Functions to convert indices to x/y (and reverse)
}
\usage{
xy2indices(x, y, nc = NULL, cel = NULL, abatch = NULL, cdf = NULL, xy.offset = NULL)
indices2xy(i, nc = NULL, cel = NULL, abatch = NULL, cdf = NULL, xy.offset = NULL)
}
\arguments{
  \item{x}{A numeric vector of \code{X} (column) position(s) for the probes.}
  \item{y}{A numeric vector of \code{Y} (row) position(s) for the probes.}
  \item{i}{A numeric vector of indices in the \code{AffyBatch} for the probes.}
  \item{nc}{total number of columns on the chip. It is usually better to
  specify either the cdf or abatch arguments rather than the number of columns.}
  \item{cel}{a corresponding object of class \code{Cel}. This has been
    deprecated. Use abatch or cdf instead.}
  \item{abatch}{a corresponding object of class
    \code{\link[affy:AffyBatch-class]{AffyBatch}}.}
  \item{cdf}{character - the name of the corresponding cdf package.}
  \item{xy.offset}{an eventual offset for the XY coordinates. See Details.}
}
\details{
  The Affymetrix scanner reads data from a GeneChip by row, and exports
  those data to a CEL file. When we read in the CEL file data to an
  \code{AffyBatch} object, we store data for each GeneChip as a single
  column in a matrix of probe-wise intensity values.

  The CDF files that Affymetrix make available for various GeneChips map
  individual probes to probesets based on their (x,y) coordinates on the
  GeneChip. Note that these coordinates are zero-based, and (x,y) is the
  same as (column, row). In other words, the x coordinate indicates the
  horizontal location of the probe, and the y coordinate indicates the
  vertical location of the probe. By convention, (0,0) is the coordinate
  location for the top left position, and (ncol-1, nrow-1) is the
  coordinate location of the lower right position.

  For most users, the mapping of probes to probeset is handled
  internally by various functions (\code{rma}, \code{espresso}, etc),
  and in general usage it is never necessary for a user to convert probe
  index position in an \code{AffyBatch} to the corresponding (x,y)
  coordinates on the GeneChip. These functions are only useful for those
  who wish to know more about the internal workings of the Affymetrix
  GeneChip.
  
  The parameter \code{xy.offset} is there for compatibility.
  For historical reasons, the xy-coordinates for the features
  on Affymetrix GeneChips were decided to start at 1 (one) rather than 0
  (zero). One can set the offset to 1 or to 0. Unless the you \_really\_
  know what you are doing, it is advisable to let it at the default
  value \code{NULL}. This way the package-wide option \code{xy.offset} is
  always used.
}
\value{
  A vector of indices or a two-columns matrix of Xs and Ys. 
}
\author{L.}
\section{Warning}{Even if one really knows what is going on, playing with
  the parameter \code{xy.offset} could be risky. Changing the package-wide
  option \code{xy.offset} appears much more sane.}
\seealso{\code{\link{indexProbes}}}
\examples{
if (require(affydata)) {
  data(Dilution)
  pm.i <- indexProbes(Dilution, which="pm", genenames="AFFX-BioC-5_at")[[1]]
  mm.i <- indexProbes(Dilution, which="mm", genenames="AFFX-BioC-5_at")[[1]]

  pm.i.xy <- indices2xy(pm.i, abatch = Dilution)
  mm.i.xy <- indices2xy(mm.i, abatch = Dilution)

  ## and back to indices
  i.pm <- xy2indices(pm.i.xy[,1], pm.i.xy[,2], cdf = "hgu95av2cdf")
  i.mm <- xy2indices(mm.i.xy[,1], mm.i.xy[,2], cdf = "hgu95av2cdf")

  identical(pm.i, as.integer(i.pm))
  identical(mm.i, as.integer(i.mm))

  image(Dilution[1], transfo=log2)
  ## plot the pm in red
  plotLocation(pm.i.xy, col="red")
  plotLocation(mm.i.xy, col="blue")
}
}
\keyword{manip}

\name{pairs.AffyBatch}
\alias{pairs.AffyBatch}
\title{plot intensities using 'pairs'}
\description{
  Plot intensities using the function 'pairs'
}
\usage{
\method{pairs}{AffyBatch}(x, panel=points, ..., transfo=I, main=NULL, oma=NULL,
                  font.main = par("font.main"),
                  cex.main = par("cex.main"), cex.labels = NULL, 
                  lower.panel=panel, upper.panel=NULL, diag.panel=NULL,
                  font.labels = 1, row1attop = TRUE, gap = 1)
}
\arguments{
  \item{x}{an \code{\link[affy:AffyBatch-class]{AffyBatch}} object.}
  \item{panel}{a function to produce a plot (see \code{\link{pairs}}).}
  \item{\dots}{extra parameters for the 'panel' function.}
  \item{transfo}{a function to transform the intensity values before
    generating the plot. 'log' and 'log2' are popular choices.}
  \item{main}{title for the plot}
  \item{oma}{see 'oma' in \code{\link{par}}.}
  \item{font.main}{see \code{\link{pairs}}.}
  \item{cex.main}{see \code{\link{pairs}}.}
  \item{cex.labels}{see \code{\link{pairs}}.}
  \item{lower.panel}{a function to produce the plots in the lower
    triangle (see \code{\link{pairs}}).}
  \item{upper.panel}{a function to produce the plots in the upper
    triangle (see \code{\link{pairs}}).}
  \item{diag.panel}{a function to produce the plots in the diagonal
    (see \code{\link{pairs}}).}
  \item{font.labels}{see \code{\link{pairs}}.}
  \item{row1attop}{see \code{\link{pairs}}.}
  \item{gap}{see \code{\link{pairs}}.}
}
\details{
  Plots with several chips can represent zillions of points. They require
  a lot of memory and can be very slow to be displayed. You may want to
  try to split of the plots, or to plot them in a device like 'png' or
  'jpeg'. 
}
\keyword{hplot}

\name{ProbeSet-class}
\docType{class}
\alias{ProbeSet-class}
\alias{mm,ProbeSet-method}
\alias{mm<-,ProbeSet,matrix-method}
\alias{pm<-,ProbeSet,matrix-method}
\alias{pm,ProbeSet-method}
\alias{show,ProbeSet-method}
\alias{barplot,ProbeSet-method}
\alias{colnames,ProbeSet-method}
\alias{express.summary.stat,ProbeSet,character,character-method}
\alias{sampleNames,ProbeSet-method}
\title{Class ProbeSet}
\description{A simple class that contains the PM and MM data for a probe
  set from one or more samples.}
\section{Objects from the Class}{
Objects can be created by applying the method \code{\link{probeset}} to
instances of AffyBatch.}
\section{Slots}{
  \describe{
    \item{\code{id}:}{Object of class \code{"character"} containing the
      probeset ID.}
    \item{\code{pm}:}{Object of class \code{"matrix"} containing the PM
      intensities. Columns represent samples and rows the different probes.}
    \item{\code{mm}:}{Object of class \code{"matrix"} containing the MM
      intensities.}
  }
}
\section{Methods}{
  \describe{
    \item{colnames}{\code{signature(x = "ProbeSet")}: the column names
      of the \code{pm} matrices which  are the sample names}
    \item{express.summary.stat}{\code{signature(x = "ProbeSet",
	  pmcorrect = "character", summary = "character")}: applies a summary
	  statistic to the probe set.}
    \item{sampleNames}{\code{signature(object = "ProbeSet")}: the column names
      of the \code{pm} matrices which  are the sample names.}
  }
}
\note{More details are contained in the vignette.}
\seealso{\code{\link[affy:AffyBatch-class]{probeset}},
  \code{\link[affy]{AffyBatch-class}}} 
\examples{
if (require(affydata)) {
  data(Dilution)
  ps <- probeset(Dilution, geneNames(Dilution)[1:2])
  names(ps)
  print(ps[[1]])
}
}
\keyword{classes}

\name{cdfFromBioC}
\alias{cdfFromBioC}
\alias{cdfFromLibPath}
\alias{cdfFromEnvironment}
\title{Functions to obtain CDF files}
\description{
  A set of functions to obtain CDF files from various locations.
}
\usage{
cdfFromBioC(cdfname, lib = .libPaths()[1], verbose = TRUE)
cdfFromLibPath(cdfname, lib = NULL, verbose=TRUE)
cdfFromEnvironment(cdfname, where, verbose=TRUE)
}
\arguments{
  \item{cdfname}{name of the CDF.}
  \item{lib}{install directory for the CDF package.}
  \item{where}{environment to search.}
  \item{verbose}{logical controlling extra output.}  
}
\details{
  These functions all take a requested CDF environment name and will
  attempt to locate that environment in the appropriate location (a
  package's data directory, as a CDF package in the .libPaths(), from a
  loaded environment or on the Bioconductor website.  If the environment
  can not be found, it will return a list of the methods tried that failed.
}
\value{
  The CDF environment or a list detailing the failed locations.
}
\author{Jeff Gentry}
\keyword{utilities}

\name{mas5calls}
\alias{mas5calls,AffyBatch-method}
\alias{mas5calls,ProbeSet-method}
\alias{mas5calls}
\alias{mas5.detection}
\alias{mas5calls.AffyBatch}
\alias{mas5calls.ProbeSet}
\title{MAS 5.0 Absolute Detection}
\description{
 Performs the Wilcoxon signed rank-based gene expression presence/absence 
 detection algorithm first implemented in the Affymetrix Microarray Suite
 version 5.
}
\usage{
mas5calls(object,...)

mas5calls.AffyBatch(object, ids = NULL, verbose = TRUE, tau = 0.015,
                    alpha1 = 0.04, alpha2 = 0.06,
                    ignore.saturated=TRUE) 

mas5calls.ProbeSet(object, tau = 0.015, alpha1 = 0.04, alpha2 = 0.06,
                   ignore.saturated=TRUE) 

mas5.detection(mat, tau = 0.015, alpha1 = 0.04, alpha2 = 0.06,
               exact.pvals = FALSE, cont.correct = FALSE)
}
\arguments{
  \item{object}{an object of class \code{AffyBatch} or \code{ProbeSet}.}
  \item{ids}{probeset IDs for which you want to compute calls.}
  \item{mat}{an n-by-2 matrix of paired values (pairs in rows), PMs first col.}
  \item{verbose}{logical. It \code{TRUE}, status of processing is reported.}
  \item{tau}{a small positive constant.}
  \item{alpha1}{a significance threshold in (0, alpha2).}
  \item{alpha2}{a significance threshold in (alpha1, 0.5).}
  \item{exact.pvals}{logical controlling whether exact p-values are computed
    (irrelevant if n<50 and there are no ties).  Otherwise the normal
    approximation is used.}
  \item{ignore.saturated}{if TRUE, do the saturation correction described in
    the paper, with a saturation level of 46000.}
  \item{cont.correct}{logical controlling whether continuity correction is
    used in the p-value normal approximation.}
  \item{\dots}{any of the above arguments that applies.}
}
\details{
 This function performs the hypothesis test:

  H0: median(Ri) = tau, corresponding to absence of transcript
  H1: median(Ri) > tau, corresponding to presence of transcript

 where Ri = (PMi - MMi) / (PMi + MMi) for each i a probe-pair in the 
 probe-set represented by data.

 Currently exact.pvals=TRUE is not supported, and cont.correct=TRUE works but
 does not give great results (so both should be left as FALSE).  The defaults
 for tau, alpha1 and alpha2 correspond to those in MAS5.0.

 The p-value that is returned estimates the usual quantity:

   Pr(observing a more "present looking" probe-set than data | data is absent)

 So that small p-values imply presence while large ones imply absence of 
 transcript.  The detection call is computed by thresholding the p-value as 
 in:

 call "P" if p-value < alpha1
 call "M" if alpha1 <= p-value < alpha2
 call "A" if alpha2 <= p-value

 This implementation has been validated against the original MAS5.0
 implementation with the following results (for exact.pvals and cont.correct
 set to F):

   Average Relative Change from MAS5.0 p-values:38\%
   Proportion of calls different to MAS5.0 calls:1.0\%

 where "average/proportion" means over all probe-sets and arrays, where 
 the data came from 11 bacterial control probe-sets spiked-in over a range
 of concentrations (from 0 to 150 pico-mols) over 26 arrays.  These are
 the spike-in data from the GeneLogic Concentration Series Spikein Dataset.

 Clearly the p-values computed here differ from those computed by MAS5.0 --
 this will be improved in subsequent releases of the affy package.   However
 the p-value discrepancies are small enough to result in the call being very
 closely aligned with those of MAS5.0 (99 percent
 were identical on the validation
 set) -- so this implementation will still be of use.
 
 The function \code{mas5.detect} is no longer the engine function for the
 others. C code is no available that computes the Wilcox test faster. The
 function is kept so that people can look at the R code (instead of C).
}

\value{
  \code{mas5.detect} returns a list containing the following components:
  \item{pval}{	a real p-value in [0,1] equal to the probability of
    observing probe-level intensities that are more present looking than
    data assuming the data represents an absent transcript; that is a
    transcript is more likely to be present for p-values closer 0.} 
  \item{call}{either "P", "M" or "A" representing a call of present,
    marginal or absent; computed by simply thresholding pval using
    alpha1 and alpha2.}

  The \code{mas5calls} method for \code{AffyBatch} returns an
  \code{ExpressionSet} with calls accessible with \code{exprs(obj)}
  and p-values available with \code{assayData(obj)[["se.exprs"]]}. The
  code \code{mas5calls} for \code{ProbeSet} returns a list with vectors
  of calls and p-values.
}
\references{
Liu, W. M. and Mei, R. and Di, X. and Ryder, T. B. and Hubbell, E. and Dee,
S. and Webster, T. A. and Harrington, C. A. and Ho, M. H. and Baid, J. and
Smeekens, S. P. (2002) Analysis of high density expression microarrays with
signed-rank call algorithms, Bioinformatics, 18(12), pp. 1593--1599.

Liu, W. and Mei, R. and Bartell, D. M. and Di, X. and Webster, T. A. and
Ryder, T. (2001) Rank-based algorithms for analysis of microarrays,
Proceedings of SPIE, Microarrays: Optical Technologies and Informatics, 4266.

Affymetrix (2002) Statistical Algorithms Description Document, Affymetrix
Inc., Santa Clara, CA,
whitepaper.
\url{http://www.affymetrix.com/support/technical/whitepapers/sadd_whitepaper.pdf},
\url{http://www.affymetrix.com/support/technical/whitepapers/sadd_whitepaper.pdf}
}
\author{Crispin Miller, Benjamin I. P. Rubinstein, Rafael A. Irizarry}
\examples{
if (require(affydata)) {
  data(Dilution)
  PACalls <- mas5calls(Dilution)
}
}
\keyword{manip}

\name{affy-deprecated}
\alias{loess.normalize}
\alias{maffy.normalize}
\alias{multiloess}
\alias{simplemultiLoess}
\title{Deprecated functions in package \sQuote{affy}}
\description{
  These functions are provided for compatibility with older versions
  of affy only, and will be defunct at the next release.
}
\details{

  The following functions are deprecated and will be made defunct; use
  the replacement indicated below:
  \itemize{

    \item{loess.normalize: \code{\link{normalize.loess}}}

    \item{maffy.normalize}
    \item{multiloess}
    \item{simplemultiLoess}

  }
}

\name{plot.ProbeSet}
\alias{plot.ProbeSet}
\title{plot a probe set}
\description{
  Plot intensities by probe set.
}
\usage{
\method{plot}{ProbeSet}(x, which=c("pm", "mm"), xlab = "probes", type = "l", ylim = NULL, ...)
}
\arguments{
  \item{x}{a \code{ProbeSet} object.}
  \item{which}{get the PM or the MM.}
  \item{xlab}{x-axis label.}
  \item{type}{plot type.}
  \item{ylim}{range of the y-axis.}
  \item{\dots}{optional arguments to be passed to \code{matplot}.}
}
\value{
  This function is only used for its (graphical) side-effect.
}
\seealso{\code{\link[affy:ProbeSet-class]{ProbeSet}}}
\examples{
data(SpikeIn)
plot(SpikeIn)
}
\keyword{hplot}
